<?php

/**
 * @package     Extly.Components
 * @subpackage  PlgSystemAutotweetEasyblog - Plugin AutoTweet NG EasyBlog-Extension
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2012 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

// Check for component
if (!JComponentHelper::getComponent('com_autotweet', true)->enabled)
{
	JError::raiseWarning('5', 'AutoTweet NG EasyBlog-Plugin - AutoTweet NG Component is not installed or not enabled.');
	return;
}

include_once JPATH_ROOT . '/administrator/components/com_autotweet/helpers/autotweetbase.php';

// EasyBlog extension plugin for AutoTweet.

/**
 * PlgSystemAutotweetEasyblog
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
 */
class PlgSystemAutotweetEasyblog extends plgAutotweetBase
{
	// Typeinfo

	// Not used at the moment
	const TYPE_ARTICLE = 1;

	// Const TYPE_COMMENT	= 2;

	protected $categories = '';

	protected $excluded_categories = '';

	protected $post_modified = 0;

	protected $show_category = 0;

	protected $show_hash = 0;

	protected $use_text = 0;

	protected $use_text_count = 100;

	protected $static_text = '';

	protected $static_text_pos = 1;

	protected $static_text_source = 0;

	// Used for tags
	protected $metakey_count = 1;

	protected $easyBlogVersion = null;

	/**
	 * PlgSystemAutotweetEasyblog
	 *
	 * @param   string  &$subject  Param
	 * @param   object  $params    Param
	 *
	 * @since	1.5
	 */
	public function PlgSystemAutotweetEasyblog(&$subject, $params)
	{
		parent::__construct($subject, $params);

		$pluginParams = $this->pluginParams;

		// Plugin specific params
		$this->categories = $pluginParams->get('categories', '');
		$this->excluded_categories = $pluginParams->get('excluded_categories', '');
		$this->post_modified = (int) $pluginParams->get('post_modified', 0);
		$this->show_category = (int) $pluginParams->get('show_category', 0);
		$this->show_hash = (int) $pluginParams->get('show_hash', 0);
		$this->use_text = (int) $pluginParams->get('use_text', 0);
		$this->use_text_count = (int) $pluginParams->get('use_text_count', 100);
		$this->static_text = $pluginParams->get('static_text', '');
		$this->static_text_pos = (int) $pluginParams->get('static_text_pos', 1);
		$this->static_text_source = (int) $pluginParams->get('static_text_source', 0);

		// Used for tags
		$this->metakey_count = (int) $pluginParams->get('metakey_count', 1);

		// Check type and range, and correct if needed
		$this->use_text_count = $this->getTextcount($this->use_text_count);

	}

	/**
	 * _initEasyBlog
	 *
	 * @return	boolean
	 *
	 * @since	1.5
	 */
	protected function _initEasyBlog()
	{
		// EasyBlog
		if (!class_exists('EasyBlogHelper'))
		{
			JLoader::register('EasyBlogHelper', JPATH_ROOT .
				DIRECTORY_SEPARATOR . 'components' .
				DIRECTORY_SEPARATOR . 'com_easyblog' .
				DIRECTORY_SEPARATOR . 'helpers' .
				DIRECTORY_SEPARATOR . 'helper.php');
			JLoader::load('EasyBlogHelper');
		}
		$this->easyBlogVersion = EasyBlogHelper::getLocalVersion();
	}

	/**
	 * Checks for new articles
	 *
	 * @return	boolean
	 *
	 * @since	1.5
	 */
	public function onAfterRoute()
	{
		if ('com_easyblog' == JFactory::getApplication()->input->get('option'))
		{

			// Publish article (backend)
			if (('publish' == JFactory::getApplication()->input->get('task', '')) && ('blogs' == JFactory::getApplication()->input->get('view', '')))
			{
				$this->_initEasyBlog();
				$cid = JFactory::getApplication()->input->get('cid', array(), 'array');
				$article = EasyBlogHelper::getTable('Blog');

				foreach ($cid as $id)
				{
					$article->load($id);

					// Post allowed?
					if (0 == $article->published)
					{
						$this->postMessage($article);
					}
				}
			}

		}

		return true;
	}

	/**
	 * onAfterEasyBlogSave
	 *
	 * @param   object  &$article  An object
	 * @param   bool    $isNew     If the content is just about to be created
	 *
	 * @return	boolean
	 *
	 * @since	1.5
	 */
	public function onAfterEasyBlogSave(&$article, $isNew)
	{
		// Content article
		if (($isNew || $this->post_modified) && (1 == $article->published))
		{
			$this->postMessage($article);
		}

		return true;
	}

	/**
	 * postMessage
	 *
	 * @param   object  $article   The item object.
	 * @param   int     $typeinfo  Type
	 *
* @return	void
	 *
	 * @since	1.5
	 */
	protected function postMessage($article, $typeinfo = self::TYPE_ARTICLE)
	{

		// Type info is not used at the moment (reserved for later use with comments function)

		$cat_filter = explode(',', str_replace(' ', '', $this->categories));
		$excl_cat_filter = explode(',', str_replace(' ', '', $this->excluded_categories));

		$cats = $this->getCategories($article->category_id);
		$catids = $cats[0];

		// Handling for multiple categories
		$isIncludeFilter = true;
		if (!empty($catids) && ('' != $this->categories))
		{
			foreach ($catids as $catid)
			{
				if (in_array($catid, $cat_filter))
				{
					$isIncludeFilter = false;
					break;
				}
			}
		}

		$isExcludeFilter = false;
		if (!empty($catids) && ('' != $this->excluded_categories))
		{
			foreach ($catids as $catid)
			{
				if (in_array($catid, $excl_cat_filter))
				{
					$isExcludeFilter = true;
					break;
				}
			}
		}

		if ((('' == $this->categories) || empty($catids) || !$isIncludeFilter) && (('' == $this->excluded_categories) || !$isExcludeFilter))
		{
			// Create url
			JLoader::register('EasyBlogRouter', JPATH_ROOT . '/components/com_easyblog/helpers/router.php');
			$url = EasyBlogRouter::getEntryRoute($article->id);

			// Get image url
			$search_desc = $article->intro . $article->content;

			// Get the first image from the text
			$image = $this->getImageFromText($search_desc);

			$native_object = json_encode($article);
			$this->postStatusMessage($article->id, $article->publish_up, $article->title, $typeinfo, $url, $image, $native_object);
		}
	}

	/**
	 * getExtendedData
	 *
	 * @param   string  $id              Param.
	 * @param   string  $typeinfo        Param.
	 * @param   string  &$native_object  Param.
	 *
	 * @return	array
	 *
	 * @since	1.5
	 */
	public function getExtendedData($id, $typeinfo, &$native_object)
	{
		// Type info is not used at the moment (reserved for later use with comments function)

		$article = json_decode($native_object);

		// Get categories

		$cats = $this->getCategories($article->category_id);
		$catids = $cats[0];
		$cat_names = $cats[1];

		// Use title or text as twitter message
		$title = $article->title;
		$article_text = $article->intro . ' ' . $article->content;
		$text = $this->getMessagetext($this->use_text, $this->use_text_count, $article->title, $article_text);
		$hashtags = '';

		// Tags and static text
		$tags = implode(',', $this->getTags($id));
		if ((2 == $this->static_text_source) || ((1 == $this->static_text_source) && (empty($tags))))
		{
			$title = $this->addStatictext($this->static_text_pos, $title, $this->static_text);
			$text = $this->addStatictext($this->static_text_pos, $text, $this->static_text);
		}
		elseif (1 == $this->static_text_source)
		{
			$hashtags .= $this->getHashtags($tags, $this->metakey_count);
		}

		// Title
		$cat_result = $this->addCategories($this->show_category, $cat_names, $title, 0);
		$title = $cat_result['text'];

		// Text
		$cat_result = $this->addCategories($this->show_category, $cat_names, $text, $this->show_hash);
		$text = $cat_result['text'];
		if ('' != $cat_result['hashtags'])
		{
			$hashtags .= ' ';
			$hashtags .= $cat_result['hashtags'];
		}

		// Return values
		$data = array(
			'title' => $title,
			'text' => $text,
			'hashtags' => $hashtags,

			// Already done when msg is inserted in queue
			'url' => '',

			// Already done when msg is inserted in queue
			'image_url' => '',

			'fulltext' => $article_text,
			'catids' => $catids,
			'cat_names' => $cat_names,
			'author' => $this->getAuthor($article),
			// 'language' => $article->language,
			// 'access' => $article->access,
			'is_valid' => true
		);

		return $data;
	}

	/**
	 * getCategories
	 *
	 * @param   array  $article_cat  Param.
	 *
	 * @return	array
	 *
	 * @since	1.5
	 */
	private function getCategories($article_cat)
	{
		$cat_ids = array();
		$cat_names = array();

		$current = $article_cat;
		while ($category = $this->_loadCategory($current))
		{
			$cat_ids[] = $category->id;
			$cat_names[] = $category->title;

			$current = $category->parent_id;
		}

		return array($cat_ids, $cat_names);
	}

	/**
	 * _loadCategory.
	 *
	 * @param   int  $catid  Param.
	 *
	 * @return	object.
	 *
	 * @since	1.5
	 */
	private function _loadCategory($catid)
	{
		$_db = JFactory::getDbo();
		$query = $_db->getQuery(true);

		$query->select('id');
		$query->select('title');
		$query->select('parent_id');

		$query->from('#__easyblog_category');

		$query->where('id = ' . $_db->quote($catid));
		$_db->setQuery($query);

		return $_db->loadObject();
	}

	/**
	 * getTags
	 *
	 * @param   int  $aid  Param.
	 *
	 * @return	array
	 *
	 * @since	1.5
	 */
	private function getTags($aid)
	{
		$table_tag = '#__easyblog_tag';
		$table_posttag = '#__easyblog_post_tag';

		$db = JFactory::getDBO();
		$query = 'SELECT t.title FROM ' . $db->quoteName($table_tag) . ' AS t, ' . $db->quoteName($table_posttag) . ' AS p'
				. ' WHERE p.post_id = ' . (int) $aid . ' AND p.tag_id = t.id';
		$db->setQuery($query);
		$tags = $db->loadColumn();

		return $tags;
	}

	/**
	 * getAuthor
	 *
	 * @param   object  &$article  Param.
	 *
	 * @return	string
	 *
	 * @since	1.5
	 */
	private function getAuthor(&$article)
	{
		$uid = $article->created_by;
		$user = JFactory::getUser($uid);

		return $user->username;
	}

}
