<?php

/**
 * @package     Extly.Components
 * @subpackage  autotweeteasydiscuss - Plugin AutoTweetNG EasyDiscuss-Extension
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2012 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

// Check for component
if (!JComponentHelper::getComponent('com_autotweet', true)->enabled)
{
	JError::raiseWarning('5', 'AutoTweet NG EasyDiscuss-Plugin - AutoTweet NG Component is not installed or not enabled.');
	return;
}

include_once JPATH_ROOT . '/administrator/components/com_autotweet/helpers/autotweetbase.php';

/**
 * PlgContentAutotweetEasyDiscuss
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
 */
class PlgContentAutotweetEasyDiscuss extends plgAutotweetBase
{
	// Typeinfo
	const TYPE_EASYDISCUSS_POST = 1;
	const TYPE_EASYDISCUSS_REPLY = 2;

	// Plugin params

	protected $categories = '';

	protected $excluded_categories = '';

	protected $show_catsec = 0;

	protected $show_hash = 0;

	protected $use_text = 0;

	protected $use_text_count = 100;

	protected $static_text = '';

	protected $static_text_pos = 1;

	protected $static_text_source = 0;

	protected $metakey_count = 1;

	protected $interval = 60;

	protected $post_replies = 0;

	/**
	 * plgContentAutotweetEasyDiscuss
	 *
	 * @param   string  &$subject  Param
	 * @param   object  $params    Param
	 *
	 * @since	1.5
	 */
	public function PlgContentAutotweetEasyDiscuss(&$subject, $params)
	{
		parent::__construct($subject, $params);

		$pluginParams = $this->pluginParams;

		// Joomla article specific params
		$this->categories = $pluginParams->get('categories', '');
		$this->excluded_categories = $pluginParams->get('excluded_categories', '');
		$this->show_catsec = (int) $pluginParams->get('show_catsec', 0);
		$this->show_hash = (int) $pluginParams->get('show_hash', 0);
		$this->use_text = (int) $pluginParams->get('use_text', 0);
		$this->use_text_count = $pluginParams->get('use_text_count', 100);
		$this->static_text = strip_tags($pluginParams->get('static_text', ''));
		$this->static_text_pos = (int) $pluginParams->get('static_text_pos', 1);
		$this->static_text_source = (int) $pluginParams->get('static_text_source', 0);
		$this->metakey_count = (int) $pluginParams->get('metakey_count', 1);
		$this->interval = (int) $pluginParams->get('interval', 60);
		$this->post_replies = (int) $pluginParams->get('post_replies', 0);

		// Correct value if value is under the minimum
		if ($this->interval < 60)
		{
			$this->interval = 60;
		}

		// Check type and range, and correct if needed
		$this->use_text_count = $this->getTextcount($this->use_text_count);
	}

	/**
	 * onContentAfterSave
	 *
	 * @param   object  $context  The context of the content passed to the plugin.
	 * @param   object  $article  A JTableContent object
	 * @param   bool    $isNew    If the content is just about to be created
	 *
	 * @return	boolean
	 *
	 * @since	1.5
	 */
	public function onContentAfterSave($context, $article, $isNew)
	{
		$isPost = ($context == 'com_easydiscuss.post');
		$isReply = ($context == 'com_easydiscuss.reply');

		if (($isPost || (($this->post_replies) && ($isReply))) && ($article->published))
		{
				if ($isPost)
				{
					$type = self::TYPE_EASYDISCUSS_POST;
				}
				else
				{
					$type = self::TYPE_EASYDISCUSS_REPLY;
				}

				$this->postEasyDiscuss($article, $type);
		}

		return true;
	}

	/**
	 * onContentAfterSave
	 *
	 * @param   object  $context  The context of the content passed to the plugin.
	 * @param   array   $pks      A list of primary key ids of the content that has changed state.
	 * @param   int     $value    The value of the state that the content has been changed to.
	 *
	 * @return	boolean
	 *
	 * @since	1.5
	 */
	/*
	public function onContentChangeState($context, $pks, $value)
	{
		$isPost = ($context == 'com_easydiscuss.post');
		$isReply = ($context == 'com_easydiscuss.reply');

		if (($isPost || (($this->post_replies) && ($isReply))) && ($value == 1))
		{
			foreach ($pks as $id)
			{
				$article->load($id);

				if ($isPost)
				{
					$type = self::TYPE_EASYDISCUSS_POST;
				}
				else
				{
					$type = self::TYPE_EASYDISCUSS_REPLY;
				}

				$this->postEasyDiscuss($article, $type);
			}
		}

		return true;
	}
	*/

	/**
	 * postEasyDiscuss
	 *
	 * @param   object  $article      The item object.
	 * @param   int     $articletype  Param.
	 *
	 * @return	boolean
	 *
	 * @since	1.5
	 */
	protected function postEasyDiscuss($article, $articletype)
	{
		if (empty($article->publish_up))
		{
			$publish_up = JFactory::getDate()->toSql();
		}
		else
		{
			$publish_up = $article->publish_up;
		}

		// Create internal url
		JLoader::register('DiscussRouter', JPATH_ROOT . '/components/com_easydiscuss/helpers/router.php');
		$discussRouter = new DiscussRouter;

		// EasyDiscuss 2.0
		if (method_exists($discussRouter, 'getEntryRoute'))
		{
			$url = DiscussRouter::getEntryRoute($article->id);
		}

		// EasyDiscuss 3.0
		if (method_exists($discussRouter, 'getPostRoute'))
		{
			if ((isset($article->parent_id)) && ($article->parent_id))
			{
				$url = DiscussRouter::getPostRoute($article->parent_id);
			}
			else
			{
				$url = DiscussRouter::getPostRoute($article->id);
			}
		}

		// Get the first image from the text
		$image_url = $this->getImageFromTextWithBrackets($article->content);

		$native_object = json_encode($article);
		$this->postStatusMessage($article->id, $publish_up, $article->title, $articletype, $url, $image_url, $native_object);
	}

	/**
	 * getCatData
	 *
	 * @param   int  $id  Param
	 *
	 * @return	int
	 *
	 * @since	1.5
	 */
	private function getCatData($id)
	{
		$result = null;

		if ('' != $id)
		{
			$db = JFactory::getDBO();
			$query = 'SELECT ' . $db->quoteName('title') . ' FROM ' . $db->quoteName('#__discuss_category')
			. ' WHERE ' . $db->quoteName('id') . ' = ' . (int) $id;
			$db->setQuery($query);
			$result = $db->loadAssoc();
		}

		return $result;
	}

	/**
	 * getExtendedData
	 *
	 * @param   string  $id              Param.
	 * @param   string  $typeinfo        Param.
	 * @param   string  &$native_object  Param.
	 *
	 * @return	array
	 *
	 * @since	1.5
	 */
	public function getExtendedData($id, $typeinfo, &$native_object)
	{
		$article = json_decode($native_object);

		// Get category path for article
		if ($article->category_id)
		{
			$cat = $this->getCatData($article->category_id);
			$catids = array($article->category_id);
			$cat_names = $cat;
		}
		else
		{
			$catids = array();
			$cat_names = array();
		}

		$article_text = $article->content;
		$article_text = $this->cleanImagesWithBrackets($article_text);

		// Use article title or text as message
		$title = $article->title;
		$text = $this->getMessagetext($this->use_text, $this->use_text_count, $article->title, $article_text);
		$hashtags = '';

		// Use metakey or static text or nothing
		if ((2 == $this->static_text_source) || ((1 == $this->static_text_source) && (empty($article->metakey))))
		{
			$title = $this->addStatictext($this->static_text_pos, $title, $this->static_text);
			$text = $this->addStatictext($this->static_text_pos, $text, $this->static_text);
		}
		elseif (1 == $this->static_text_source)
		{
			$hashtags .= $this->getHashtags($article->metakey, $this->metakey_count);
		}

		// Title
		$catsec_result = $this->addCategories($this->show_catsec, $cat_names, $title, 0);
		$title = $catsec_result['text'];

		// Text
		$catsec_result = $this->addCategories($this->show_catsec, $cat_names, $text, $this->show_hash);
		$text = $catsec_result['text'];
		if ('' != $catsec_result['hashtags'])
		{
			$hashtags .= ' ';
			$hashtags .= $catsec_result['hashtags'];
		}

		$data = array(
			'title' => $title,
			'text' => $text,
			'hashtags' => $hashtags,

			// Already done when msg is inserted in queue
			'url' => '',

			// Already done when msg is inserted in queue
			'image_url' => '',

			'fulltext' => $article_text,
			'catids' => $catids,
			'cat_names' => $cat_names,
			'author' => $this->getAuthorUsername($article->user_id),

			// 'language' => $article->language,
			// 'access' => $article->access,

			'is_valid' => true
		);

		return $data;
	}

	// JUST FOR COMPAT WITH 6.3 - TO BE REMOVED in V6.4

	/**
	 * getImageFromTextWithBrackets
	 *
	 * @param   string  &$img_parsetext  Param.
	 *
	 * @return	string
	 *
	 * @since	1.5
	 */
	protected function getImageFromTextWithBrackets(&$img_parsetext)
	{
		$pattern = '/\[img\]([^\[]+)\[\/img\]/is';
		if (preg_match($pattern, $img_parsetext, $match))
		{
			return $match[1];
		}
		return null;
	}

	/**
	 * cleanImagesWithBrackets
	 *
	 * @param   string  &$img_parsetext  Param.
	 *
	 * @return	string
	 *
	 * @since	1.5
	 */
	protected function cleanImagesWithBrackets(&$img_parsetext)
	{
		$pattern = '/\[img\]([^\[]+)\[\/img\]/is';
		$text = preg_replace($pattern, '', $img_parsetext);
		return $text;
	}

	// JUST FOR COMPAT WITH 6.3 - TO BE REMOVED in V6.4

}
