<?php

/**
 * @package     Extly.Components
 * @subpackage  plgSystemAutotweetK2 - Plugin AutoTweet NG K2-Extension
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2012 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

// Check for component
if (!JComponentHelper::getComponent('com_autotweet', true)->enabled)
{
	JError::raiseWarning('5', 'AutoTweet NG K2-Plugin - AutoTweet NG Component is not installed or not enabled.');
	return;
}

// Check for k2 extension
if (!JComponentHelper::getComponent('com_k2', true)->enabled)
{
	JError::raiseWarning('5', 'AutoTweet NG K2-Plugin - K2 extensions is not installed or not enabled.');
	return;
}

include_once JPATH_ROOT . '/administrator/components/com_autotweet/helpers/autotweetbase.php';

/**
 * PlgSystemAutotweetK2 - K2 extension plugin for AutoTweet
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
*/
class PlgSystemAutotweetK2 extends plgAutotweetBase
{

	protected $categories = '';

	protected $excluded_categories = '';

	protected $post_modified = 0;

	protected $show_category = 0;

	protected $show_hash = 0;

	protected $use_text = 0;

	protected $use_text_count = 100;

	protected $static_text = '';

	protected $static_text_pos = 1;

	protected $static_text_source = 0;

	protected $metakey_count = 1;

	// -1 means: nothing special to do
	protected $post_modified_as_new = -1;

	protected $post_old = 0;

	protected $post_old_date = '0000-00-00';

	protected $polling = 1;

	protected $cronjob = 0;

	/**
	 * plgSystemAutotweetK2
	 *
	 * @param   string  &$subject  Param
	 * @param   object  $params    Param
	 *
	 * @since	1.5
	 */
	public function plgSystemAutotweetK2(&$subject, $params)
	{
		parent::__construct($subject, $params);

		JTable::addIncludePath(JPATH_ROOT . '/administrator/components/com_k2/tables');
		JLoader::register('K2Table', JPATH_ADMINISTRATOR . '/components/com_k2/tables/table.php');
		JLoader::register('K2HelperRoute', JPATH_ROOT . '/components/com_k2/helpers/route.php');

		$pluginParams = $this->pluginParams;

		// K2 specific params
		$this->categories = $pluginParams->get('categories', '');
		$this->excluded_categories = $pluginParams->get('excluded_categories', '');
		$this->post_modified = (int) $pluginParams->get('post_modified', 0);
		$this->show_category = (int) $pluginParams->get('show_category', 0);
		$this->show_hash = (int) $pluginParams->get('show_hash', 0);
		$this->use_text = (int) $pluginParams->get('use_text', 0);
		$this->use_text_count = (int) $pluginParams->get('use_text_count', 100);
		$this->static_text = $pluginParams->get('static_text', '');
		$this->static_text_pos = (int) $pluginParams->get('static_text_pos', 1);
		$this->static_text_source = (int) $pluginParams->get('static_text_source', 0);
		$this->metakey_count = (int) $pluginParams->get('metakey_count', 1);

		$this->post_old = (int) $pluginParams->get('post_old', 0);
		$this->post_old_date = $pluginParams->get('post_old_date', '0000-00-00');
		$this->polling = (int) $pluginParams->get('polling', 0);
		$this->cronjob = (int) $pluginParams->get('cronjob', 0);

		// Check type and range, and correct if needed
		$this->use_text_count = $this->getTextcount($this->use_text_count);
	}

	/*
	 * tricky things to enable posting for first publish of modified posts also when post modified is not enabled
	* This means: When a post is published the first time it is always published as new now!
	*/

	/**
	 * onBeforeK2Save
	 *
	 * @param   object  &$article  A JTableContent object
	 * @param   bool    $isNew     If the content is just about to be created
	 *
	 * @return	boolean
	 *
	 * @since	1.6
	 */
	public function onBeforeK2Save(&$article, $isNew)
	{
		if (!$isNew && !$this->post_modified && (1 == (int) $article->published))
		{
			$old_article = JTable::getInstance('K2Item', 'Table');
			$old_article->load($article->id);

			// When article is modified and is not published, handle as new article
			if (0 == (int) $old_article->published)
			{
				$this->post_modified_as_new = $article->id;
			}
		}

		return true;
	}

	/**
	 * onAfterK2Save
	 *
	 * @param   object  &$article  A JTableContent object
	 * @param   bool    $isNew     If the content is just about to be created
	 *
	 * @return	boolean
	 *
	 * @since	1.5
	 */
	public function onAfterK2Save(&$article, $isNew)
	{
		if (($isNew || $this->post_modified || ($this->post_modified_as_new == (int) $article->id))
			&& (1 == (int) $article->published))
		{
			$this->postMessage($article);
		}

		return true;
	}

	/**
	 * Checks for new articles in the database (polling!!!)
	 *
	 * @return	boolean
	 *
	 * @since	1.5
	 */
	public function onAfterRoute()
	{
		if (('com_k2' == JFactory::getApplication()->input->get('option')) && ('items' == JFactory::getApplication()->input->get('view', '')) && ('publish' == JFactory::getApplication()->input->get('task', '')))
		{
			$ids = JFactory::getApplication()->input->get('cid');
			$article = JTable::getInstance('K2Item', 'Table');

			foreach ($ids as $id)
			{
				$article->load($id);

				// Post allowed?
				if (0 == $article->published)
				{
					$this->postMessage($article);
				}
			}
		}
		else
		{
			if (($this->polling) && (JFactory::getApplication()->isSite()))
			{
				$this->executeContentPolling();
			}
		}

		return true;
	}

	/**
	 * executeContentPolling
	 *
	 * @return	boolean
	 *
	 * @since	1.6
	 */
	protected function executeContentPolling()
	{
		// Get the data for this plugin
		$plugin = 'k2';
		$automat = JTable::getInstance('AutotweetAutomator', 'Table');

		if ($automat)
		{
			$loaded = $automat->load(array('plugin' => $plugin));
		}
		else
		{
			$loaded = false;
		}

		if ($loaded)
		{
			$last = JFactory::getDate($automat->lastexec);
		}
		else
		{
			$last = JFactory::getDate('1999-11-30 00:00:00');
		}

		$now = JFactory::getDate();
		$diff = $now->toUnix() - $last->toUnix();

		if ($diff > $this->interval)
		{
			$entry = array(
					'plugin' => $plugin,
					'lastexec' => $now->toSql()
			);
			$automat->bind($entry);
			$automat->store();

			// Reduced from 1 day to 12 hours [ustorck 2011-06-15]
			$check_from = $now->toUnix();
			$check_from = $check_from - $this->interval;

			// $check_from = $check_from - 86400;
			// 12 hours
			$check_from = $check_from - 43200;
			$check_from = JFactory::getDate($check_from);

			// Set date for posts
			$post_old_mode = false;

			if ($this->post_old)
			{
				// Special case: posting for old articles is enabled
				$post_old_mode = true;
				$last_post = JFactory::getDate($this->post_old_date);

				// Disable old article posting
				$this->disablePostOld();
			}
			else
			{
				$last_post = $check_from;
			}

			// Get new and changed articles form db
			$table_content = '#__k2_items';
			$table_msglog = '#__autotweet_msglog';
			$table_queue = '#__autotweet_queue';

			// Get articles only when they are not in the queue and not in the message log for time horizon
			$db = JFactory::getDBO();
			$query = 'SELECT * FROM ' . $db->quoteName($table_content)
			. ' WHERE (' . $db->quoteName('modified') . ' > ' . $db->Quote($last_post->toSql())
			. ' or ' . $db->quoteName('created') . ' > ' . $db->Quote($last_post->toSql()) . ')'
					. ' AND ' . $db->quoteName('id') . ' NOT IN (SELECT ' . $db->quoteName('ref_id') . ' FROM ' . $db->quoteName($table_queue)
					. ' WHERE ' . $db->quoteName('plugin') . ' = ' . $db->Quote('autotweetk2') . ')'
							. ' AND ' . $db->quoteName('id') . ' NOT IN (SELECT ' . $db->quoteName('ref_id') . ' FROM ' . $db->quoteName($table_msglog)
							. ' WHERE ' . $db->quoteName('plugin') . ' = ' . $db->Quote('autotweetk2')
							. ' AND ' . $db->quoteName('postdate') . ' > ' . $db->Quote($check_from->toSql()) . ')';

			$db->setQuery($query);
			$articles = $db->loadObjectList();

			// Post articles
			foreach ($articles as $article)
			{
				if (('0000-00-00 00:00:00' == $article->modified) || (empty($article->modified)))
				{
					$isNew = true;
				}
				else
				{
					$isNew = false;
				}

				if (($isNew || $this->post_modified || $post_old_mode)
					&& ((0 < $article->featured) || (0 == $this->post_featured_only))
					&& (1 == $article->published))
				{
					$k2article = JTable::getInstance('K2Item', 'Table');
					$k2article->load($article->id);

					$this->postMessage($k2article);
				}
			}
		}
	}

	/**
	 * postMessage
	 *
	 * @param   object  &$article  The item object.
	 *
	 * @return	boolean
	 *
	 * @since	1.5
	 */
	protected function postMessage(&$article)
	{
		$cat_filter = explode(',', str_replace(' ', '', $this->categories));
		$excl_cat_filter = explode(',', str_replace(' ', '', $this->excluded_categories));

		$cats = $this->getCategories($article->catid);
		$catids = $cats[0];

		// Handling for multiple categories
		$isIncludeFilter = true;
		if (!empty($catids) && ('' != $this->categories))
		{
			foreach ($catids as $catid)
			{
				if (in_array($catid, $cat_filter))
				{
					$isIncludeFilter = false;
					break;
				}
			}
		}

		$isExcludeFilter = false;
		if (!empty($catids) && ('' != $this->excluded_categories))
		{
			foreach ($catids as $catid)
			{
				if (in_array($catid, $excl_cat_filter))
				{
					$isExcludeFilter = true;
					break;
				}
			}
		}

		if ((('' == $this->categories) || empty($catids) || !$isIncludeFilter)
			&& (('' == $this->excluded_categories) || !$isExcludeFilter))
		{
			$url = null;

			// Use image form path
			$image_url = $this->getImage($article);

			$native_object = json_encode($article);
			$this->postStatusMessage($article->id, $article->publish_up, $article->title, 0, $url, $image_url, $native_object);
		}
	}

	/**
	 * getExtendedData
	 *
	 * @param   string  $id              Param.
	 * @param   string  $typeinfo        Param.
	 * @param   string  &$native_object  Param.
	 *
	 * @return	array
	 *
	 * @since	1.5
	 */
	public function getExtendedData($id, $typeinfo, &$native_object)
	{
		// Cli mode
		if (!defined('K2_JVERSION'))
		{
			JLoader::register('plgSystemK2', JPATH_PLUGINS . '/system/k2/k2.php');
			JLoader::load('plgSystemK2');
			$plugin = JPluginHelper::getPlugin('system', 'k2');
			$className = 'plg' . $plugin->type . $plugin->name;
			if (!class_exists($className))
			{
				return array('is_valid' => false);
			}
			$dispatcher = JDispatcher::getInstance();
			$plugin = new $className($dispatcher, (array) $plugin);
			$plugin->onAfterInitialise();
		}

		$article = json_decode($native_object);

		// Get categories
		$cats = $this->getCategories($article->catid);
		$catids = $cats[0];
		$cat_names = $cats[1];
		$cat_alias = $cats[2];

		// Use title or text as twitter message
		$title = $article->title;
		$text = $this->getMessagetext($this->use_text, $this->use_text_count, $article->title, $article->introtext);
		$hashtags = '';

		// Use metakey or static text or nothing
		if ((2 == $this->static_text_source) || ((1 == $this->static_text_source) && (empty($article->metakey))))
		{
			$title = $this->addStatictext($this->static_text_pos, $title, $this->static_text);
			$text = $this->addStatictext($this->static_text_pos, $text, $this->static_text);
		}
		elseif (1 == $this->static_text_source)
		{
			$hashtags .= $this->getHashtags($article->metakey, $this->metakey_count);
		}

		// Title
		$cat_result = $this->addCategories($this->show_category, $cat_names, $title, 0);
		$title = $cat_result['text'];

		// Text
		$cat_result = $this->addCategories($this->show_category, $cat_names, $text, $this->show_hash);
		$text = $cat_result['text'];
		if ('' != $cat_result['hashtags'])
		{
			$hashtags .= ' ';
			$hashtags .= $cat_result['hashtags'];
		}

		// Use main category for article url
		$cat_slug = $catids[0] . ':' . urlencode($cat_alias[0]);
		$id_slug = $article->id . ':' . urlencode($article->alias);

		JLoader::load('K2HelperRoute');
		$url = K2HelperRoute::getItemRoute($id_slug, $cat_slug);

		// Return values
		$data = array(
				'title' => $title,
				'text' => $text,
				'hashtags' => $hashtags,

				'url' => $url,

				// Already done when msg is inserted in queue
				// 'image_url' => '',

				'fulltext' => $article->introtext,
				'catids' => $catids,
				'cat_names' => $cat_names,
				'author' => $this->getAuthor($article),
				'language' => $article->language,
				'access' => $article->access,
				'is_valid' => true
		);

		return $data;
	}

	/**
	 * getCategories
	 *
	 * @param   array  $article_cat  Param.
	 *
	 * @return	array
	 *
	 * @since	1.5
	 */
	private function getCategories($article_cat)
	{
		$cat_ids = array();
		$cat_names = array();
		$cat_alias = array();

		JLoader::load('K2Table');
		$row = JTable::getInstance('K2Category', 'Table');
		$id = (int) $article_cat;

		do
		{
			$row->load($id);
			$cat_ids[] = $row->id;
			$cat_names[] = $row->name;
			$cat_alias[] = $row->alias;

			$id = $row->parent;
		}
		while ($row->parent > 0);

		return array($cat_ids, $cat_names, $cat_alias);
	}

	/**
	 * getImage
	 *
	 * @param   object  $article  Param.
	 *
	 * @return	string
	 *
	 * @since	1.5
	 */
	private function getImage($article)
	{
		$image = '';

		// Ckeck for an image in the path. If there is no image, try to use the first image in the article text.
		$imagefilename = md5("Image" . $article->id);
		$imagepath = 'media/k2/items/src/' . $imagefilename . '.jpg';

		// Check for the file
		if (!file_exists(JPATH_ROOT . DIRECTORY_SEPARATOR . $imagepath))
		{
			// If file does not exist, get the first image from the introtext and fulltext
			// Get the first image from the text
			$fulltext = $article->introtext . $article->fulltext;
			$image = $this->getImageFromText($fulltext);
		}
		else
		{
			// Use image form path
			$image = $imagepath;
		}

		return $image;
	}

	/**
	 * getAuthor
	 *
	 * @param   object  $article  Param.
	 *
	 * @return	string
	 *
	 * @since	1.5
	 */
	private function getAuthor($article)
	{
		if (empty($article->modified_by))
		{
			$uid = $article->created_by;
		}
		else
		{
			$uid = $article->modified_by;
		}
		return $this->getAuthorUsername($uid);
	}

	/**
	 * disablePostOld
	 *
	 * @param   object  $plugin  Param.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	protected function disablePostOld($plugin = 'autotweetk2')
	{
		// Get plugin id
		$table = '#__extensions';

		$db = JFactory::getDBO();

		$query = 'SELECT ' . $db->quoteName('extension_id') . ' FROM ' . $db->quoteName($table)
		. ' WHERE ' . $db->quoteName('element') . ' = ' . $db->Quote($plugin)
		. ' AND ' . $db->quoteName('type') . ' = ' . $db->Quote('plugin');

		$db->setQuery($query);
		$id = (int) $db->loadResult();

		// Save parameter
		$this->pluginParams->set('post_old', 0);
		$table = JTable::getInstance('extension');
		$table->load($id);

		$table->params = $this->pluginParams->toString();

		if (!$table->store())
		{
			JError::raiseWarning(500, 'Can not save parameter for AutoTweet Content-Plugin: ' . $table->getError());
		}
	}

}
