<?php

/**
 * @package     Extly.Components
 * @subpackage  PlgUserAutotweetNewuser - Plugin AutoTweet NG New User-Extension for Joomla!
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2012 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

// Check for component
if (!JComponentHelper::getComponent('com_autotweet', true)->enabled)
{
	JError::raiseWarning('5', 'AutoTweet NG New User-Plugin - AutoTweet NG Component is not installed or not enabled.');
	return;
}

include_once JPATH_ROOT . '/administrator/components/com_autotweet/helpers/autotweetbase.php';

/**
 * PlgSystemAutotweetNewuser class.
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
 */
class PlgUserAutotweetNewuser extends plgAutotweetBase
{
	const TYPE_NEWUSER		= 1;
	const TYPE_UPDATEUSER	= 2;

	protected $post_modified		= 0;

	protected $static_text_new		= '';

	protected $static_text_update	= '';

	protected $static_text_pos		= 2;

	protected $show_user			= 2;

	protected $static_title			= '';

	protected $url_extension		= '';

	// -1 means: nothing special to do
	protected $user_activation		= -1;

	/**
	 * Constructor.
	 *
	 * @param   string  &$subject  Param
	 * @param   array   $params    Configuration
	 *
	 * @since	1.5
	 */
	public function plgUserAutotweetNewuser(&$subject, $params)
	{
		parent::__construct($subject, $params);

		$pluginParams = $this->pluginParams;

		$this->post_modified		= (int) $pluginParams->get('post_modified', 0);
		$this->static_text_new		= strip_tags($pluginParams->get('static_text_new', 'has joined the community'));
		$this->static_text_update	= strip_tags($pluginParams->get('static_text_update', 'has updated user data'));
		$this->static_text_pos		= (int) $pluginParams->get('static_text_pos', 2);
		$this->show_user			= (int) $pluginParams->get('show_user', 2);
		$this->static_title			= strip_tags($pluginParams->get('static_title', 'New user'));
		$this->url_extension		= $pluginParams->get('url_extension', '');
	}

	/**
	 * It's used to check if this is an user activation or an normal user update
	 *
	 * @param   object  $user   Param
	 * @param   bool    $isNew  Param
	 *
	 * @return	bool.
	 *
	 * @since	1.5
	 */
	public function onUserBeforeSave($user, $isNew)
	{
		if (!$isNew	&& !$this->post_modified && (1 == $user['block']))
		{
			// When user is modified and was blocked before, handle as new user
			$this->user_activation = $user['id'];
		}

		return true;
	}

	/**
	 * onUserAfterSave
	 *
	 * @param   array   $user     An associative array of the columns in the user table.
	 * @param   bool    $isNew    Boolean to identify if this is a new user (true - insert) or an existing one (false - update)
	 * @param   bool    $success  Boolean to identify if the store was successful
	 * @param   string  $msg      Error message if store failed
	 *
	 * @return	bool.
	 *
	 * @since	1.5
	 */
	public function onUserAfterSave($user, $isNew, $success, $msg)
	{
		if ($isNew || ($this->user_activation == $user['id']))
		{
			$type = self::TYPE_NEWUSER;
		}
		else
		{
			$type = self::TYPE_UPDATEUSER;
		}

		if ((($type == self::TYPE_NEWUSER) || $this->post_modified) && $success)
		{
			$this->postStatusMessage($user['id'], JFactory::getDate()->toSql(), $user['username'], $type);
		}

		return true;
	}

	/**
	 * getData
	 *
	 * @param   string  $id        Param.
	 * @param   string  $typeinfo  Param.
	 *
	 * @return	array
	 *
	 * @since	1.5
	 */
	public function getData($id, $typeinfo)
	{
		$user =& JFactory::getUser($id);
		if (!empty($user))
		{
			// Username
			switch ($this->show_user)
			{
				case 1:
					$username = $user->username;
					break;
				case 2:
					$username = $user->name;
					break;
				default:
					$username = '';
					break;
			}

			if ($typeinfo == self::TYPE_NEWUSER)
			{
				$text = $this->static_text_new;
			}
			else
			{
				$text = $this->static_text_update;
			}

			// Message text
			$text = $this->addStatictext($this->static_text_pos, $username, $text);

			// Create url:  as default the site url is routed and this is done by the url routing
			if (empty($this->url_extension))
			{
				$url = "/";
			}
			else
			{
				$url = $this->url_extension;
			}

			$data = array (
					'title'		=> $this->static_title,
					'text'		=> $text,
					'hashtags'	=> '',
					'url'		=> $url,
					'image_url'	=> '',
					'fulltext'	=> '',
					'catids'	=> array (),
					'author'	=> $user->username,
					'is_valid'	=> true
			);
		}
		else
		{
			$data = array ('is_valid' => false);
		}

		return $data;
	}

}
