<?php

/**
 * @package     Extly.Components
 * @subpackage  plgSystemAutotweetZOO - Plugin AutoTweet NG ZOO-Extension
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2012 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

// Check for component
if (!JComponentHelper::getComponent('com_autotweet', true)->enabled)
{
	JError::raiseWarning('5', 'AutoTweet NG VirtueMart-Plugin - AutoTweet NG Component is not installed or not enabled.');
	return;
}

// Check for extension
if (!JComponentHelper::getComponent('com_virtuemart', true)->enabled)
{
	JError::raiseWarning('5', 'AutoTweet NG VirtueMart-Plugin - VirtueMart extensions is not installed or not enabled.');
	return;
}

include_once JPATH_ROOT . '/administrator/components/com_autotweet/helpers/autotweetbase.php';

/**
 * PlgSystemAutotweetVirtuemart - VirtueMart extension plugin for AutoTweet.
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
 */
class PlgSystemAutotweetVirtuemart extends plgAutotweetBase
{
	// Typeinfo
	const TYPE_ARTICLE	= 1;
	const TYPE_REVIEW	= 2;

	protected $categories			= '';

	protected $excluded_categories	= '';

	protected $post_modified		= 0;

	protected $show_category		= 0;

	protected $show_hash_cat		= 0;

	protected $use_text				= 0;

	protected $use_text_count		= 100;

	protected $static_text			= '';

	protected $static_text_pos		= 0;

	protected $post_comments			= 0;

	protected $comment_use_text			= 2;

	protected $comment_use_text_count	= 100;

	// Db table to get next item id
	protected $table_items			= '#__virtuemart_products';

	/**
	 * PlgSystemAutotweetVirtuemart
	 *
	 * @param   string  &$subject  Param
	 * @param   object  $params    Param
	 *
	 * @since	1.5
	 */
	public function plgSystemAutotweetVirtuemart (&$subject, $params)
	{
		parent::__construct($subject, $params);

		// Add path to Virtuemart tables
		if (!class_exists('VmConfig'))
		{
			include_once JPATH_ADMINISTRATOR . '/components/com_virtuemart/helpers/config.php';
		}
		$config = VmConfig::loadConfig();

		$pluginParams = $this->pluginParams;

		$this->categories				= $pluginParams->get('categories', '');
		$this->excluded_categories		= $pluginParams->get('excluded_categories', '');
		$this->post_modified			= (int) $pluginParams->get('post_modified', 0);
		$this->show_category			= (int) $pluginParams->get('show_category', 0);
		$this->show_hash_cat			= (int) $pluginParams->get('show_hash_cat', 0);
		$this->use_text					= (int) $pluginParams->get('use_text', 0);
		$this->use_text_count			= (int) $pluginParams->get('use_text_count', 100);
		$this->static_text				= $pluginParams->get('static_text', '');
		$this->static_text_pos			= (int) $pluginParams->get('static_text_pos', 0);

		$this->post_comments			= (int) $pluginParams->get('post_comments', 0);
		$this->comment_use_text			= (int) $pluginParams->get('comment_use_text', 0);
		$this->comment_use_text_count	= (int) $pluginParams->get('comment_use_text_count', 100);

		// Check type and range, and correct if needed
		$this->use_text_count = $this->getTextcount($this->use_text_count);
	}

	/**
	 * Handles normal posts and later published articles
	 *
	 * @return	boolean
	 *
	 * @since	1.5
	 */
	public function onAfterRoute()
	{
		if ('com_virtuemart' == JFactory::getApplication()->input->get('option'))
		{
			$task	= JFactory::getApplication()->input->get('task', '');
			$view	= JFactory::getApplication()->input->get('view', '');
			$id		= (int) JFactory::getApplication()->input->get('virtuemart_product_id', 0);

			// Normal post
			if (('product' == $view) && (('save' == $task) || ('apply' == $task)) && (0 < (int) JFactory::getApplication()->input->get('published', 0)))
			{
				// When article is modified and is not published, handle as new article
				$post_modified_as_new = false;
				if (!empty($id) && !$this->post_modified)
				{
					$item = JTable::getInstance('Products', 'Table');
					$item->load($id);

					if (0 == (int) $item->published)
					{
						$post_modified_as_new = true;
					}
				}

				if ($this->post_modified || (!$this->post_modified && empty($id)) || $post_modified_as_new)
				{
					$this->postMessage($id, JFactory::getApplication()->input->get('categories', array(), 'ARRAY'), JFactory::getApplication()->input->get('product_name'), self::TYPE_ARTICLE);
				}
			}
			elseif (('publish' == $task) && ('product' == $view))
			{
				if (!is_array($id))
				{
					$ids = array ($id);
				}
				else
				{
					$ids = $id;
				}

				$item = JTable::getInstance('Products', 'Table');
				foreach ($ids as $id)
				{
					$item->load($id);

					// Post allowed?
					if (0 < (int) $item->published)
					{
						$catids = $this->getCategoryIDs($id);
						$this->postMessage($id, $catids, $item->product_name, self::TYPE_ARTICLE);
					}
				}
			}
			elseif (((('ratings' == $view) && (('saveReview' == $task) || ('applyReview' == $task)))
					|| (('productdetails' == $view) && ('review' == $task)))
					&& (0 < (int) JFactory::getApplication()->input->get('published', 0)) && $this->post_comments)
			{
				$catids = $this->getCategoryIDs((int) JFactory::getApplication()->input->get('virtuemart_product_id'));
				$this->postMessage((int) JFactory::getApplication()->input->get('virtuemart_rating_review_id'), $catids, JFactory::getApplication()->input->get('comment'), self::TYPE_REVIEW);
			}
			elseif (('publish' == $task) && ('ratings' == $view) && $this->post_comments)
			{
				$ids = JFactory::getApplication()->input->get('cid', array(), 'ARRAY');
				$review = JTable::getInstance('Rating_reviews', 'Table');

				foreach ($ids as $id)
				{
					$review->load($id);

					// Post allowed?
					if (0 == (int) $review->published)
					{
						$catids = $this->getCategoryIDs($review->virtuemart_product_id);
						$this->postMessage($id, $catids, $review->comment, self::TYPE_REVIEW);
					}
				}
			}
		}

		return true;
	}

	/**
	 * postMessage
	 *
	 * @param   int     $id        An object
	 * @param   array   $catids    Param
	 * @param   string  $desc      Param
	 * @param   int     $typeinfo  Param
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	protected function postMessage($id, $catids, $desc, $typeinfo)
	{
		$cat_filter			= explode(',', str_replace(' ', '', $this->categories));
		$excl_cat_filter	= explode(',', str_replace(' ', '', $this->excluded_categories));

		// Handling for multiple categories
		$isIncludeFilter = true;
		if (!empty($catids) && ('' != $this->categories))
		{
			foreach ($catids as $catid)
			{
				if (in_array($catid, $cat_filter))
				{
					$isIncludeFilter = false;
					break;
				}
			}
		}

		$isExcludeFilter = false;
		if (!empty($catids) && ('' != $this->excluded_categories))
		{
			foreach ($catids as $catid)
			{
				if (in_array($catid, $excl_cat_filter))
				{
					$isExcludeFilter = true;
					break;
				}
			}
		}

		if ((('' == $this->categories) || empty($catids) || !$isIncludeFilter)
			&& (('' == $this->excluded_categories) || !$isExcludeFilter))
		{
			if (empty($id))
			{
				$id	= $this->getID($this->table_items);
			}

			$this->postStatusMessage($id, JFactory::getDate()->toSql(), $desc, $typeinfo);
		}
	}

	/**
	 * getData
	 *
	 * @param   string  $id        Param.
	 * @param   string  $typeinfo  Param.
	 *
	 * @return	array
	 *
	 * @since	1.5
	 */
	public function getData($id, $typeinfo)
	{
		$item = JTable::getInstance('Products', 'Table');

		// Check if this is a comment or an article
		if (self::TYPE_REVIEW == $typeinfo)
		{
			$review = JTable::getInstance('Rating_reviews', 'Table');
			$review->load($id);
			$item->load($review->virtuemart_product_id);
		}
		else
		{
			$review = null;
			$item->load($id);
		}

		if (!empty($item))
		{
			$catids = $this->getCategoryIDs($item->virtuemart_product_id);

			// Hashtags are handled seperatly now
			$hashtags = '';

			// Categorie handling
			$cat_names = array ();
			$i = 0;
			$cat_id = 0;
			$cat = JTable::getInstance('Categories', 'Table');
			foreach ($catids as $catid)
			{
				$i++;
				$cat->load($catid);
				$cat_names[] = $cat->category_name;
				if (1 == $i)
				{
					$cat_id = $cat->virtuemart_category_id;
				}
			}

			// Get the itemid
			// Only default component itemid
			$needles = array();
			$itemid = $this->getItemid('com_virtuemart', $needles);

			// Set internal url
			$url = 'index.php?option=com_virtuemart&view=productdetails&virtuemart_product_id=' .
				$item->virtuemart_product_id .
				'&virtuemart_category_id=' . $cat_id .
				'&Itemid=' . $itemid;

			// Use title or text as twitter message
			$title = $item->product_name;

			if (self::TYPE_REVIEW == $typeinfo)
			{
				$desc = $review->comment;
				$use_text = $this->comment_use_text;
				$use_text_count = $this->comment_use_text_count;
			}
			else
			{
				$desc = $item->product_s_desc . ' ' . $item->product_desc;
				$use_text = $this->use_text;
				$use_text_count = $this->use_text_count;
			}

			$text = $this->getMessagetext($use_text, $use_text_count, $item->product_name, $desc);

			// Add static text
			$title = $this->addStatictext($this->static_text_pos, $title, $this->static_text);
			$text = $this->addStatictext($this->static_text_pos, $text, $this->static_text);

			// Add categories
			$catsec_result = $this->addCategories($this->show_category, $cat_names, $title, 0);
			$title = $catsec_result['text'];

			$catsec_result = $this->addCategories($this->show_category, $cat_names, $text, $this->show_hash_cat);
			$text = $catsec_result['text'];
			if ('' != $catsec_result['hashtags'])
			{
				$hashtags .= ' ';
				$hashtags .= $catsec_result['hashtags'];
			}

			// Return values
			$data = array (
					'title'		=> $title,
					'text'		=> $text,
					'hashtags'	=> $hashtags,
					'url'		=> $url,
					'image_url'	=> $this->getImage($item->virtuemart_product_id),
					'fulltext'	=> $desc,
					'catids'	=> $catids,
					'is_valid'	=> true
			);
		}
		else
		{
			$data = array ('is_valid' => false);
		}

		return $data;
	}

	/**
	 * Get category ids for all related cats
	 *
	 * @param   int  $id  Param.
	 *
	 * @return	array
	 *
	 * @since	1.5
	 */
	private function getCategoryIDs($id)
	{
		$table = '#__virtuemart_product_categories';

		$db = &JFactory::getDBO();
		$query = 'SELECT '	. $db->quoteName('virtuemart_category_id') . ' FROM ' . $db->quoteName($table) . ' WHERE '
				. $db->quoteName('virtuemart_product_id') . ' = ' . (int) $id
				. ' ORDER BY ' . $db->quoteName('virtuemart_category_id') . ' ASC ';

		$db->setQuery($query);

		return $db->loadColumn();
	}

	/**
	 * getImage
	 *
	 * @param   int  $id  Param.
	 *
	 * @return	array
	 *
	 * @since	1.5
	 */
	private function getImage($id)
	{
		$db		= &JFactory::getDBO();

		$prod_media_table	= '#__virtuemart_product_medias';
		$media_table		= '#__virtuemart_medias';

		$sub_query = 'SELECT '	. $db->quoteName('virtuemart_media_id') . ' FROM ' . $db->quoteName($prod_media_table)
		. ' WHERE ' . $db->quoteName('virtuemart_product_id') . ' = ' . (int) $id;

		$query = 'SELECT '	. $db->quoteName('file_url_thumb') . ' FROM ' . $db->quoteName($media_table)
		. ' WHERE ' . $db->quoteName('virtuemart_media_id') . ' IN (' . $sub_query . ') '
				. ' AND ' . $db->quoteName('file_is_product_image') . ' = 1 LIMIT  1';

		$db->setQuery($query);

		return $db->loadResult();
	}

}
