<?php

/**
 * @package     Extly.Components
 * @subpackage  plgSystemAutotweetZOO - Plugin AutoTweet NG ZOO-Extension
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2012 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

// Check for component
if (!JComponentHelper::getComponent('com_autotweet', true)->enabled)
{
	JError::raiseWarning('5', 'AutoTweet NG ZOO-Plugin - AutoTweet NG Component is not installed or not enabled.');
	return;
}

// Check for ZOO extension
if (!JComponentHelper::getComponent('com_zoo', true)->enabled)
{
	JError::raiseWarning('5', 'AutoTweet NG ZOO-Plugin - ZOO extensions is not installed or not enabled.');
	return;
}

include_once JPATH_ROOT . '/administrator/components/com_autotweet/helpers/autotweetbase.php';

// Zoo files
require_once JPATH_ADMINISTRATOR . '/components/com_zoo/config.php';

/**
 * ZOO extension plugin for AutoTweet.
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
 */
class PlgSystemAutotweetZOO extends plgAutotweetBase
{
	// Typeinfo

	const TYPE_ARTICLE = 1;
	const TYPE_COMMENT = 2;

	protected $categories = '';

	protected $excluded_categories = '';

	protected $post_modified = 0;

	protected $show_category = 0;

	protected $show_hash = 0;

	protected $use_text = 0;

	protected $use_text_count = 100;

	protected $static_text = '';

	protected $static_text_pos = 1;

	protected $static_text_source = 0;

	// Used for tags
	protected $metakey_count = 1;

	protected $post_comments = 0;

	protected $comment_use_text = 2;

	protected $comment_use_text_count = 100;

	// -1 means: we are not in comment posting mode
	protected $comment_posting_mode = -1;

	/**
	 * PlgSystemAutotweetZOO
	 *
	 * @param   string  &$subject  Param
	 * @param   object  $params    Param
	 *
	 * @since	1.5
	 */
	public function PlgSystemAutotweetZOO(&$subject, $params)
	{
		parent::__construct($subject, $params);

		$pluginParams = $this->pluginParams;

		// Plugin specific params
		$this->categories = $pluginParams->get('categories', null);
		$this->excluded_categories = $pluginParams->get('excluded_categories', null);
		$this->post_modified = (int) $pluginParams->get('post_modified', 0);
		$this->show_category = (int) $pluginParams->get('show_category', 0);
		$this->show_hash = (int) $pluginParams->get('show_hash', 0);
		$this->use_text = (int) $pluginParams->get('use_text', 0);
		$this->use_text_count = (int) $pluginParams->get('use_text_count', 100);
		$this->static_text = $pluginParams->get('static_text', '');
		$this->static_text_pos = (int) $pluginParams->get('static_text_pos', 1);
		$this->static_text_source = (int) $pluginParams->get('static_text_source', 0);

		// Used for tags
		$this->metakey_count = (int) $pluginParams->get('metakey_count', 1);

		$this->post_comments = (int) $pluginParams->get('post_comments', 0);
		$this->comment_use_text = (int) $pluginParams->get('comment_use_text', 0);
		$this->comment_use_text_count = (int) $pluginParams->get('comment_use_text_count', 100);

		// Check type and range, and correct if needed
		$this->use_text_count = $this->getTextcount($this->use_text_count);
		$this->comment_use_text_count = $this->getTextcount($this->comment_use_text_count);
	}

	/**
	 * onAfterInitialise handler
	 *
	 * Adds ZOO event listeners
	 *
	 * @access	public
	 * @return null
	 */
	public function onAfterInitialise()
	{
		// Get the ZOO App instance
		$zoo = App::getInstance('zoo');

		// Register events
		$zoo->event->dispatcher->connect('item:saved', array($this, 'itemSaved'));
		$zoo->event->dispatcher->connect('item:stateChanged', array($this, 'itemStateChanged'));
		$zoo->event->dispatcher->connect('comment:saved', array($this, 'commentSaved'));
		$zoo->event->dispatcher->connect('comment:stateChanged', array('$this', 'commentStateChanged'));
	}

	/**
	 * itemSaved
	 *
	 * @param   object  $event  An object
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function itemSaved($event)
	{
		// Itemsaved is also triggered when article is published.
		// So there is no need to use itemStateChanged and publish is handled here!
		$item = $event->getSubject();
		$isNew = $event['new'];

		if (($isNew || $this->post_modified) && (1 == (int) $item->state) && ($this->comment_posting_mode != $item->id))
		{
			// Clear id to avoid mistakes
			$this->comment_posting_mode = -1;

			$this->postMessage($item, self::TYPE_ARTICLE);
		}
	}

	/**
	 * itemStateChanged
	 *
	 * @param   object  $event  An object
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function itemStateChanged($event)
	{
		// Saved is also triggered when article is published.
		// So we have to do some special handling for state changes only when post for modified artciles is disabled!
		$item = $event->getSubject();

		if ((0 == $this->post_modified) && (1 == (int) $item->state))
		{
			$this->postMessage($item, self::TYPE_ARTICLE);
		}
	}

	/**
	 * commentSaved
	 *
	 * @param   object  $event  An object
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function commentSaved($event)
	{
		$comment = $event->getSubject();
		$this->comment_posting_mode = $comment->getItem()->id;

		if ($this->post_comments && (1 == (int) $comment->state))
		{
			$this->postMessage($comment, self::TYPE_COMMENT);
		}
	}

	/**
	 * commentStateChanged
	 *
	 * @param   object  $event  An object
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function commentStateChanged($event)
	{
		// Clear id to avoid mistakes
		$this->comment_posting_mode = -1;
	}

	/**
	 * postMessage
	 *
	 * @param   object  $item      An object
	 * @param   int     $typeinfo  Param
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	protected function postMessage($item, $typeinfo)
	{
		// Check if this is a comment or an article
		if (self::TYPE_COMMENT == $typeinfo)
		{
			$article = $item->getItem();
		}
		else
		{
			$article = $item;
		}

		$cat_filter = $this->categories;
		$excl_cat_filter = $this->excluded_categories;

		$catids = $article->getRelatedCategoryIds();

		// Handling for multiple categories
		$isIncludeFilter = true;
		if (!empty($catids) && ($this->categories))
		{
			foreach ($catids as $catid)
			{
				if (in_array($catid, $cat_filter))
				{
					$isIncludeFilter = false;
					break;
				}
			}
		}

		$isExcludeFilter = false;
		if (!empty($catids) && ($this->excluded_categories))
		{
			foreach ($catids as $catid)
			{
				if (in_array($catid, $excl_cat_filter))
				{
					$isExcludeFilter = true;
					break;
				}
			}
		}

		if (((!$this->categories) || empty($catids) || !$isIncludeFilter)
			&& ((!$this->excluded_categories) || !$isExcludeFilter))
		{
			$url = null;

			// Create url
			$zoo = App::getInstance('zoo');
			$url = $zoo->route->item($article, false);

			// Get image url
			$image_desc = $this->getImageAndDesc($article);

			// Handle comment
			if (self::TYPE_COMMENT == $typeinfo)
			{
				$desc = $item->content;
				$publish_up = $item->created;
			}
			else
			{
				$desc = $article->name;
				$publish_up = $article->publish_up;
			}

			$native_object = json_encode($item);
			$this->postStatusMessage($item->id, $publish_up, $desc, $typeinfo, $url, $image_desc[0], $native_object);
		}
	}

	/**
	 * getExtendedData
	 *
	 * @param   string  $id              Param.
	 * @param   string  $typeinfo        Param.
	 * @param   string  &$native_object  Param.
	 *
	 * @return	array
	 *
	 * @since	1.5
	 */
	public function getExtendedData($id, $typeinfo, &$native_object)
	{
		$item = json_decode($native_object);

		// Get zoo
		$zoo = App::getInstance('zoo');

		// Check if this is a comment or an article
		if (self::TYPE_COMMENT == $typeinfo)
		{
			$comment = $zoo->table->comment->get($item->id);
			$article = $zoo->table->item->get($comment->getItem()->id);
		}
		else
		{
			$comment = null;
			$article = $zoo->table->item->get($item->id);
		}

		if (!empty($article))
		{
			// Get categories
			$cat_names = array();
			$cats = $article->getRelatedCategories();
			foreach ($cats as $cat)
			{
				$cat_names[] = $cat->name;
			}

			// Use title or text as twitter message
			$title = $article->name;
			$image_desc = $this->getImageAndDesc($article);

			if (self::TYPE_COMMENT == $typeinfo)
			{
				$zoodesc = $comment->content;
				$use_text = $this->comment_use_text;
				$use_text_count = $this->comment_use_text_count;
			}
			else
			{
				$zoodesc = $image_desc[1];
				$use_text = $this->use_text;
				$use_text_count = $this->use_text_count;
			}

			$text = $this->getMessagetext($use_text, $use_text_count, $article->name, $zoodesc);
			$hashtags = '';

			// Tags and static text
			$tags = implode(',', $article->getTags());

			if ((2 == $this->static_text_source) || ((1 == $this->static_text_source) && (empty($tags))))
			{
				$title = $this->addStatictext($this->static_text_pos, $title, $this->static_text);
				$text = $this->addStatictext($this->static_text_pos, $text, $this->static_text);
			}
			elseif (1 == $this->static_text_source)
			{
				$hashtags .= $this->getHashtags($tags, $this->metakey_count);
			}

			// Title
			$cat_result = $this->addCategories($this->show_category, $cat_names, $title, 0);
			$title = $cat_result['text'];

			// Text
			$cat_result = $this->addCategories($this->show_category, $cat_names, $text, $this->show_hash);
			$text = $cat_result['text'];
			if ('' != $cat_result['hashtags'])
			{
				$hashtags .= ' ';
				$hashtags .= $cat_result['hashtags'];
			}

			// Return values
			$data = array(
					'title' => $title,
					'text' => $text,
					'hashtags' => $hashtags,

					// Already done when msg is inserted in queue
					'url' => $url,

					// Already done when msg is inserted in queue
					'image_url' => '',

					'fulltext' => $zoodesc,
					'catids' => $article->getRelatedCategoryIds(),
					'cat_names' => $cat_names,
					'author' => $this->getAuthor($article),
					// 'language' => $article->language,
					'is_valid' => true
			);
		}
		else
		{
			$data = array('is_valid' => false);
		}

		return $data;
	}

	/**
	 * getImageAndDesc
	 *
	 * @param   object  &$article  Param.
	 *
	 * @return	array
	 *
	 * @since	1.5
	 */
	private function getImageAndDesc(&$article)
	{
		$image = '';
		$text = '';

		$zoo = App::getInstance('zoo');
		$zoo_version = $zoo->zoo->version();

		if (version_compare($zoo_version, '2.5') >= 0)
		{
			// ZOO 2.5 or higher: now ZOO has dynamic fields and so it is more complicated to get images and description text
			$zElements = json_decode($article->elements);
			$zTextCode = '2e3c9e69-1f9e-4647-8d13-4e88094d2790';
			$zTeaserCode = 'cdce6654-4e01-4a7f-9ed6-0407709d904c';
			$zImageCode = 'c26feca6-b2d4-47eb-a74d-b067aaae5b90';

			// Add all text elements to text
			if (isset($zElements->$zTextCode))
			{
				foreach ($zElements->$zTextCode as $txt)
				{
					$text .= $txt->value;
				}
			}

			// Find image: first search for teaser or for image. If there is no image/teaser, search in text for image tag
			if (isset($zElements->$zTeaserCode) && !empty($zElements->$zTeaserCode->file))
			{
				$image = $zElements->$zTeaserCode->file;
			}
			elseif (isset($zElements->$zImageCode) && !empty($zElements->$zImageCode->file))
			{
				$image = $zElements->$zImageCode->file;
			}
			elseif (!empty($text))
			{
				$image = $this->getImageFromText($text);
			}
		}
		else
		{
			JError::raiseWarning('5', 'AutoTweet NG ZOO-Plugin - ZOO 2.5 or higher support only.');
		}

		return array($image, $text);
	}

	/**
	 * getAuthor
	 *
	 * @param   object  &$article  Param.
	 *
	 * @return	string
	 *
	 * @since	1.5
	 */
	private function getAuthor(&$article)
	{
		if (empty($article->modified_by))
		{
			$uid = $article->created_by;
		}
		else
		{
			$uid = $article->modified_by;
		}
		return $this->getAuthorUsername($uid);
	}

}
