<?php

/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - AutoTweetNG posts content to social channels (Twitter, Facebook, LinkedIn, etc).
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2012 Prieco, S.A. All rights reserved.
 * @license     http://http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

defined('JPATH_AUTOTWEET') || define('JPATH_AUTOTWEET', JPATH_ADMINISTRATOR . '/components/com_autotweet');
defined('JPATH_AUTOTWEET_HELPERS') || define('JPATH_AUTOTWEET_HELPERS', JPATH_AUTOTWEET . '/helpers');

/**
 *  This class is a facade for the posting function of AutoTweet.
 *  It should be used as an API for external components only.
 **/

/**
 * Instructions to enhance AutoTweet  for external component:
 *
 * There are two ways to use AutoTweet with your own component.
 * 1. Write an extension plugin for AutoTweet.
 * 2. Post (and log) a message with the API function  AutotweetAPI::postMessage()
 *
 * 1. Write an extension plugin for AutoTweet: Your plugin must derive form the base class  plgAutotweetBase
 * (see /administrator/components/com_autotweet/helpers/autotweetbase.php)  and must implement the function
 * getData or getExtendedData to return the data for the post. Also it must trigger the function postStatusMessage
 * to queue the message for posting. See my own AutoTweet extension plugins as example.
 *
 * The standard Content or Weblinks plugins are simple and have all the required features.
 *
 * 2. Post (and log) a message with the API function  AutotweetAPI::postMessage(): With this function you can send
 * a message over AutoTweet without the need to write a plugin. So you can call this function directly from your component.
 * This functions send the message directly (The queue is not used).
 *
	 * Function to post a message from an external component. This message is NOT queued.
	 *
	 * $article_id	id of article, event, forum post, ...
	 * $data = array (
	 *	'title'	=> 'Title of the article, event, forum post, ...',
	 *	'text'	=> 'Text for the message.',
	 *	'hashtags'	=> 'hashtags (They are inserted automatically by AutoTweet for channels supporting hashtags.',
	 *	'url'		=> 'The URL of the article. (not shortened)'
	 *	'image_url'	=> 'URL for the image to show (if there is an image)'
	 *	'fulltext'	=> 'Enhanced text for the description (e. g. Facebook). Not available for all channels.'
	 *	'catids'	=> 'array with category, container, etc. ids for the article, post...'
	 *	'author'	=> 'creator or editor of the article'
	 * 	'event'	=> array (
	 *		'location'		=> 'Location of the event'
	 *		'street'		=> 'Street is optional'
	 *		'city'			=> 'Must be a valid City for Facebook'
	 *		'privacy'		=> 'OPEN, CLOSED or SECRET'
	 *		'start_time'	=> 'yyyy-mm-dd hh:ss'
	 *		'end_time'	=> 'yyyy-mm-dd hh:ss'
	 *	);
	 * );
	 *
	 * AutoTweetNG v6.4 Breaking Changes
	 * ---------------------------------
	 * All messages follow the same processing rules:
	 *
	 * "Source" -> Request -> Post -> "Social Networks"
	 *
	 * - If the "Source" is a plugin, getData or getExtendedData are called to enrich the message.
	 *
	 * - A Manual Post is now also a Plugin, following the same rules.
	 *
	 * - If the "Source" is not plugin, getData or getExtendedData are not called and the Request
	 * must have all required information for a valid Post
	 *
	 *
*/

/**
 * AutotweetAPI
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
 */
abstract class AutotweetAPI
{

	/**
	 * __construct
	 *
	 * @since	1.5
	 */
	private function __construct()
	{
		// Nothing to do
	}

	/**
	 * postMessage
	 *
	 * @param   string  $article_id     Param
	 * @param   string  $data           Param
	 * @param   string  $source         Param
	 * @param   string  $autopublish    Param
	 * @param   enum    $show_url_mode  Param
	 *
	 * @return	bool
	 *
	 * @since	1.5
	 */
	public static function postMessage($article_id, $data, $source, $autopublish = true, $show_url_mode = AutotweetPostHelper::SHOWURL_END)
	{
		RequestHelp::insertRequest(
			$description,
			JFactory::getDate()->toMySql()
		);
	}

	/**
	 * postMessage
	 *
	 * @param   string  $articleid      Param
	 * @param   string  $source_plugin  Param
	 * @param   string  $publish_up     Param
	 * @param   string  $description    Param
	 * @param   string  $typeinfo       Param
	 * @param   string  $url            Param
	 * @param   string  $image_url      Param
	 * @param   object  $native_object  Param
	 *
	 * @return	bool
	 *
	 * @since	1.5
	 */
	public static function postManualMessage(
		$articleid,
		$source_plugin,
		$publish_up,
		$description,
		$typeinfo = 0,
		$url = '',
		$image_url = '',
		$native_object = null)
	{
		JLoader::register('CparamsHelper', JPATH_AUTOTWEET_HELPERS . '/cparams.php');
		JLoader::register('AutotweetPostHelper', JPATH_AUTOTWEET_HELPERS . '/autotweetposthelper.php');
		JLoader::register('AutotweetBaseHelper', JPATH_AUTOTWEET_HELPERS . '/autotweetbasehelper.php');
		JLoader::register('RouteHelp', JPATH_AUTOTWEET_HELPERS . '/route.php');

		$helper = AutotweetPostHelper::getInstance();
		return $helper->queueMessage(
				$articleid,
				$source_plugin,
				$publish_up,
				$description,
				$typeinfo = 0,
				$url = '',
				$image_url = '',
				$native_object = null
			);
	}

}
