<?php

/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - AutoTweetNG posts content to social channels (Twitter, Facebook, LinkedIn, etc).
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2012 Prieco, S.A. All rights reserved.
 * @license     http://http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

/**
 * AutotweetControllerPosts
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
*/
class AutotweetControllerPosts extends FOFController
{

	/**
	 * publish.
	 *
	 * @return	void
	 */
	public function publish()
	{
		// CSRF prevention
		if ($this->csrfProtection)
		{
			$this->_csrfProtection();
		}

		$this->approveposts();
	}

	/**
	 * publish.
	 *
	 * @return	void
	 */
	public function unpublish()
	{
		// CSRF prevention
		if ($this->csrfProtection)
		{
			$this->_csrfProtection();
		}

		$this->cancelposts();
	}

	/**
	 * approveposts.
	 *
	 * @return	void
	 */
	protected function approveposts()
	{
		$model = $this->getThisModel();
		if (!$model->getId())
		{
			$model->setIDsFromRequest();
		}

		$status = $model->approve();

		// Check if i'm using an AJAX call, in this case there is no need to redirect
		$format = FOFInput::getString('format', '', $this->input);
		if ($format == 'json')
		{
			echo json_encode($status);
			return;
		}

		// Redirect
		if ($customURL = FOFInput::getString('returnurl', '', $this->input))
		{
			$customURL = base64_decode($customURL);
		}

		$url = !empty($customURL) ? $customURL : 'index.php?option=' . $this->component . '&view=' . FOFInflector::pluralize($this->view);
		if (!$status)
		{
			$this->setRedirect($url, $model->getError(), 'error');
		}
		else
		{
			$this->setRedirect($url);
		}
		return;
	}

	/**
	 * cancelposts.
	 *
	 * @return	void
	 */
	protected function cancelposts()
	{
		$model = $this->getThisModel();
		if (!$model->getId())
		{
			$model->setIDsFromRequest();
		}

		$status = $model->cancel();

		// Check if i'm using an AJAX call, in this case there is no need to redirect
		$format = FOFInput::getString('format', '', $this->input);
		if ($format == 'json')
		{
			echo json_encode($status);
			return;
		}

		// Redirect
		if ($customURL = FOFInput::getString('returnurl', '', $this->input))
		{
			$customURL = base64_decode($customURL);
		}

		$url = !empty($customURL) ? $customURL : 'index.php?option=' . $this->component . '&view=' . FOFInflector::pluralize($this->view);
		if (!$status)
		{
			$this->setRedirect($url, $model->getError(), 'error');
		}
		else
		{
			$this->setRedirect($url);
		}
		return;
	}

}
