<?php

/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - AutoTweetNG posts content to social channels (Twitter, Facebook, LinkedIn, etc).
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2012 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

// Base class for extension plugins for AutoTweet

jimport('joomla.database.table');
jimport('joomla.error.error');
jimport('joomla.plugin.plugin');

include_once 'autotweetplugin.php';

/**
 * plgAutotweetBase
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
 */
abstract class PlgAutotweetBase extends JPlugin implements IAutotweetPlugin
{

	protected $pluginParams = null;

	protected $autopublish = true;

	protected $show_url;

	/**
	 * plgAutotweetBase
	 *
	 * @param   string  &$subject  Param
	 * @param   object  $params    Param
	 *
	 * @since	1.5
	 */
	public function plgAutotweetBase(&$subject, $params)
	{
		parent::__construct($subject, $params);

		// Load FOF
		if (!defined('FOF_INCLUDED'))
		{
			include_once JPATH_LIBRARIES . '/fof/include.php';
		}
		if (!defined('FOF_INCLUDED'))
		{
			JError::raiseError(
			'500',
			'Your AutoTweetNG installation is broken; please re-install.
			Alternatively, extract the installation archive and copy the fof directory inside your site\'s libraries directory.');
		}

		defined('JPATH_AUTOTWEET') || define('JPATH_AUTOTWEET', JPATH_ADMINISTRATOR . '/components/com_autotweet');
		defined('JPATH_AUTOTWEET_HELPERS') || define('JPATH_AUTOTWEET_HELPERS', JPATH_AUTOTWEET . '/helpers');

		JLoader::register('CparamsHelper', JPATH_AUTOTWEET_HELPERS . '/cparams.php');

		JLoader::register('AutotweetPostHelper', JPATH_AUTOTWEET_HELPERS . '/autotweetposthelper.php');
		JLoader::register('AutotweetBaseHelper', JPATH_AUTOTWEET_HELPERS . '/autotweetbasehelper.php');
		JLoader::register('RouteHelp', JPATH_AUTOTWEET_HELPERS . '/route.php');
		JLoader::register('AutotweetLog', JPATH_AUTOTWEET_HELPERS . '/autotweetlog.php');

		// Load component language file for use with plugin
		$jlang = JFactory::getLanguage();
		$jlang->load('com_autotweet');

		// Since Joomla 1.6 params can be used directly without creating a JParameter object
		$this->pluginParams = $this->params;

		if ((int) $this->pluginParams->get('autopublish', 1))
		{
			$this->autopublish = true;
		}
		else
		{
			$this->autopublish = false;
		}

		$surl = (int) $this->pluginParams->get('show_url', 2);
		if (2 == $surl)
		{
			$this->show_url = AutotweetPostHelper::SHOWURL_END;
		}
		elseif (1 == $surl)
		{
			$this->show_url = AutotweetPostHelper::SHOWURL_BEGINNING;
		}
		else
		{
			$this->show_url = AutotweetPostHelper::SHOWURL_OFF;
		}
	}

	/**
	 * Returns publish mode for plugin (default is true, so this works also for plugin without autopublish option).
	 *
	 * @return	bool	true, if autopublishing is enabled for plugin
	 */
	public function isAutopublish()
	{
		return $this->autopublish;
	}

	/**
	 * Returns url mode for plugin.
	 *
	 * @return	int	urlmode (0 =  no url, 1 = show at the beginning, 2 = show at the end of message)
	 */
	public function getShowUrlMode()
	{
		return $this->show_url;
	}

	/**
	 * Queues a message for posting over AutoTweet.
	 * typeinfo:	only needed when 2 different types of messages are returned (see Kunena plugin)
	 *
	 * @param   int     $id              Param
	 * @param   date    $publish_up      Param
	 * @param   string  $description     Param
	 * @param   int     $typeinfo        Param
	 * @param   string  $url             Param
	 * @param   string  $image_url       Param
	 * @param   object  &$native_object  Param
	 *
	 * @return	boolean		true, if message is queued for posting
	 *
	 * @since	1.5
	 */
	protected function postStatusMessage($id, $publish_up, $description, $typeinfo = 0, $url = '', $image_url = '', &$native_object = null)
	{
		$plug_id = $this->_name;

		$helper = AutotweetPostHelper::getInstance();
		$result = $helper->queueMessage($id, $plug_id, $publish_up, $description, $typeinfo, $url, $image_url, $native_object);

		return $result;
	}

	/**
	 * check type and range of textcount parameter, and correct if needed
	 *
	 * @param   int  $textcount  Param.
	 *
	 * @return	int
	 *
	 * @since	1.5
	 */
	protected function getTextcount($textcount)
	{
		return AutotweetBaseHelper::getTextcount($textcount);
	}

	/**
	 * Use title or text as twitter message
	 *
	 * @param   bool    $usetext    Param.
	 * @param   int     $textcount  Param.
	 * @param   string  $title      Param.
	 * @param   string  $text       Param.
	 *
	 * @return	int
	 *
	 * @since	1.5
	 */
	protected function getMessagetext($usetext, $textcount, $title, $text)
	{
		return AutotweetBaseHelper::getMessagetext($usetext, $textcount, $title, $text);
	}

	/**
	 * Replaces spaces for hashtags
	 *
	 * @param   string  $word  Param.
	 *
	 * @return	string
	 *
	 * @since	1.5
	 */
	protected function getAsHashtag($word)
	{
		return AutotweetBaseHelper::getAsHashtag($word);
	}

	/**
	 * Returns hashtags from comma sperated string (metakey field)
	 *
	 * @param   string  $metakey  Param.
	 * @param   int     $count    Param.
	 *
	 * @return	array
	 *
	 * @since	1.5
	 */
	protected function getHashtags($metakey, $count = 1)
	{
		return AutotweetBaseHelper::getHashtags($metakey, $count);
	}

	/**
	 * Add static text / hashtags to message
	 *
	 * @param   int     $textpos     Param.
	 * @param   string  $text        Param.
	 * @param   string  $statictext  Param.
	 *
	 * @return	string
	 *
	 * @since	1.5
	 */
	protected function addStatictext($textpos, $text, $statictext)
	{
		return AutotweetBaseHelper::addStatictext($textpos, $text, $statictext);
	}

	/**
	 * Apply the text pattern in the data array
	 *
	 * @param   string  $pattern  Param.
	 * @param   array   &$data    Param.
	 *
	 * @return	string
	 *
	 * @since	1.5
	 */
	protected function applyTextPattern($pattern, &$data)
	{
		return AutotweetBaseHelper::applyTextPattern($pattern, $data);
	}

	/**
	 * Add category / section to message text
	 *
	 * @param   int     $show      Param
	 * @param   int     $section   Param
	 * @param   int     $category  Param
	 * @param   string  $text      Param
	 * @param   bool    $add_hash  Param
	 *
	 * @return	string
	 *
	 * @since	1.5
	 */
	protected function addCatsec($show, $section, $category, $text, $add_hash = false)
	{
		return AutotweetBaseHelper::addCatsec($show, $section, $category, $text, $add_hash);
	}

	/**
	 * Special implementation to ad multiple categories
	 *
	 * @param   int     $show        Param
	 * @param   array   $categories  Param
	 * @param   string  $text        Param
	 * @param   bool    $add_hash    Param
	 *
	 * @return	string
	 *
	 * @since	1.5
	 */
	protected function addCategories($show, $categories, $text, $add_hash = false)
	{
		return AutotweetBaseHelper::addCategories($show, $categories, $text, $add_hash);
	}

	/**
	 * Database helpers: returns the next free id for the table
	 *
	 * @param   object  $table  Param
	 *
	 * @return	string
	 *
	 * @since	1.5
	 */
	protected function getID($table)
	{
		return AutotweetBaseHelper::getID($table);
	}

	/**
	 * Better implementation to handle multiple menu entry for component (multiple itemids)
	 *
	 * @param   object  $comp_name  Param
	 * @param   object  $needles    Param
	 * @param   object  $q_view     Param
	 *
	 * @return	int
	 *
	 * @since	1.5
	 */
	protected function getItemid($comp_name, $needles, $q_view = 'view')
	{
		return AutotweetBaseHelper::getItemid($comp_name, $needles, $q_view);
	}

	/**
	 * getData
	 *
	 * @param   string  $id        Param.
	 * @param   string  $typeinfo  Param.
	 *
	 * @return	array
	 *
	 * @since	1.5
	 */
	public function getData($id, $typeinfo)
	{
		JError::raiseWarning('5', 'AutoTweet NG Plugin - getData not implemented by plugin.');
	}

	/**
	 * getExtendedData - To return the data array from the native object
	 *
	 * @param   string  $id              Param.
	 * @param   string  $typeinfo        Param.
	 * @param   string  &$native_object  Param.
	 *
	 * @return	array
	 *
	 * @since	1.5
	 */

	/*
	public function getExtendedData($id, $typeinfo, &$native_object)
	{
		JError::raiseWarning('5', 'AutoTweet NG Plugin - getExtendedData not implemented by plugin.');
	}
	*/

	/**
	 * getImageFromText
	 *
	 * @param   string  &$img_parsetext  Param.
	 *
	 * @return	string
	 *
	 * @since	1.5
	 */
	protected function getImageFromText(&$img_parsetext)
	{
		$image = '';
		if (class_exists('DOMDocument'))
		{
			$doc = new DOMDocument;
			@$doc->loadHTML($img_parsetext);
			$imgtags = $doc->getElementsByTagName('img');
			if (0 < $imgtags->length)
			{
				$imgtag = $imgtags->item(0);
				$image = $imgtag->getAttribute('src');
			}
		}
		else
		{
			$logger = AutotweetLog::getInstance();
			$logger->log(JLog::WARNING, 'Class DOMDocument not found in autotweetcontent.php - text not parsed for image');
		}

		return $image;
	}

	/**
	 * checkIncludedCategoryFilter.
	 *
	 * @param   array  $catids  Param.
	 *
* @return	void
	 *
	 * @since	1.5
	 */
	protected function checkIncludedCategoryFilter($catids)
	{
		// Handling for multiple categories
		$isIncludeFilter = true;
		if (!empty($catids) && !empty($this->categories))
		{
			foreach ($catids as $catid)
			{
				if (in_array($catid, $this->categories))
				{
					$isIncludeFilter = false;
					break;
				}
			}
		}
		$checkIncludeFilter = (empty($this->categories) || empty($catids) || !$isIncludeFilter);
		return $checkIncludeFilter;
	}

	/**
	 * checkExcludedCategoryFilter.
	 *
	 * @param   array  $catids  Param.
	 *
* @return	void
	 *
	 * @since	1.5
	 */
	protected function checkExcludedCategoryFilter($catids)
	{
		$isExcludeFilter = false;
		if (!empty($catids) && !empty($this->excluded_categories))
		{
			foreach ($catids as $catid)
			{
				if (in_array($catid, $this->excluded_categories))
				{
					$isExcludeFilter = true;
					break;
				}
			}
		}
		$checkExcludeFilter = (empty($this->excluded_categories) || !$isExcludeFilter);
		return $checkExcludeFilter;
	}

	/**
	 * checkAccessLevelFilter.
	 *
	 * @param   int  $accesslevel  Param.
	 *
* @return	void
	 *
	 * @since	1.5
	 */
	protected function checkAccessLevelFilter($accesslevel)
	{
		return (empty($this->accesslevels) || in_array($accesslevel, $this->accesslevels));
	}

	/**
	 * getAuthorUsername.
	 *
	 * @param   int  $uid  Param.
	 *
	 * @return	string
	 *
	 * @since	1.5
	 */
	protected function getAuthorUsername($uid)
	{
		$user = JFactory::getUser($uid);
		return $user->username;
	}

	/**
	 * getArticleAuthor
	 *
	 * @param   string  &$article  Param
	 *
	 * @return	string
	 *
	 * @since	1.5
	 */
	protected function getArticleAuthor(&$article)
	{
		if (('0000-00-00 00:00:00' == $article->modified) || (empty($article->modified)))
		{
			$uid = $article->created_by;
		}
		else
		{
			$uid = $article->modified_by;
		}

		$user = JFactory::getUser($uid);
		return $user->username;
	}

	/**
	 * disablePostOld
	 *
	 * @param   string  $plugin  Param.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	protected function disablePostOld($plugin = 'autotweetcontent')
	{
		// Get plugin id
		$table = '#__extensions';

		$db = JFactory::getDBO();

		$query = 'SELECT ' . $db->quoteName('extension_id') . ' FROM ' . $db->quoteName($table)
		. ' WHERE ' . $db->quoteName('element') . ' = ' . $db->Quote($plugin)
		. ' AND ' . $db->quoteName('type') . ' = ' . $db->Quote('plugin');

		$db->setQuery($query);
		$id = (int) $db->loadResult();

		// Save parameter
		$this->pluginParams->set('post_old', 0);
		$table = JTable::getInstance('extension');
		$table->load($id);

		$table->params = $this->pluginParams->toString();

		if (!$table->store())
		{
			JError::raiseWarning(500, 'Can not save parameter for AutoTweet ' . $plugin . 'Plugin: ' . $table->getError());
		}
	}

	/**
	 * executeContentPolling
	 *
	 * @return	boolean
	 *
	 * @since	1.6
	 */
	protected function executeContentPolling()
	{
		// Get the data for this plugin
		$plugin = 'content';
		$automat = FOFTable::getAnInstance('Automator', 'AutoTweetTable');

		if ($automat)
		{
			$loaded = $automat->load(array('plugin' => $plugin));
		}
		else
		{
			$loaded = false;
		}

		if ($loaded)
		{
			$last = JFactory::getDate($automat->lastexec);
		}
		else
		{
			$last = JFactory::getDate();
		}

		$now = JFactory::getDate();
		$diff = $now->toUnix() - $last->toUnix();

		if ($diff > $this->interval)
		{
			$entry = array(
					'plugin' => $plugin,
					'lastexec' => $now->toSql()
			);
			$automat->bind($entry);
			$automat->store();

			// 12 hours overlapping, just to be sure - 12 * 3600 - 120 *360

			// 4 hrs aprox
			$check_from = $now->toUnix() - ($this->interval * 90);
			$check_from = JFactory::getDate($check_from);

			// Set date for posts
			$post_old_mode = false;
			if ($this->post_old)
			{
				// Special case: posting for old articles is enabled
				$post_old_mode = true;
				$last_post = JFactory::getDate($this->post_old_date);

				// Disable old article posting
				$this->disablePostOld();
			}
			else
			{
				$last_post = $check_from;
			}

			// Get new and changed articles form db
			$table_content = '#__content';

			// Get articles only when they are not in the queue and not in the message log for time horizon
			$db = JFactory::getDBO();
			$query = $this->getPollingQuery('AutotweetContent', $table_content, $last_post);

			$db->setQuery($query);
			$articles = $db->loadObjectList();

			$logger = AutotweetLog::getInstance();
			$logger->log(JLog::INFO, 'PollingQuery: ' . $query);
			$logger->log(JLog::INFO, 'PollingQuery: found ' . count($articles));

			// Post articles
			foreach ($articles as $article)
			{
				if (('0000-00-00 00:00:00' == $article->modified) || (empty($article->modified)))
				{
					$isNew = true;
				}
				else
				{
					$isNew = false;
				}

				if (($isNew || $this->post_modified || $post_old_mode)
					&& ((0 < $article->featured) || (0 == $this->post_featured_only))
					&& (1 == $article->state))
				{
					$this->postArticle($article);
				}
			}
		}
	}

	/**
	 * getPollingQuery
	 *
	 * @param   string  $plugin         Param
	 * @param   string  $table_content  Param
	 * @param   JDate   $check_from     Param
	 *
	 * @return	string
	 *
	 * @since	1.6
	 */
	protected function getPollingQuery($plugin, $table_content, $check_from)
	{
		// Calculate date for interval
		$now = JFactory::getDate();
		$check_until = $now->toUnix();
		$check_until = $check_until - 5 * 60;
		$check_until = JFactory::getDate($check_until);

		$table_posts = '#__autotweet_posts';
		$table_requests = '#__autotweet_requests';

		$db = JFactory::getDBO();
		$query = 'SELECT * FROM ' . $db->quoteName($table_content)
			. ' WHERE (('
			. $db->quoteName('modified') . ' > ' . $db->Quote($check_from) . ' AND '
			. $db->quoteName('modified') . ' < ' . $db->Quote($check_until)
			. ') OR ('
			. $db->quoteName('created') . ' > ' . $db->Quote($check_from) . ' AND '
			. $db->quoteName('created') . ' < ' . $db->Quote($check_until) .
			'))'

			. ' AND ' . $db->quoteName('id') . ' NOT IN (SELECT ' . $db->quoteName('ref_id') . ' FROM ' . $db->quoteName($table_requests)
				. ' WHERE ' . $db->quoteName('plugin') . ' = ' . $db->Quote($plugin) . ')'

			. ' AND ' . $db->quoteName('id') . ' NOT IN (SELECT ' . $db->quoteName('ref_id') . ' FROM ' . $db->quoteName($table_posts)
				. ' WHERE ' . $db->quoteName('plugin') . ' = ' . $db->Quote($plugin) . ')';

		if ((isset($this->categories)) && (is_array($this->categories)))
		{
			$categories = array_filter($this->categories);
			if (count($categories) > 0)
			{
				$query = $query . ' AND ' . $db->quoteName('catid') . ' IN (' . implode(',', $categories) . ')';
			}
		}

		if ((isset($this->excluded_categories)) && (is_array($this->excluded_categories)))
		{
			$categories = array_filter($this->excluded_categories);
			if (count($categories) > 0)
			{
				$query = $query . ' AND ' . $db->quoteName('catid') . ' NOT IN (' . implode(',', $categories) . ')';
			}
		}

		return $query;
	}

	/**
	 * getContentCategories
	 *
	 * @param   array  $article_cat  Param.
	 *
	 * @return	array
	 *
	 * @since	1.5
	 */
	protected function getContentCategories($article_cat)
	{
		$cat_ids = array();
		$cat_names = array();
		$cat_alias = array();

		$row = JTable::getInstance('category');

		// JomSocial Conflict Category ?
		if (!method_exists($row, 'load'))
		{
			if (version_compare(JVERSION, '3.0', 'ge'))
			{
				include_once JPATH_SITE . '/libraries/legacy/table/category.php';
			}
			else
			{
				include_once JPATH_SITE . '/libraries/joomla/database/table/category.php';
			}
			$db = JFactory::getDbo();
			$row = new JTableCategory($db);
		}

		$row->load($article_cat);
		while ($row->parent_id > 0)
		{
			$cat_ids[] = $row->id;
			$cat_names[] = $row->title;
			$cat_alias[] = $row->alias;

			$row->load($row->parent_id);
		}

		return array($cat_ids, $cat_names, $cat_alias);
	}

	/**
	 * getImageFromTextWithBrackets
	 *
	 * @param   string  &$img_parsetext  Param.
	 *
	 * @return	string
	 *
	 * @since	1.5
	 */
	protected function getImageFromTextWithBrackets(&$img_parsetext)
	{
		$pattern = '/\[img\]([^\[]+)\[\/img\]/is';
		if (preg_match($pattern, $img_parsetext, $match))
		{
			return $match[1];
		}
		return null;
	}

	/**
	 * cleanImagesWithBrackets
	 *
	 * @param   string  &$img_parsetext  Param.
	 *
	 * @return	string
	 *
	 * @since	1.5
	 */
	protected function cleanImagesWithBrackets(&$img_parsetext)
	{
		$pattern = '/\[img\]([^\[]+)\[\/img\]/is';
		$text = preg_replace($pattern, '', $img_parsetext);
		return $text;
	}

}
