<?php

/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - AutoTweetNG posts content to social channels (Twitter, Facebook, LinkedIn, etc).
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2012 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

/**
 * Helper functions for AutoTweet plugins.
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
 */
abstract class AutotweetBaseHelper
{

	/**
	 * AutotweetBaseHelper
	 *
	 * @since	1.0
	 */
	private function AutotweetBaseHelper()
	{
		// Static class
	}

	/**
	 * cleanText
	 *
	 * @param   string  $text  Param
	 *
	 * @return	string
	 *
	 * @since	1.0
	 */
	private static function cleanText($text)
	{
		return trim(strip_tags(JString::str_ireplace('&nbsp;', ' ', htmlspecialchars_decode($text))));
	}

	// Check type and range of textcount parameter, and correct if needed

	/**
	 * getTextcount
	 *
	 * @param   string  $textcount  Param
	 *
	 * @return	int
	 *
	 * @since	1.0
	 */
	public static function getTextcount($textcount)
	{
		if (is_numeric($textcount) || ('' == $textcount))
		{
			$textcount = (int) $textcount;
			if ($textcount > 400)
			{
				$textcount = 400;
			}
			elseif ($textcount < 10)
			{
				$textcount = 10;
			}
		}
		else
		{
			$textcount = 100;
		}

		return $textcount;
	}

	/**
	 * Use title or text as twitter message
	 *
	 * @param   bool    $usetext    Param
	 * @param   int     $textcount  Param
	 * @param   string  $title      Param
	 * @param   string  $text       Param
	 *
	 * @return	string
	 *
	 * @since	1.0
	 */
	public static function getMessagetext($usetext, $textcount, $title, $text)
	{
		$message = '';

		switch ($usetext)
		{
			// Show title only
			case 0:
				$message = $title;
				break;

			// Show text only
			case 1:
				if ('' != $text)
				{
					$message = $text;
				}
				else
				{
					$message = $title;
				}
				break;

			// Show title and text
			case 2:
				if ('' != $text)
				{
					$message = $title . ': ' . $text;
				}
				else
				{
					$message = $title;
				}
				break;
			default:
				$message = $title;
		}

		return JString::substr(self::cleanText($message), 0, $textcount);
	}

	/**
	 * Replaces spaces for hashtags
	 *
	 * @param   string  $word  Param
	 *
	 * @return	string
	 *
	 * @since	1.0
	 */
	public static function getAsHashtag($word)
	{
		if ('' != $word)
		{
			$word = self::cleanText($word);
			$word = str_ireplace(' ', '', $word);
			$word = str_ireplace('-', '', $word);
			$hash = '#' . $word;
		}
		else
		{
			$hash = '';
		}

		return $hash;
	}

	/**
	 * Returns hashtags from comma sperated string (metakey field)
	 *
	 * @param   string  $metakey  Param
	 * @param   int     $count    Param
	 *
	 * @return	string
	 *
	 * @since	1.0
	 */
	public static function getHashtags($metakey, $count = 1)
	{
		$hashtags = '';

		if (!empty($metakey))
		{
			$i = 0;
			$words = explode(',', $metakey);
			foreach ($words as $word)
			{
				$i++;
				if ($i > $count)
				{
					break;
				}
				if ('' != $hashtags)
				{
					$hashtags .= ' ';
				}
				$hashtags .= self::getAsHashtag($word);
			}
		}

		return $hashtags;
	}

	/**
	 * Add static text / hashtags to message
	 *
	 * @param   int     $textpos     Param
	 * @param   string  $text        Param
	 * @param   string  $statictext  Param
	 *
	 * @return	string
	 *
	 * @since	1.0
	 */
	public static function addStatictext($textpos, $text, $statictext)
	{
		if (AutotweetPostHelper::STATICTEXT_BEGINNING == $textpos)
		{
			$textpos = 1;
		}
		elseif (AutotweetPostHelper::STATICTEXT_END == $textpos)
		{
			$textpos = 2;
		}

		switch ($textpos)
		{
			// Dont use static_text, use original text
			case 0:
				$result_text = $text;
				break;

			// Show static text at the beginning of message text
			case 1:
				$result_text = $statictext;
				if (!empty($text))
				{
					$result_text .= ' ' . $text;
				}
				break;

			// Show static text at the end of message text
			case 2:
				$result_text = $text;
				if (!empty($result_text))
				{
					$result_text .= ' ';
				}
				$result_text .= $statictext;
				break;
			default:
				$result_text = $text;
		}

		return $result_text;
	}

	/**
	 * Apply the text pattern in the data array
	 *
	 * @param   string  $pattern  Param
	 * @param   array   &$post    Param
	 *
	 * @return	void
	 *
	 * @since	1.0
	 */
	public function applyTextPattern($pattern, &$post)
	{
		$pattern = str_replace('[author]', $post->author, $pattern);
		$pattern = str_replace('[fulltext]', $post->fulltext, $pattern);
		if (strpos($pattern, '[hashtags]') !== false)
		{
			$pattern = str_replace('[hashtags]', $post->hashtags, $pattern);
			$post->hashtags = '';
		}
		$pattern = str_replace('[text]', $post->text, $pattern);
		$pattern = str_replace('[message]', $post->message, $pattern);
		$pattern = str_replace('[title]', $post->title, $pattern);

		$cats = $post->cat_names;
		$count_cats = count($cats);
		if (strpos($pattern, '[maincat]') !== false)
		{
			if ($count_cats > 0)
			{
				$maincat = self::hashtize($cats[0]);
				$pattern = str_replace('[maincat]', $maincat, $pattern);
			}
		}
		if (strpos($pattern, '[lastcat]') !== false)
		{
			if ($count_cats > 0)
			{
				$lastcat = self::hashtize($cats[$count_cats - 1]);
				$pattern = str_replace('[lastcat]', $lastcat, $pattern);
			}
		}
		if (strpos($pattern, '[allcats]') !== false)
		{
			if ($count_cats > 0)
			{
				array_walk($cats, 'AutotweetBaseHelper::hashtize');
				$allcats = join(' ', $cats);
				$pattern = str_replace('[allcats]', $allcats, $pattern);
			}
		}

		$post->message = $pattern;
	}

	/**
	 * Hashtize
	 *
	 * @param   string  $text  Param
	 *
	 * @return	string
	 *
	 * @since	1.0
	 */
	public static function hashtize($text)
	{
		return '#' . str_replace(' ', '', ucwords($text));
	}

	/**
	 * Add category / section to message text
	 *
	 * @param   bool    $show      Param
	 * @param   int     $section   Param
	 * @param   int     $category  Param
	 * @param   string  $text      Param
	 * @param   bool    $add_hash  Param
	 *
	 * @return	array
	 *
	 * @since	1.0
	 */
	public static function addCatsec($show, $section, $category, $text, $add_hash = false)
	{
		$result_text = $text;
		$hashtags = '';

		if ($add_hash)
		{
			// Show as hashtags
			$section = self::getAsHashtag($section);
			$category = self::getAsHashtag($category);

			switch ($show)
			{
				// Do nothing, use original text
				case 0:
					break;

				// Show section only
				case 1:
					$hashtags = $section;
					break;

				// Show section and category
				case 2:
					$hashtags = $section . ' ' . $category;
					break;

				// Show category only (new feature since 3.0 stable)
				case 3:
					$hashtags = $category;
					break;
			}
		}
		else
		{
			switch ($show)
			{
				// Show as pretext (part of message)
				// Do nothing, use original text
				case 0:
					break;

				// Show section only
				case 1:
					$result_text = $section . ': ' . $text;
					break;

				// Show section and category
				case 2:
					$result_text = $section . '/' . $category . ': ' . $text;
					break;

				// Show category only (new feature since 3.0 stable)
				case 3:
					$result_text = $category . ': ' . $text;
					break;
			}
		}

		$result = array(
			'text' => $result_text,
			'hashtags' => $hashtags
		);

		return $result;
	}

	/**
	 * Special implementation to ad multiple categories
	 *
	 * @param   bool    $show        Param
	 * @param   int     $categories  Param
	 * @param   string  $text        Param
	 * @param   bool    $add_hash    Param
	 *
	 * @return	array
	 *
	 * @since	1.0
	 */
	public static function addCategories($show, $categories, $text, $add_hash = false)
	{
		$result_text = $text;
		$hashtags = '';

		if (!empty($categories))
		{
			if ($add_hash)
			{
				switch ($show)
				{
					// Do nothing, use original text
					case 0:
						break;

					// Show first category only
					case 1:
						$hashtags = self::getAsHashtag($categories[0]);
						break;

					// Show all categories
					case 2:
						$hashtags = self::getHashtags(implode(',', $categories), count($categories));
						break;
				}
			}
			else
			{
				switch ($show)
				{
					// Do nothing, use original text
					case 0:
						break;

					// Show first category only
					case 1:
						$result_text = $categories[0] . ': ' . $text;
						break;

					// Show all categories
					case 2:
						$result_text = trim(implode('/', $categories)) . ': ' . $text;
						break;
				}
			}
		}

		$result = array(
			'text' => $result_text,
			'hashtags' => $hashtags
		);

		return $result;
	}

	/**
	 * Database helpers: returns the next free id for the table
	 *
	 * @param   string  $table  Param
	 *
	 * @return	int
	 *
	 * @since	1.0
	 */
	public static function getID($table)
	{
		$db = JFactory::getDBO();

		$prefix = $db->getPrefix();
		$table = str_replace('#__', $prefix, $table);

		$query = 'SHOW TABLE STATUS LIKE ' . $db->Quote($table);
		$db->setQuery($query);
		$result = $db->loadAssoc();

		$next_key = (int) $result['Auto_increment'];

		return $next_key;
	}

	/**
	 * Better implementation to handle multiple menu entry for component (multiple itemids)
	 *
	 * @param   string  $comp_name  Param
	 * @param   array   $needles    Param
	 * @param   string  $q_view     Param
	 *
	 * @return	int
	 *
	 * @since	1.0
	 */
	public static function getItemid($comp_name, $needles, $q_view = 'view')
	{
		$component = JComponentHelper::getComponent($comp_name);

		$menus = JApplication::getMenu('site', array());
		$items = $menus->getItems('component_id', $component->id);

		$match = null;

		foreach ($needles as $needle => $id)
		{
			foreach ($items as $item)
			{
				if ((@$item->query[$q_view] == $needle) && (@$item->query['id'] == $id))
				{
					$match = $item;
					break;
				}
			}

			if (isset($match))
			{
				break;
			}
		}

		// Defaults if no item is found
		if (!isset($match))
		{
			// Get first public itemid
			foreach ($items as $item)
			{
				if (0 == (int) $item->access)
				{
					$match = $item;
					break;
				}
			}

			// Last chance: get first itemid (also when private)
			if (!isset($match) && !empty($items))
			{
				$match = $items[0];
			}
		}

		// Set id if item is found
		if (isset($match))
		{
			$match = $match->id;
		}

		return $match;
	}

}
