<?php

/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - AutoTweetNG posts content to social channels (Twitter, Facebook, LinkedIn, etc).
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2012 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

/**
 * Logger for AutoTweet debug info.
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
 */
class AutotweetLog
{
	const LOG_LEVEL_OFF = 0;
	const LOG_LEVEL_ERRORS = 1;
	const LOG_LEVEL_ERRORS_WARNS = 2;
	const LOG_LEVEL_ALL = 3;

	const LOG_MODE_LOGFILE = 0;
	const LOG_MODE_LOGFILE_SCREEN = 1;

	// 0 = off, 1 = errors only, 2 = errors and warnings, 3 = all
	protected $level	= 0;

	// 0 = to logfile only, 1 = to logfile and on screen
	protected $mode		= 0;

	// Joomla JLog
	protected $logger	= null;

	private static $_instance	= null;

	/**
	 * AutotweetLog
	 *
	 * @param   int  $log_level  Param
	 * @param   int  $log_mode   Param
	 *
	 * @since	1.5
	 */
	protected function AutotweetLog($log_level, $log_mode)
	{
		$this->level	= (int) $log_level;
		$this->mode		= (int) $log_mode;

		if ($log_level)
		{
			$config = array('text_file' => 'autotweet.log');
			if (version_compare(JVERSION, '3.0.0', 'ge'))
			{
				jimport('joomla.log.logger.formattedtext');
			}
			else
			{
				jimport('joomla.log.loggers.formattedtext');
			}

			if (version_compare(JVERSION, '3.0.0', 'ge'))
			{
				$this->logger = new JLogLoggerFormattedtext($config);
			}
			else
			{
				$this->logger = new JLoggerFormattedText($config);
			}
		}
	}

	/**
	 * getInstance
	 *
	 * @param   int  $log_level  Param
	 * @param   int  $log_mode   Param
	 *
	 * @return	object
	 *
	 * @since	1.5
	 */
	public static function &getInstance($log_level = null, $log_mode = null)
	{
		if (!self::$_instance)
		{
			if (!$log_level)
			{
				$log_level = CparamsHelper::getParam('log_level', 0);
			}
			if (!$log_mode)
			{
				$log_mode = CparamsHelper::getParam('log_mode', 0);
			}

			self::$_instance = new AutotweetLog($log_level, $log_mode);
		}

		return self::$_instance;
	}

	/**
	 * log
	 *
	 * @param   string  $status   Param
	 * @param   string  $comment  Param
	 *
	 * @return	object
	 *
	 * @since	1.5
	 */
	public function log($status, $comment)
	{
		$log_result = false;

		if ($this->level)
		{
			// Logging is enabled
			if (((JLog::INFO == $status) && (3 <= (int) $this->level))
				|| ((JLog::WARNING == $status) && (2 <= (int) $this->level))
				|| ((JLog::ERROR == $status) && (1 <= (int) $this->level)))
			{
				if (empty($this->logger))
				{
					JError::raiseWarning($this->level, 'AutoTweetNG Log - Warning: Logger not initialized, entry not written to logfile.');
				}
				else
				{
					$entry = new JLogEntry($comment, $status);
					$this->logger->addEntry($entry);
				}
			}

			if ($this->mode == self::LOG_MODE_LOGFILE_SCREEN)
			{
				// Enhanced logging enabled: display messages also on screen
				$message = 'AutoTweetNG debug log - ' . $status . ': ' . htmlspecialchars($comment);

				if ((JLog::INFO == $status) && (3 <= (int) $this->level))
				{
					JError::raiseNotice($this->level, $message);
				}
				elseif (((JLog::WARNING == $status) && (2 <= (int) $this->level)) || ((JLog::ERROR == $status) && (1 <= (int) $this->level)))
				{
					JError::raiseWarning($this->level, $message);
				}
			}
		}

		return $log_result;
	}

	/**
	 * isLogging
	 *
	 * @return	boolean
	 *
	 * @since	1.5
	 */
	public function isLogging()
	{
		return $this->level;
	}

}
