<?php

/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - AutoTweetNG posts content to social channels (Twitter, Facebook, LinkedIn, etc).
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2012 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

/**
 * FbAppHelper class.
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
*/
class FbAppHelper
{
	protected $app_id;

	protected $secret;

	// Facebook long-lived oauth token for user
	protected $access_token;

	private $_facebook = null;

	/**
	 * FbAppHelper.
	 *
	 * @param   string  $app_id        Params.
	 * @param   string  $secret        Params.
	 * @param   string  $access_token  Params.
	 *
	 * @since	1.5
	 */
	public function FbAppHelper($app_id, $secret, $access_token)
	{
		require_once dirname(__FILE__) . '/facebook-php-sdk/facebook.php';

		$this->app_id = $app_id;
		$this->secret = $secret;
		$this->access_token = $access_token;
	}

	/**
	 * login.
	 *
	 * @return	object
	 *
	 * @since	1.5
	 */
	public function login()
	{
		if (!$this->_facebook)
		{
			$this->_facebook = new facebookphpsdk\Facebook(array('appId' => $this->app_id, 'secret' => $this->secret, 'cookie' => true,));
			$this->_facebook->setAccessToken($this->access_token);
		}

		return $this->_facebook;
	}

	/**
	 * verify.
	 *
	 * @return	boolean
	 *
	 * @since	1.5
	 */
	public function verify()
	{
		$result = null;

		try
		{
			if (empty($this->access_token))
			{
				$result = array(false, 'Facebook Token not entered.');
			}
			else
			{
				$perm_result = $this->_facebook->api("/me/permissions", array('access_token' => $this->access_token,));
				$permissions = $perm_result['data'][0];
				$needed_perms = array('publish_stream', 'manage_pages', 'user_events', 'user_groups', 'create_event');
				$result_string = '';

				foreach ($needed_perms as $needed_perm)
				{
					if (!isset($permissions[$needed_perm]) || ($permissions[$needed_perm] != 1))
					{
						if (!empty($result_string))
						{
							$result_string .= ',';
						}
						$result_string .= $needed_perm;
					}
				}

				if (!empty($result_string))
				{
					$result = array(false, 'Permissions ' . $result_string . ' not granted.');
				}
				else
				{
					$result = array(true, 'OK');
				}
			}
		}
		catch (facebookphpsdk\FacebookApiException $e)
		{
			// There is no permission check for pages, groups etc. at the moment (for profile only)
			if (false === strpos($e->__toString(), 'GraphMethodException: Unsupported get request'))
			{
				$result = array(false, $e->__toString());
			}
			else
			{
				$result = array(true, 'OK');
			}
		}

		return $result;
	}

	/**
	 * getUser.
	 *
	 * @return	object
	 *
	 * @since	1.5
	 */
	public function getUser()
	{
		$user = $this->_facebook->api("/me");
		return $user;
	}

	/**
	 * _getPagesAsChannel.
	 *
	 * @return	array
	 *
	 * @since	1.5
	 */
	private function _getPagesAsChannel()
	{
		$result = array();

		try
		{
			$pages = $this->_facebook->api("/me/accounts");

			foreach ($pages['data'] as $page)
			{
				$result[] = array(
						'type' => 'Page',
						'id' => $page['id'],
						'name' => $page['name'],
						'access_token' => (isset($page['access_token'])  ? $page['access_token'] : $this->access_token)
				);
			}
		}
		catch (facebookphpsdk\FacebookApiException $e)
		{
			$result[] = array(
					'type' => 'Error',
					'id' => '0',
					'name' => 'Pages ' . $e->__toString(),
					'token' => ''
			);
		}

		return $result;
	}

	/**
	 * _getGroupsAsChannel.
	 *
	 * @return	array
	 *
	 * @since	1.5
	 */
	private function _getGroupsAsChannel()
	{
		$result = array();

		try
		{
			$groups = $this->_facebook->api("/me/groups");

			foreach ($groups['data'] as $group)
			{
				$result[] = array(
						'type' => 'Group',
						'id' => $group['id'],
						'name' => $group['name'],
						'access_token' => (isset($group['access_token'])  ? $group['access_token'] : $this->access_token)
				);
			}
		}
		catch (facebookphpsdk\FacebookApiException $e)
		{
			$result[] = array(
					'type' => 'Error',
					'id' => '0',
					'name' => 'Groups ' . $e->__toString(),
					'token' => ''
			);
		}

		return $result;
	}

	/**
	 * _getEventsAsChannel.
	 *
	 * @return	array
	 *
	 * @since	1.5
	 */
	private function _getEventsAsChannel()
	{
		$result = array();

		try
		{
			$events = $this->_facebook->api("/me/events");

			foreach ($events['data'] as $event)
			{
				$result[] = array(
						'type' => 'Event',
						'id' => $event['id'],
						'name' => $event['name'],
						'access_token' => (isset($event['access_token'])  ? $event['access_token'] : $this->access_token)
				);
			}
		}
		catch (facebookphpsdk\FacebookApiException $e)
		{
			$result[] = array(
					'type' => 'Error',
					'id' => '0',
					'name' => 'Events ' . $e->__toString(),
					'token' => ''
			);
		}

		return $result;
	}

	/**
	 * _getUserAsChannel.
	 *
	 * @return	array
	 *
	 * @since	1.5
	 */
	private function _getUserAsChannel()
	{
		$result = array();

		try
		{
			$user = $this->getUser();

			$result[] = array(
					'type' => 'User',
					'id' => $user['id'],
					'name' => $user['name'],
					'access_token' => (isset($user['access_token'])  ? $user['access_token'] : $this->access_token)
			);
		}
		catch (facebookphpsdk\FacebookApiException $e)
		{
			$result[] = array(
					'type' => 'Error',
					'id' => '0',
					'name' => 'User ' . $e->__toString(),
					'token' => ''
			);
		}
		return $result;
	}

	/**
	 * getChannels.
	 *
	 * @return	array
	 *
	 * @since	1.5
	 */
	public function getChannels()
	{
		$user = $this->_getUserAsChannel();
		$pages = $this->_getPagesAsChannel();
		$groups = $this->_getGroupsAsChannel();
		$events = $this->_getEventsAsChannel();

		return array_merge($user, $pages, $groups, $events);
	}

	/**
	 * getAlbums.
	 *
	 * @param   string  $channelid  Params
	 *
	 * @return	array
	 *
	 * @since	1.5
	 */
	public function getAlbums($channelid)
	{
		$result = array();

		try
		{
			$albums = $this->_facebook->api("/{$channelid}/albums");

			foreach ($albums['data'] as $album)
			{
				$result[] = array(
						'type' => 'Album',
						'id' => $album['id'],
						'name' => $album['name']
				);
			}
		}
		catch (facebookphpsdk\FacebookApiException $e)
		{
			$result[] = array(
					'type' => 'Error',
					'id' => '0',
					'name' => 'Album ' . $e->__toString(),
			);
		}

		return $result;
	}

}
