<?php

/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - AutoTweetNG posts content to social channels (Twitter, Facebook, LinkedIn, etc).
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2012 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

JLoader::import('linkedinbase', dirname(__FILE__));

/**
 * LinkedinChannelHelper class.
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
*/
class LinkedinChannelHelper extends LinkedinBaseChannelHelper
{

	/**
	 * sendMessage.
	 *
	 * @param   string  $message  Params
	 * @param   object  $data     Params
	 *
	 * @return  boolean
	 *
	 * @since   1.5
	 */
	public function sendMessage($message, $data)
	{
		return $this->sendLinkedinMessage(
				$message,
				$data->title,
				$data->fulltext,
				$data->url,
				$data->image_url,
				$this->getMediaMode()
			);
	}

	/**
	 * sendFacebookMessage.
	 *
	 * @param   string  $message     Params
	 * @param   string  $title       Params
	 * @param   string  $text        Params
	 * @param   string  $url         Params
	 * @param   string  $image_url   Params
	 * @param   string  $media_mode  Params
	 *
	 * @return  boolean
	 *
	 * @since   1.5
	 */
	protected function sendLinkedinMessage($message, $title, $text, $url, $image_url, $media_mode)
	{
		$result = null;
		$content = array();

		// Post message and/or media
		switch ($media_mode)
		{
			case 'attachment':
				$post_attach = true;
				$post_msg = false;
				break;
			case 'both':
				$post_msg = true;
				$post_attach = true;
				break;
			case 'message':
			default:
				$post_msg = true;
				$post_attach = false;
		}

		// Media: do also not post when text and image are empty
		if ($post_attach && !empty($title) && !empty($url))
		{
			// Prepare content
			$content['title'] = $title;
			$content['submitted-url'] = $url;
			$content['submitted-image-url'] = $image_url;

			$text = TextHelper::truncString($text, JString::strlen($text), self::MAX_CHARS_TEXT, true);
			$content['description'] = $text;
		}

		// Message
		if ($post_msg)
		{
			$content['comment'] = $message;
		}

		// Default for visibility
		$private = false;

		try
		{
			$api = $this->getApiInstance();
			$response = $api->share('new', $content, $private);
			if ($response['success'] === true)
			{
				$result = array(true, 'OK');
			}
			else
			{
				$msg = $response['info']['http_code'] . ' ' . JText::_('COM_AUTOTWEET_HTTP_ERR_' . $response['info']['http_code']);
				$result = array(false, $msg);
			}
		}
		catch (LinkedInException $e)
		{
			$result = array(false, $e->getMessage());
		}

		return $result;
	}

}
