<?php

/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - AutoTweetNG posts content to social channels (Twitter, Facebook, LinkedIn, etc).
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2012 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

/**
 * TwAppHelper class.
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
*/
class TwAppHelper
{

	/**
	 * TwAppHelper.
	 *
	 * @param   string  $consumer_key         Params.
	 * @param   string  $consumer_secret      Params.
	 * @param   string  $access_token         Params.
	 * @param   string  $access_token_secret  Params.
	 *
	 * @since	1.5
	 */
	public function TwAppHelper($consumer_key, $consumer_secret, $access_token, $access_token_secret)
	{
		require_once dirname(__FILE__) . '/tmhOAuth/tmhOAuth.php';

		$this->consumer_key = $consumer_key;
		$this->consumer_secret = $consumer_secret;
		$this->access_token = $access_token;
		$this->access_token_secret = $access_token_secret;
	}

	/**
	 * login.
	 *
	 * @return	object
	 *
	 * @since	1.5
	 */
	public function login()
	{
		if (!$this->_twitter)
		{
			$this->_twitter = new tmhOAuth\tmhOAuth(
					array('consumer_key' => $this->consumer_key,
							'consumer_secret' => $this->consumer_secret,
							'user_token' => $this->access_token,
							'user_secret' => $this->access_token_secret)
			);
		}

		return $this->_twitter;
	}

	/**
	 * verify.
	 *
	 * @return	boolean
	 *
	 * @since	1.5
	 */
	public function verify()
	{
		$api = $this->login();

		$code = $api->request('GET', $api->url('1/account/verify_credentials'));
		$code = trim($code);
		$msg = $code . ' ' . JText::_('COM_AUTOTWEET_HTTP_ERR_' . $code);

		if ($code == 200)
		{
			$response = $api->response;
			$accesslevel = $response['headers']['X-Access-Level'];
			$msg .= ' (access level: ' . $accesslevel . ')';

			if ($accesslevel == 'read-write')
			{
				$user = json_decode($response['response']);
				$result = true;
				return array(true, $msg, $user);
			}
			else
			{
				return array(false, 'Read-Only Token!');
			}
		}

		return array(false, $msg);
	}

	/**
	 * getUser.
	 *
	 * @return	object
	 *
	 * @since	1.5
	 */
	public function getUser()
	{
		$user = $this->_facebook->api("/me");
		return $user;
	}

}
