<?php

/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - AutoTweetNG posts content to social channels (Twitter, Facebook, LinkedIn, etc).
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2012 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

/**
 * AutoTweet Twitter channel.
**/

JLoader::import('channel', dirname(__FILE__));

/**
 * TwitterChannelHelper
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
*/
class TwitterChannelHelper extends ChannelHelper
{

	const MAX_PIC_LINK_LEN = 24;

	protected $twitter = null;

	/**
	 * createMessage
	 *
	 * @param   string  $data  Param
	 *
	 * @return	string
	 *
	 * @since	1.5
	 */
	public function createMessage($data)
	{
		$message = $data->text;

		if (!empty($data->hashtags))
		{
			$message = $message . ' ' . $data->hashtags;
		}

		return $message;
	}

	/**
	 * sendMessage
	 *
	 * @param   string  $message  Param
	 * @param   array   $data     Param
	 *
	 * @return	bool
	 *
	 * @since	1.5
	 */
	public function sendMessage($message, $data)
	{
		$image_url = $data->image_url;
		$media_mode = $this->getMediaMode();

		if (($media_mode != 'message') && !empty($image_url))
		{
			$image = $this->loadImage($image_url);
			$result = $this->_sendTwitterMessageWithImage($message, $image);

			if (($this->has_tmp_file) && (is_file($image)))
			{
				JFile::delete($image);
			}
			return $result;
		}
		else
		{
			return $this->_sendTwitterMessage($message, $image_url);
		}
	}

	/**
	 * Internal service functions
	 *
	 * @return	object
	 *
	 * @since	1.5
	 */
	protected function getApiInstance()
	{
		if (!$this->twitter)
		{
			require_once dirname(__FILE__) . '/tmhOAuth/tmhOAuth.php';

			$this->twitter = new tmhOAuth\tmhOAuth(
					array('consumer_key' => $this->get('consumer_key'),
							'consumer_secret' => $this->get('consumer_secret'),
							'user_token' => $this->get('access_token'),
							'user_secret' => $this->get('access_token_secret'))
			);
		}

		return $this->twitter;
	}

	/**
	 * verifyTwitterAccountData
	 *
	 * @return	array
	 *
	 * @since	1.5
	 */
	protected function verifyTwitterAccountData()
	{
		$api = $this->getApiInstance();

		$code = $api->request('GET', $api->url('1/account/verify_credentials'));
		$code = trim($code);
		$msg = $code . ' ' . JText::_('COM_AUTOTWEET_HTTP_ERR_' . $code);

		if ($code == 200)
		{
			$msg .= ' (access level: ' . $api->response['headers']['x_access_level'] . ')';
			$result = true;

			$decoded = json_decode($api->response['response']);
			$msg .= ' (' . $decoded->error . ')';
			$result = false;
		}

		return array($result, $msg);
	}

	/**
	 * _sendTwitterMessage
	 *
	 * @param   string  $status_msg  Param
	 * @param   string  $image_url   Param
	 *
	 * @return	array
	 *
	 * @since	1.5
	 */
	private function _sendTwitterMessage($status_msg, $image_url)
	{
		$api = $this->getApiInstance();
		$code = $api->request(
				'POST',
				$api->url('1.1/statuses/update'),
				array('status' => $status_msg)
		);
		return $this->_processResponse($code, $api);
	}

	/**
	 * _sendTwitterMessageWithImage
	 *
	 * @param   string  $status_msg  Param
	 * @param   string  $image_url   Param
	 *
	 * @return	array
	 */
	private function _sendTwitterMessageWithImage($status_msg, $image_url)
	{
		$api = $this->getApiInstance();
		$code = $api->request(
				'POST',
				$api->url('1.1/statuses/update_with_media'),
				array(
						'media[]' => "@{$image_url};type=image/jpeg;filename={$image_url}",
						'status' => $status_msg
		),

		// Use auth
		true,

		// Multipart
		true
		);
		return $this->_processResponse($code, $api);
	}

	/**
	 * _processResponse
	 *
	 * @param   int     $code  Param
	 * @param   object  &$api  Param
	 *
	 * @return	array
	 */
	private function _processResponse($code, &$api)
	{
		$code = trim($code);
		$response = json_decode($api->response['response']);
		$response->id = $response->id_str;

		if ($code == 200)
		{

			$msg = 'Twitter id: ' . $response->id;
			$result = true;
		}
		elseif ($code == 0)
		{
			$code = $api->response['errno'];
			$response = $api->response['error'];

			return array(false, $code . ' - ' . $response);
		}
		else
		{
			foreach ($response as $resp)
			{
				foreach ($resp as $r)
				{
					$msg = $code . ' ' . JText::_('COM_AUTOTWEET_HTTP_ERR_' . $code) . ' (' . $r->message . ')';
					break;
				}
			}
			$result = false;
		}

		return array($result, $msg);
	}

	/**
	 * getMaxCharsWithImage
	 *
	 * @return  int
	 *
	 * @since	1.5
	 */
	public function getMaxCharsWithImage()
	{
		return $this->getMaxChars() - self::MAX_PIC_LINK_LEN;
	}

}
