<?php

/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - AutoTweetNG posts content to social channels (Twitter, Facebook, LinkedIn, etc).
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2012 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

/**
 * Helper for posts form AutoTweet to channels (twitter, Facebook, ...)
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
*/
abstract class RequestHelp
{

	/**
	 * queueMessage
	 *
	 * @param   string  $articleid      Param
	 * @param   string  $source_plugin  Param
	 * @param   string  $publish_up     Param
	 * @param   string  $description    Param
	 * @param   string  $typeinfo       Param
	 * @param   string  $url            Param
	 * @param   string  $image_url      Param
	 * @param   object  $native_object  Param
	 *
	 * @return	boolean
	 *
	 * @since	1.5
	 */
	public static function insertRequest(
		$articleid,
		$source_plugin,
		$publish_up,
		$description,
		$typeinfo = 0,
		$url = '',
		$image_url = '',
		$native_object = null)
	{
		$logger = AutotweetLog::getInstance();

		// Check if message is already queued (it makes no sense to queue message more than once when modfied)
		// if message is already queued, correct the publish date

		$requestsModel = FOFModel::getTmpInstance('Requests', 'AutoTweetModel');
		$requestsModel->set('ref_id', $articleid);
		$requestsModel->set('plugin', $source_plugin);
		$requestsModel->set('typeinfo', $typeinfo);
		$list = $requestsModel->getItemList();
		if (count($list) > 0)
		{
			$id = $list[0]->id;
		}
		else
		{
			// New entry
			$id = 0;
		}

		// Avoid databse warnings when desc is longer then expected
		if (!empty($description))
		{
			$description = JString::substr($description, 0, 512);
		}

		$routeHelp = RouteHelp::getInstance();
		$url = $routeHelp->getAbsoluteUrl($url);

		if (empty($image_url))
		{
			// Default image: used in media mode when no image is available
			$image_url = CparamsHelper::getParam('default_image', '');
		}
		if (!empty($image_url))
		{
			$image_url = $routeHelp->getAbsoluteUrl($image_url, true);
		}

		$row = FOFModel::getTmpInstance('Requests', 'AutoTweetModel')->getTable();
		$row->reset();
		if ($id)
		{
			$row->load($id);
		}

		$request = array(
				'id' => $id,
				'ref_id' => $articleid,
				'plugin' => $source_plugin,
				'publish_up' => $publish_up,
				'description' => $description,
				'typeinfo' => $typeinfo,
				'url' => $url,
				'image_url' => $image_url,
				'native_object' => $native_object,
				'published' => 0
		);
		$queued = $row->save($request);
		if (!$queued)
		{
			$logger->log(
					JLog::ERROR, 'queueMessage: error storing message to database message queue, article id = '
					. $articleid . ', error message = ' . $row->getError()
			);
		}
		else
		{
			$logger->log(JLog::INFO, 'queueMessage: message stored/updated to database message queue, article id = ' . $articleid);
		}

		return $queued;
	}

	/**
	 * processRequests
	 *
	 * @param   array  $rids  Param
	 *
	 * @return	boolean
	 *
	 * @since	1.5
	 */
	public static function processRequests($rids)
	{
		$requestsModel = FOFModel::getTmpInstance('Requests', 'AutoTweetModel');
		$requestsModel->set('rids', $rids);
		$requests = $requestsModel->getItemList(true);

		return self::publishRequests($requests);
	}

	/**
	 * publishRequests
	 *
	 * @param   array  &$requests  Param
	 *
	 * @return	boolean
	 *
	 * @since	1.5
	 */
	public static function publishRequests(&$requests)
	{
		JLoader::register('SharingHelper', JPATH_AUTOTWEET_HELPERS . '/sharing.php');

		$sharinghelper = SharingHelper::getInstance();

		foreach ($requests as $request)
		{
			try
			{
				if ($sharinghelper->publishRequest($request))
				{
					// Remove only, when post is logged successfully
					self::processed($request->id);
				}
				else
				{
					self::saveError($request->id);
				}

			}
			catch (Exception $e)
			{
				self::saveError($request->id, $e->getMessage());
			}
		}

		return true;
	}

	/**
	 * processed
	 *
	 * @param   int  $id  Param
	 *
	 * @return	boolean
	 *
	 * @since	1.5
	 */
	public static function processed($id)
	{
		$request = FOFModel::getTmpInstance('Requests', 'AutotweetModel')->getTable();
		$request->reset();
		if ($request->load($id))
		{
			$data = array();

			$data['published'] = true;
			$nativeObject = json_decode($request->native_object);
			$nativeObject->error = false;
			$nativeObject->error_message = 'Ok!';
			$data['native_object'] = json_encode($nativeObject);

			$request->save($data);
		}
	}

	/**
	 * saveError
	 *
	 * @param   int     $id       Param
	 * @param   string  $message  Param
	 *
	 * @return	boolean
	 *
	 * @since	1.5
	 */
	public static function saveError($id, $message = null)
	{
		$request = FOFModel::getTmpInstance('Requests', 'AutotweetModel')->getTable();
		$request->reset();
		if ($request->load($id))
		{
			$nativeObject = json_decode($request->native_object);

			$nativeObject->error = true;

			if ($message)
			{
				$nativeObject->error_message = $message;
			}
			else
			{
				$nativeObject->error_message = 'COM_AUTOTWEET_ERROR_PROCESSING';
			}

			$data = array();
			$data['native_object'] = json_encode($nativeObject);
			$data['published'] = true;

			$request->save($data);
		}
	}

	/**
	 * getRequestList
	 *
	 * @param   JDate  $check_date  Param
	 * @param   int    $limit       Param
	 *
	 * @return	array
	 *
	 * @since	1.5
	 */
	public static function getRequestList($check_date, $limit)
	{
		$requestsModel = FOFModel::getTmpInstance('Requests', 'AutoTweetModel');
		$requestsModel->set('until_date', $check_date->toSql());
		$requestsModel->set('filter_order', 'publish_up');
		$requestsModel->set('filter_order_Dir', 'ASC');
		$requestsModel->set('limit', $limit);
		return $requestsModel->getItemList();
	}

}
