<?php
/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - AutoTweetNG posts content to social channels (Twitter, Facebook, LinkedIn, etc).
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2012 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

/**
 * SelectHelper
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
*/
class SelectHelper
{

	/**
	 * channels.
	 *
	 * @param   string  $selected  The key that is selected
	 * @param   string  $id        The id for the field
	 * @param   array   $attribs   Additional HTML attributes for the <select> tag
	 * @param   array   &$config   Config
	 *
	 * @return  string  HTML
	 *
	 * @since   3.0
	 */
	public static function channels($selected = null, $id = 'channel', $attribs = array(), &$config = null)
	{
		$channelsModel = FOFModel::getTmpInstance('Channels', 'AutoTweetModel');
		$channelsModel->set('published', 1);
		$items = $channelsModel->getItemList(true);

		$options = array();
		$options[] = JHTML::_('select.option', 0, '-' . JText::_('JSELECT') . '-');
		if (count($items))
		{
			foreach ($items as $item)
			{
				$options[] = JHTML::_('select.option', $item->id, $item->name);
			}
		}
		return self::genericlist($options, $id, $attribs, $selected, $id);
	}

	/**
	 * plugins.
	 *
	 * @param   string  $selected  The key that is selected
	 * @param   string  $id        The id for the field
	 * @param   array   $attribs   Additional HTML attributes for the <select> tag
	 * @param   array   &$config   Config
	 *
	 * @return  string  HTML
	 *
	 * @since   3.0
	 */
	public static function plugins($selected = null, $id = 'plugin', $attribs = array(), &$config = null)
	{
		$pluginsModel = FOFModel::getTmpInstance('Plugins', 'AutoTweetModel');
		$pluginsModel->set('extension_plugins_only', true);
		$pluginsModel->set('published_only', true);
		$items = $pluginsModel->getItemList(true);

		// $jlang = JFactory::getLanguage();

		// $jlang->load($item->name);
		// JText::_('PLG_' . $item->element . '_NAME')

		$options = array();
		$options[] = JHTML::_('select.option', 0, '-' . JText::_('JSELECT') . '-');
		if (count($items))
		{
			foreach ($items as $item)
			{
				$name = $pluginsModel->getSimpleName($item->element);
				$options[] = JHTML::_('select.option', $item->element, $name);
			}
		}
		return self::genericlist($options, $id, $attribs, $selected, $id);
	}

	/**
	 * ruletypes.
	 *
	 * @param   string  $selected  The key that is selected
	 * @param   string  $id        The id for the field
	 * @param   array   $attribs   Additional HTML attributes for the <select> tag*
	 *
	 * @return  string  HTML
	 *
	 * @since   3.0
	 */
	public static function ruletypes($selected = null, $id = 'ruletype', $attribs = array())
	{
		$channeltypes = FOFModel::getTmpInstance('Ruletypes', 'AutotweetModel');
		$items = $channeltypes->getItemList(true);

		$options = array();
		$options[] = JHTML::_('select.option', 0, '-' . JText::_('JSELECT') . '-');
		if (count($items))
		{
			foreach ($items as $item)
			{
				$options[] = JHTML::_('select.option', $item->id, $item->name);
			}
		}
		return self::genericlist($options, $id, $attribs, $selected, $id);
	}

	/**
	 * channeltypes.
	 *
	 * @param   string  $selected  The key that is selected
	 * @param   string  $id        The id for the field
	 * @param   array   $attribs   Additional HTML attributes for the <select> tag*
	 *
	 * @return  string  HTML
	 *
	 * @since   3.0
	 */
	public static function channeltypes($selected = null, $id = 'channeltype', $attribs = array())
	{
		$channeltypes = FOFModel::getTmpInstance('Channeltypes', 'AutotweetModel');
		$items = $channeltypes->getItemList(true);

		$options = array();
		$options[] = JHTML::_('select.option', 0, '-' . JText::_('JSELECT') . '-');
		if (count($items))
		{
			foreach ($items as $item)
			{
				$options[] = JHTML::_('select.option', $item->id, $item->name);
			}
		}
		return self::genericlist($options, $id, $attribs, $selected, $id);
	}

	/**
	 * getChanneltypeName.
	 *
	 * @param   string  $channeltype  Param
	 *
	 * @return  string
	 *
	 * @since   3.0
	 */
	public static function getChanneltypeName($channeltype)
	{
		static $channeltypes = null;

		if (is_null($channeltypes))
		{
			$channeltypesModel = FOFModel::getTmpInstance('Channeltypes', 'AutotweetModel');
			$items = $channeltypesModel->getItemList(true);

			if (count($items))
			{
				foreach ($items as $item)
				{
					$channeltypes[$item->id] = $channeltypesModel->getIcon($item->id) . ' - ' . $item->name;
				}
			}
			else
			{
				return '?';
			}
		}

		if (array_key_exists($channeltype, $channeltypes))
		{
			return $channeltypes[$channeltype];
		}
		else
		{
			return '?';
		}
	}

	/**
	 * getChannelName.
	 *
	 * @param   string  $channel   Param
	 * @param   bool    $isModule  Param
	 *
	 * @return  string
	 *
	 * @since   3.0
	 */
	public static function getChannelName($channel, $isModule = false)
	{
		static $channels = null;

		if (is_null($channels))
		{
			FOFModel::getTmpInstance('Channeltypes', 'AutotweetModel');
			$itemsModel = FOFModel::getTmpInstance('Channels', 'AutotweetModel');
			$itemsModel->set('published', true);

			$items = $itemsModel->getItemList(true);

			if (count($items))
			{
				foreach ($items as $item)
				{
					$icon = AutotweetModelChanneltypes::getIcon($item->channeltype_id);
					if ($isModule)
					{
						$channels[$item->id] = $icon;
					}
					else
					{
						$channels[$item->id] = $icon . ' - ' . $item->name;
					}
				}
			}
			else
			{
				return '?';
			}
		}

		if (array_key_exists($channel, $channels))
		{
			return $channels[$channel];
		}
		else
		{
			return '?';
		}
	}

	/**
	 * genericlist.
	 *
	 * @param   array   $list      The key that is selected
	 * @param   string  $name      The name for the field
	 * @param   array   $attribs   Additional HTML attributes for the <select> tag*
	 * @param   string  $selected  The key that is selected
	 * @param   string  $idTag     The id for the field
	 *
	 * @return  string  HTML
	 *
	 * @since   3.0
	 */
	protected static function genericlist($list, $name, $attribs, $selected, $idTag)
	{
		if (empty($attribs))
		{
			$attribs = null;
		}
		else
		{
			$temp = '';
			foreach ($attribs as $key => $value)
			{
				$temp .= $key . ' = "' . $value . '"';
			}
			$attribs = $temp;
		}

		return JHTML::_('select.genericlist', $list, $name, $attribs, 'value', 'text', $selected, $idTag);
	}

	/**
	 * showstatictext.
	 *
	 * @param   string  $selected  The key that is selected
	 * @param   string  $id        The id for the field
	 * @param   array   $attribs   Additional HTML attributes for the <select> tag*
	 *
	 * @return  string  HTML
	 *
	 * @since   3.0
	 */
	public static function showstatictext($selected = null, $id = 'show_static_text', $attribs = array() )
	{
		$options = array();

		$selected = ($selected ? $selected : 'selected');

		// Get media modes
		$modes = self::getShowStaticTextEnum();

		// Generate html
		foreach ($modes as $mode)
		{
			$options[] = JHtml::_('select.option', $mode, JText::_(self::getTextForEnum($mode)), 'value', 'text');
		}

		return self::genericlist($options, $id, $attribs, $selected, $id);
	}

	/**
	 * showurl.
	 *
	 * @param   string  $selected  The key that is selected
	 * @param   string  $id        The id for the field
	 * @param   array   $attribs   Additional HTML attributes for the <select> tag*
	 *
	 * @return  string  HTML
	 *
	 * @since   3.0
	 */
	public static function showurl($selected = null, $id = 'show_url', $attribs = array() )
	{
		$options = array();

		$selected = ($selected ? $selected : 'selected');

		// Get media modes
		$modes = self::getShowurlEnum();

		// Generate html
		foreach ($modes as $mode)
		{
			$options[] = JHtml::_('select.option', $mode, JText::_(self::getTextForEnum($mode)), 'value', 'text');
		}

		return self::genericlist($options, $id, $attribs, $selected, $id);
	}

	/**
	 * mediamodes.
	 *
	 * @param   string  $selected  The key that is selected
	 * @param   string  $id        The id for the field
	 * @param   array   $attribs   Additional HTML attributes for the <select> tag*
	 *
	 * @return  string  HTML
	 *
	 * @since   3.0
	 */
	public static function autopublish($selected = null, $id = 'autopublish', $attribs = array() )
	{
		$options = array();

		$selected = ($selected ? $selected : 'on');

		// Get media modes
		$modes = self::getAutopublishEnum();

		// Generate html
		foreach ($modes as $mode)
		{
			$options[] = JHtml::_('select.option', $mode, JText::_(self::getTextForEnum($mode)), 'value', 'text');
		}

		return self::genericlist($options, $id, $attribs, $selected, $id);
	}

	/**
	 * mediamodes.
	 *
	 * @param   string  $selected  The key that is selected
	 * @param   string  $id        The id for the field
	 * @param   array   $attribs   Additional HTML attributes for the <select> tag*
	 *
	 * @return  string  HTML
	 *
	 * @since   3.0
	 */
	public static function mediamodes($selected = null, $id = 'type', $attribs = array() )
	{
		$options = array();
		$options[] = JHTML::_('select.option', '', '-' . JText::_('JSELECT') . '-');

		// Get media modes
		$modes = self::getMediaModeEnum();

		// Generate html
		foreach ($modes as $mode)
		{
			$options[] = JHtml::_('select.option', $mode, JText::_(self::getTextForEnum($mode)), 'value', 'text');
		}

		return self::genericlist($options, $id, $attribs, $selected, $id);
	}

	/**
	 * pubstates.
	 *
	 * @param   string  $selected  The key that is selected
	 * @param   string  $id        The id for the field
	 * @param   array   $attribs   Additional HTML attributes for the <select> tag*
	 *
	 * @return  string  HTML
	 *
	 * @since   3.0
	 */
	public static function pubstates($selected = null, $id = 'type', $attribs = array() )
	{
		$options = array();
		$options[] = JHTML::_('select.option', '', '-' . JText::_('JSELECT') . '-');

		// Get media modes
		$modes = self::getPubstateEnum();

		// Generate html
		foreach ($modes as $mode)
		{
			$options[] = JHtml::_('select.option', $mode, JText::_(self::getTextForEnum($mode)), 'value', 'text');
		}

		return self::genericlist($options, $id, $attribs, $selected, $id);
	}

	/**
	 * tasktypes.
	 *
	 * @param   string  $selected  The key that is selected
	 * @param   string  $id        The id for the field
	 * @param   array   $attribs   Additional HTML attributes for the <select> tag*
	 *
	 * @return  string  HTML
	 *
	 * @since   3.0
	 */
	public static function tasktypes($selected = null, $id = 'type', $attribs = array() )
	{
		$options = array();
		$options[] = JHTML::_('select.option', '', '-' . JText::_('JSELECT') . '-');

		$options[] = JHTML::_('select.option', '1', JText::_('COM_AUTOTWEET_TASKS_TYPE_1'));
		$options[] = JHTML::_('select.option', '2', JText::_('COM_AUTOTWEET_TASKS_TYPE_2'));
		$options[] = JHTML::_('select.option', '3', JText::_('COM_AUTOTWEET_TASKS_TYPE_3'));
		$options[] = JHTML::_('select.option', '4', JText::_('COM_AUTOTWEET_TASKS_TYPE_4'));
		$options[] = JHTML::_('select.option', '5', JText::_('COM_AUTOTWEET_TASKS_TYPE_5'));
		$options[] = JHTML::_('select.option', '6', JText::_('COM_AUTOTWEET_TASKS_TYPE_6'));
		$options[] = JHTML::_('select.option', '7', JText::_('COM_AUTOTWEET_TASKS_TYPE_7'));

		return self::genericlist($options, $id, $attribs, $selected, $id);
	}

	/**
	 * published.
	 *
	 * @param   string  $selected  The key that is selected
	 * @param   string  $id        The id for the field
	 * @param   array   $attribs   Additional HTML attributes for the <select> tag*
	 *
	 * @return  string  HTML
	 *
	 * @since   3.0
	 */
	public static function published($selected = null, $id = 'enabled', $attribs = array())
	{
		$options = array();
		$options[] = JHTML::_('select.option', '', '-' . JText::_('JSELECT') . '-');
		$options[] = JHTML::_('select.option', 1, JText::_('JPUBLISHED'));
		$options[] = JHTML::_('select.option', 0, JText::_('JUNPUBLISHED'));

		return self::genericlist($options, $id, $attribs, $selected, $id);
	}

	/**
	 * Method to getPubstateEnum.
	 *
	 * @return	array.
	 *
	 * @since	1.5
	 */
	public static function getPubstateEnum()
	{
		return array('success', 'error', 'approve', 'cronjob');
	}

	/**
	 * Method to getMediaModeEnum.
	 *
	 * @return	array.
	 *
	 * @since	1.5
	 */
	public static function getMediaModeEnum()
	{
		return array('message', 'attachment', 'both');
	}

	/**
	 * Method to getAutopublishEnum.
	 *
	 * @return	array.
	 *
	 * @since	1.5
	 */
	public static function getAutopublishEnum()
	{
		return array('selected', 'on', 'off');
	}

	/**
	 * Method to getShowUrlEnum.
	 *
	 * @return	array.
	 *
	 * @since	1.5
	 */
	public static function getShowUrlEnum()
	{
		return array('selected', 'off', 'beginning_of_message', 'end_of_message');
	}

	/**
	 * Method to getShowStaticTextEnum.
	 *
	 * @return	array.
	 *
	 * @since	1.5
	 */
	public static function getShowStaticTextEnum()
	{
		return array('off', 'beginning_of_message', 'end_of_message');
	}

	/**
	 * Method to getTextForEnum.
	 *
	 * @param   string  $enum_string  Param
	 * @param   bool    $with_icon    Param
	 * @param   bool    $isModule     Param
	 *
	 * @return	string.
	 *
	 * @since	1.5
	 */
	public static function getTextForEnum($enum_string, $with_icon = false, $isModule = false)
	{
		switch ($enum_string)
		{
			case 'selected':
				$result = '-' . JText::_('JSELECT') . '-';
				break;

			case 'default':
				$result = JText::_('JDEFAULT');
				break;
			case 'on':
				$result = JText::_('JON');
				break;
			case 'off':
				$result = JText::_('JOFF');
				break;

			case 'message':
				$result = JText::_('COM_AUTOTWEET_OPTION_MEDIAMODE_MESSAGE');
				break;
			case 'attachment':
				$result = JText::_('COM_AUTOTWEET_OPTION_MEDIAMODE_ATTACHMENT');
				break;
			case 'both':
				$result = JText::_('COM_AUTOTWEET_OPTION_MEDIAMODE_BOTH');
				break;

			case 'beginning_of_message':
				$result = JText::_('COM_AUTOTWEET_OPTION_POSITION_BEGINNINGOFMESSAGE');
				break;
			case 'end_of_message':
				$result = JText::_('COM_AUTOTWEET_OPTION_POSITION_ENDOFMESSAGE');
				break;

			case 'success':
				$result = ($with_icon ? '<i style="color: green;" class="xticon-check"></i>' : '') . ($isModule ? '' : ' - ' . JText::_('COM_AUTOTWEET_STATE_PUBSTATE_SUCCESS'));
				break;
			case 'error':
				$result = ($with_icon ? ' <i style="color: red;" class="xticon-remove"></i>' : '') . ($isModule ? '' : ' - ' . JText::_('COM_AUTOTWEET_STATE_PUBSTATE_ERROR'));
				break;
			case 'approve':
				$result = ($with_icon ? ' <i class="xticon-check-empty"></i>' : '') . ($isModule ? '' : ' - ' . JText::_('COM_AUTOTWEET_STATE_PUBSTATE_APPROVE'));
				break;
			case 'cronjob':
				$result = ($with_icon ? ' <i class="xticon-time"></i>' : '') . ($isModule ? '' : ' - ' . JText::_('COM_AUTOTWEET_STATE_PUBSTATE_CRONJOB'));
				break;
			case 'cancelled':
				$result = ($with_icon ? ' <i style="color: grey;" class="xticon-thumbs-down"></i>' : '') . ($isModule ? '' : ' - ' . JText::_('COM_AUTOTWEET_STATE_PUBSTATE_CANCELLED'));
				break;

			default:
				$result = 'AUTOTWEET_MISSING_LANGUAGE_STRING';
				break;
		}

		return $result;
	}

	/**
	 * yesno.
	 *
	 * @param   string  $selected  The key that is selected
	 * @param   string  $name      The id for the field
	 * @param   array   $attribs   Additional HTML attributes for the <select> tag*
	 * @param   string  $id        The id for the field
	 *
	 * @return  string  HTML
	 *
	 * @since   3.0
	 */
	public static function yesno($selected = null, $name = 'yesno', $attribs = array(), $id = null)
	{
		$options = array();
		$options[] = JHTML::_('select.option', 1, JText::_('JYES'));
		$options[] = JHTML::_('select.option', 0, JText::_('JNO'));

		return self::genericlist($options, $name, $attribs, $selected, $id);
	}

	/**
	 * channels.
	 *
	 * @param   string  $selected      The key that is selected
	 * @param   string  $id            The id for the field
	 * @param   array   $attribs       Additional HTML attributes for the <select> tag*
	 * @param   string  $app_id        Params
	 * @param   string  $secret        Params
	 * @param   string  $access_token  Params
	 *
	 * @return  string  HTML
	 *
	 * @since   3.0
	 */
	public static function fbchannels(
		$selected = null,
		$id = 'xtform[fbchannel_id]',
		$attribs = array(),
		$app_id = null,
		$secret = null,
		$access_token = null)
	{
		$options = array();
		$attribs = array();

		if ($access_token)
		{
			require_once dirname(__FILE__) . '/../helpers/channels/fbapp.php';

			try
			{
				$fbAppHelper = new FbAppHelper($app_id, $secret, $access_token);
				if ($fbAppHelper->login())
				{
					$channels = $fbAppHelper->getChannels();
					foreach ($channels as $channel)
					{
						$nm = $channel['name'];
						if ((empty($nm)) || ($nm == 'null'))
						{
							$nm = $channel['id'];
						}
						$opt = JHTML::_('select.option', $channel['id'], $channel['type'] . ': ' . $nm);
						$opt->access_token = array('access_token' => $channel['access_token']);
						$options[] = $opt;
					}

					$attribs['id'] = $id;
					$attribs['list.attr'] = null;
					$attribs['list.translate'] = false;
					$attribs['option.key'] = 'value';
					$attribs['option.text'] = 'text';
					$attribs['list.select'] = $selected;
					$attribs['option.attr'] = 'access_token';

					return JHtmlSelect::genericlist($options, $id, $attribs);
				}
				else
				{
					$error_message = 'Facebook Login Failed!';
					$options[] = JHTML::_('select.option', '', $error_message);
				}
			}
			catch (Exception $e)
			{
				$error_message = $e->__toString();
				$options[] = JHTML::_('select.option', '', $error_message);
			}
		}
		else
		{
			$options[] = JHTML::_('select.option', '', '-' . JText::_('JSELECT') . '-');
		}

		return self::genericlist($options, $id, $attribs, $selected, $id, array('option.attr' => 'access_token'));
	}

	/**
	 * fbalbums.
	 *
	 * @param   string  $selected      The key that is selected
	 * @param   string  $id            The id for the field
	 * @param   array   $attribs       Additional HTML attributes for the <select> tag*
	 * @param   string  $app_id        Params
	 * @param   string  $secret        Params
	 * @param   string  $access_token  Params
	 * @param   string  $channelid     Params
	 *
	 * @return  string  HTML
	 *
	 * @since   3.0
	 */
	public static function fbalbums(
		$selected = null,
		$id = 'xtform[fbalbum_id]',
		$attribs = array(),
		$app_id = null,
		$secret = null,
		$access_token = null,
		$channelid = null)
	{
		$options = array();
		$attribs = array();

		if ($access_token)
		{
			require_once dirname(__FILE__) . '/../helpers/channels/fbapp.php';

			try
			{
				$fbAppHelper = new FbAppHelper($app_id, $secret, $access_token);
				if ($fbAppHelper->login())
				{
					$albums = $fbAppHelper->getAlbums($channelid);
					foreach ($albums as $album)
					{
						$nm = $album['name'];
						if ((empty($nm)) || ($nm == 'null'))
						{
							$nm = $album['id'];
						}
						$opt = JHTML::_('select.option', $album['id'], $nm);
						$options[] = $opt;
					}

					$attribs['id'] = $id;
					$attribs['list.attr'] = null;
					$attribs['list.translate'] = false;
					$attribs['option.key'] = 'value';
					$attribs['option.text'] = 'text';
					$attribs['list.select'] = $selected;

					return JHtmlSelect::genericlist($options, $id, $attribs);
				}
				else
				{
					$error_message = 'Facebook Login Failed!';
					$options[] = JHTML::_('select.option', '', $error_message);
				}
			}
			catch (Exception $e)
			{
				$error_message = $e->__toString();
				$options[] = JHTML::_('select.option', '', $error_message);
			}
		}
		else
		{
			$options[] = JHTML::_('select.option', '', '-' . JText::_('JSELECT') . '-');
		}

		return self::genericlist($options, $id, $attribs, $selected, $id, array('option.attr' => 'access_token'));
	}

	/**
	 * category.
	 *
	 * @param   string  $name        The key that is selected
	 * @param   string  $extension   The id for the field
	 * @param   string  $selected    Additional HTML attributes for the <select> tag*
	 * @param   string  $javascript  Params
	 * @param   string  $order       Params
	 * @param   string  $size        Params
	 * @param   string  $sel_cat     Params
	 * @param   string  $readonly    Params
	 *
	 * @return  string  HTML
	 *
	 * @since   3.0
	 */
	public static function category($name, $extension, $selected = null, $javascript = null, $order = null, $size = 1, $sel_cat = 1, $readonly = false)
	{
		$categories = JHtml::_('category.options', $extension);
		if ($sel_cat)
		{
			array_unshift($categories, JHTML::_('select.option',  '0', JText::_('JOption_Select_Category')));
		}

		$category = JHTML::_(
				'select.genericlist',
				$categories,
				$name,
				($readonly ? 'readonly="readonly" ' : '') . 'class="inputbox" size="' . $size . '" ' . $javascript,
				'value', 'text',
				$selected
		);

		return $category;
	}

	/**
	 * languages.
	 *
	 * @param   string  $selected     The key that is selected
	 * @param   string  $id           The id for the field
	 * @param   array   $attribs      Additional HTML attributes for the <select> tag*
	 * @param   string  $show_select  Params
	 *
	 * @return  string  HTML
	 *
	 * @since   3.0
	 */
	public static function languages($selected = null, $id = 'language', $attribs = array(), $show_select = false )
	{
		jimport('joomla.language.helper');
		$languages = JLanguageHelper::getLanguages('lang_code');
		$options = array();

		if ($show_select)
		{
			$options[] = JHTML::_('select.option', '', '---');
		}

		$options[] = JHTML::_('select.option', '*', JText::_('JALL_LANGUAGE'));
		if (!empty($languages))
		{
			foreach ($languages as $key => $lang)
			{
				$options[] = JHTML::_('select.option', $key, $lang->title);
			}
		}

		return self::genericlist($options, $id, $attribs, $selected, $id);
	}

}
