<?php

/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - AutoTweetNG posts content to social channels (Twitter, Facebook, LinkedIn, etc).
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2012 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

/**
 * Helper for posts form AutoTweet to channels (twitter, Facebook, ...)
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
 */
class SharingHelper
{

	// Seconds
	const RESEND_DELAY = 1;

	// Max chars for title and fulltext (to avoid database and other errors
	const MAX_CHARS_TITLE = 255;
	const MAX_CHARS_FULLTEXT = 5000;

	// Cron job mode
	protected $cron_enabled = 0;

	// Duplicate post detection
	protected $dpcheck_enabled = 1;

	// 3 hours
	protected $dpcheck_time_intval = 10800;

	protected $shorturl_always = 1;

	protected $resend_attempts = 2;

	// Logging
	protected $logger;

	private static $_instance = null;

	protected $routeHelp = null;

	protected $current_short_url = null;

	/**
	 * SharingHelper
	 *
	 * @since	1.5
	 */
	protected function SharingHelper()
	{
		JLoader::register('CparamsHelper', JPATH_AUTOTWEET_HELPERS . '/cparams.php');
		JLoader::register('AutotweetLog', JPATH_AUTOTWEET_HELPERS . '/autotweetlog.php');

		JLoader::register('ChannelFactory', JPATH_AUTOTWEET_HELPERS . '/channels/channelfactory.php');
		JLoader::register('RequestHelp', JPATH_AUTOTWEET_HELPERS . '/request.php');
		JLoader::register('PostHelper', JPATH_AUTOTWEET_HELPERS . '/post.php');

		JLoader::register('TextHelper', JPATH_AUTOTWEET_HELPERS . '/text.php');
		JLoader::register('ShorturlHelper', JPATH_AUTOTWEET_HELPERS . '/shorturl.php');

		JLoader::register('RuleEngineHelper', JPATH_AUTOTWEET_HELPERS . '/ruleengine.php');

		JLoader::register('AutotweetBaseHelper', JPATH_AUTOTWEET_HELPERS . '/autotweetbasehelper.php');

		// Cron job mode
		$this->cron_enabled = CparamsHelper::getParam('cron_enabled', 0);

		// Duplicate post detection
		$this->dpcheck_enabled = CparamsHelper::getParam('dpcheck_enabled', 1);

		// Hours to seconds
		$this->dpcheck_time_intval = CparamsHelper::getParam('dpcheck_time_intval', 24) * 3600;

		$this->shorturl_always = CparamsHelper::getParam('shorturl_always', 1);

		$this->resend_attempts = CparamsHelper::getParam('resend_attempts', 2);

		// Init AutoTweet logging
		$this->logger = AutotweetLog::getInstance();
	}

	/**
	 * getInstance
	 *
	 * @return	Instance
	 *
	 * @since	1.5
	 */
	public static function &getInstance()
	{
		if (!self::$_instance)
		{
			self::$_instance = new SharingHelper;
		}

		return self::$_instance;
	}

	/**
	 * publishRequest
	 *
	 * @param   array   $request  Param
	 * @param   object  $userid   Param
	 *
	 * @return	boolean
	 *
	 * @since	1.5
	 */
	public function publishRequest($request, $userid = null)
	{
		// Default: generate new entry for repost of entry with state success
		$postid = 0;

		// Get source plugin for message
		// Gets the plugin that has triggered the message
		$pluginsModel = FOFModel::getTmpInstance('Plugins', 'AutoTweetModel');
		$plugin = $pluginsModel->createPlugin($request->plugin);

		$this->logger->log(JLog::INFO, 'post_entry: ' . print_r($request, true));

		if (empty($plugin))
		{
			$this->logger->log(
					JLog::WARNING, 'publishRequest: unknown plugin. Source: ' . $request->plugin
			);
			$post = $request;
		}
		else
		{
			// Get data from plugin
			if (method_exists($plugin, 'getExtendedData'))
			{
				if (!isset($request->native_object) && isset($request->params))
				{
					$request->native_object = $request->params;
				}
				$post = $plugin->getExtendedData($request->ref_id, $request->typeinfo, $request->native_object);
			}
			else
			{
				$post = $plugin->getData($request->ref_id, $request->typeinfo);
			}
		}

		// Check if post is valid to avoid spam; if not remove post from queue
		if (empty($post) || !array_key_exists('is_valid', $post) || !$post['is_valid'])
		{
			$this->logger->log(
					JLog::ERROR, 'publishRequest: message not valid (spam or technical problem - old plugin?), queue id = ' .
					$request->id
			);
			RequestHelp::saveError($request->id, 'COM_AUTOTWEET_ERROR_PUBLISHREQUEST');

			return false;
		}
		else
		{
			// Convert array to object
			$json  = json_encode($post);
			$post = json_decode($json);
		}

		$post->ref_id = $request->ref_id;
		$post->plugin = $request->plugin;
		$post->postdate = $request->publish_up;

		// Data native_object initialization
		if (isset($request->native_object))
		{
			$post->native_object = $request->native_object;
		}
		else
		{
			$post->native_object = '';
		}

		// Url
		if ((isset($request->url)) && (!empty($request->url)))
		{
			$post->url = $request->url;
		}
		elseif ((isset($post->url)) && (!empty($post->url)))
		{
			$routeHelp = RouteHelp::getInstance();
			$url = $routeHelp->getAbsoluteUrl($post->url);
			$post->url = $url;
		}
		else
		{
			$post->url = '';
		}
		$this->logger->log(JLog::INFO, 'publishRequest: url = ' . $post->url);

		// Image url
		if ((isset($request->image_url)) && (!empty($request->image_url)))
		{
			$post->image_url = $request->image_url;
		}
		elseif ((isset($post->image_url)) && (!empty($post->image_url)))
		{
			$routeHelp = RouteHelp::getInstance();
			$url = $routeHelp->getAbsoluteUrl($post->image_url);
			$post->image_url = $url;
		}
		else
		{
			$post->image_url = '';
		}
		$this->logger->log(JLog::INFO, 'publishRequest: image url = ' . $post->image_url);

		// Truncate title and fulltext for new messages ('if' is for backward compatibillity)
		if (isset($post->title))
		{
			$title = TextHelper::cleanText($post->title);
			$post->title = TextHelper::truncString($title, JString::strlen($title), self::MAX_CHARS_TITLE);
		}
		else
		{
			$post->title = '';
		}

		if (isset($post->fulltext))
		{
			$fulltext = TextHelper::cleanText($post->fulltext);
			$post->fulltext = TextHelper::truncString($fulltext, JString::strlen($fulltext), self::MAX_CHARS_FULLTEXT, true);
		}
		else
		{
			$post->fulltext = '';
		}

		if (!isset($post->hashtags))
		{
			$post->hashtags = '';
		}

		if (!isset($post->catids))
		{
			$post->catids = array();
		}

		if (!isset($post->author))
		{
			$post->author = '';
		}

		if (isset($post->event))
		{
			$post->event_data = json_encode($post->event);
		}
		else
		{
			$post->event_data = json_encode(array());
		}

		if (!isset($post->language))
		{
			$post->language = '';
		}

		if (!isset($post->access))
		{
			$post->access = '';
		}

		$post->autopublish = $plugin->isAutopublish();
		$post->show_url = $plugin->getShowUrlMode();

		return $this->sendRequest($request, $post, $userid);
	}

	/**
	 * sendRequest
	 *
	 * @param   string  &$request  Param
	 * @param   string  &$post     Param
	 * @param   object  $userid    Param
	 *
	 * @return	boolean
	 *
	 * @since	1.5
	 */
	protected function sendRequest(&$request, &$post, $userid = null)
	{
		$success = false;
		$this->logger->log(JLog::INFO, 'sendRequest: ref_id = ' . $request->ref_id);

		$rule_engine = RuleEngineHelper::getInstance();

		$plugin = $request->plugin;
		$rule_engine->load($plugin);

		// Channels - Rules
		$channel_rules = array();
		$hasRules = $rule_engine->hasRules();
		if ($hasRules)
		{
			$this->logger->log(JLog::INFO, 'getChannels:Rules found for plugin ' . $plugin);

			// New post, get channels to post for
			$channel_rules = $rule_engine->getChannels($request->plugin, $post);
			$channel_rules_ids = array_keys($channel_rules);
		}
		else
		{
			$this->logger->log(JLog::INFO, 'getChannels: No rules found for plugin ' . $plugin);
			$channel_rules_ids = array();
		}

		// Rest of the Channels
		$channels = ChannelFactory::getInstance()->getChannels();
		$channels_ids = array_keys($channels);

		$remaining_channels_ids = array_diff($channels_ids, $channel_rules_ids);

		// Save orginal url for log and other usages
		$post->org_url = $post->url;
		$this->current_short_url = null;

		// A request for each Channel - Rule
		$result_msg = '';
		$initial_autopublish_state = $post->autopublish;
		$initial_show_url_state = $post->show_url;
		foreach ($channel_rules as $channel_id => $rule)
		{
			// There's a rule, but the channel is not enabled
			if (!isset($channels[$channel_id]))
			{
				continue;
			}
			$channel = $channels[$channel_id];

			$post->id = 0;
			$post->channel_id = $channel_id;
			$post->autopublish = ($channel->isAutopublish() && $initial_autopublish_state);
			$post->show_url = $initial_show_url_state;

			$rule_engine->executeRule($rule, $channel, $post);

			$message = $post->message;
			$post->message = TextHelper::truncString($message, JString::strlen($message), $channel->getMaxChars());

			$success = $this->_sendRequest($channel, $post);

			// If one channel fails, it's stopped
			if (!$success)
			{
				return false;
			}
		}

		// A request for each of the remaining Channels
		foreach ($remaining_channels_ids as $channel_id)
		{
			$channel = $channels[$channel_id];

			$post->id = 0;
			$post->channel_id = $channel_id;
			$post->autopublish = ($channel->isAutopublish() && $initial_autopublish_state);
			$post->show_url = $initial_show_url_state;

			$message = $channel->createMessage($post);
			$post->message = TextHelper::truncString($message, JString::strlen($message), $channel->getMaxChars());

			$success = $this->_sendRequest($channel, $post, $userid);

			// If one channel fails, it's stopped
			if (!$success)
			{
				return false;
			}
		}

		// True when message is sent
		return $success;
	}

	/**
	 * _sendRequest
	 *
	 * @param   string  &$channel  Param
	 * @param   string  &$post     Param
	 * @param   object  $userid    Param
	 *
	 * @return	boolean
	 *
	 * @since	1.5
	 */
	protected function _sendRequest(&$channel, &$post, $userid = null)
	{
		// Send new post when autopublish is enabled and cron is disabled
		$send_now = ($post->autopublish && !$this->cron_enabled);

		// Sending is allowed, when autopublish for channel and plugin is allowed or post is approved from backend (and not in cron modeI)
		if ($send_now)
		{
			$response = $this->sharePost($channel, $post, $userid);
			$state = $response['state'];
			$result_msg = $response['result_msg'];
		}
		else
		{
			if (!$post->autopublish)
			{
				// Approve needed
				$state = AutotweetPostHelper::POST_APPROVE;
				$result_msg = 'COM_AUTOTWEET_MSG_POSTRESULT_APPROVE';
			}
			elseif ($this->cron_enabled)
			{
				// Cron mode enabled
				$state = AutotweetPostHelper::POST_CRONJOB;
				$result_msg = 'COM_AUTOTWEET_MSG_POSTRESULT_CRONJOB';
			}
			else
			{
				// ERROR: This sould NOT happen!
				$state = AutotweetPostHelper::POST_ERROR;
				$result_msg = 'COM_AUTOTWEET_MSG_POSTRESULT_ERROR';
			}
		}

		// Store message in log
		return PostHelper::savePost($state, $result_msg, $post, $userid);
	}

	/**
	 * publishPost
	 *
	 * @param   array   $post    Param
	 * @param   object  $userid  Param
	 *
	 * @return	boolean
	 *
	 * @since	1.5
	 */
	public function publishPost($post, $userid = null)
	{
		// Where are here, so ...
		$post->autopublish = true;

		// Decode event data back to array
		$post->event = json_decode($post->event_data, true);

		return $this->sendPost($post, $userid);
	}

	/**
	 * sendPost
	 *
	 * @param   string  &$post   Param
	 * @param   int     $userid  Param
	 *
	 * @return	boolean
	 *
	 * @since	1.5
	 */
	protected function sendPost(&$post, $userid = null)
	{
		$this->logger->log(JLog::INFO, 'sendPost: postid = ' . $post->id);

		// Post for specific channel
		$channelFactory = ChannelFactory::getInstance();
		if (!$channelFactory->hasChannel($post->channel_id))
		{
			PostHelper::savePost(AutotweetPostHelper::POST_ERROR, 'COM_AUTOTWEET_ERR_CHANNEL_NOTFOUND', $post, $userid);
			return false;
		}

		$channel = ChannelFactory::getInstance()->getChannel($post->channel_id);
		$channel_id = $channel->getChannelId();

		$response = $this->sharePost($channel, $post, $userid);

		// Store message in log
		return PostHelper::savePost($response['state'], $response['result_msg'], $post, $userid);
	}

	/**
	 * sharePost
	 *
	 * @param   object  &$channel  Param
	 * @param   object  &$post     Param
	 * @param   int     $userid    Param
	 *
	 * @return	array
	 */
	protected function sharePost(&$channel, &$post, $userid = null)
	{
		// Check for duplicate post
		if ($this->dpcheck_enabled)
		{
			$isDuplicated = PostHelper::isDuplicatedPost(
					$post->id,
					$post->ref_id,
					$post->plugin,
					$post->channel_id,
					$post->message,
					$this->dpcheck_time_intval
			);

			if ($isDuplicated)
			{
				$this->logger->log(
						JLog::INFO, 'sendPost: duplicate post detection - message is already posted, article id = '
						. $post->ref_id . ', plugin = ' . $post->plugin . ', interval = ' . $this->dpcheck_time_intval
				);

				return array(
					'state' => AutotweetPostHelper::POST_ERROR,
					'result_msg' => 'COM_AUTOTWEET_ERROR_DUPLICATED'
				);
			}
		}

		// Get short url one time and if needed only (better performance)
		if ((AutotweetPostHelper::SHOWURL_OFF != $post->show_url)
			&& !empty($post->org_url)
			&& !isset($this->current_short_url))
		{
			$shorturlHelper = ShorturlHelper::getInstance();
			$this->current_short_url = $shorturlHelper->getShortUrl($post->org_url);
		}

		// Has Image in the Post and channel requires image?
		if ((isset($post->image_url)) && ($channel->getMediaMode() != 'message'))
		{
			$max_chars = $channel->getMaxCharsWithImage();
		}
		else
		{
			$max_chars = $channel->getMaxChars();
		}

		// Construct url and truncate message, if necessary
		$msg_url = TextHelper::getMessageWithURL(
				$post->message,
				$post->org_url,
				$this->current_short_url,
				$max_chars,
				$post->show_url,
				$this->shorturl_always
		);

		// Switch original url to short url to use short url also for all other links
		$post->url = $msg_url[0];
		$message = $msg_url[1];

		// Construct status message
		switch ($post->show_url)
		{
			case AutotweetPostHelper::SHOWURL_OFF:
				// Dont show url, do nothing
				break;
			case AutotweetPostHelper::SHOWURL_BEGINNING:
				// Show url at beginning of message
				$message = $post->url . ' ' . $message;
				break;
			case AutotweetPostHelper::SHOWURL_END:
				// Show url at end of message
				$message = $message . ' ' . $post->url;
				break;
		}
		return $this->sharePostChannel($message, $channel, $post);
	}

	/**
	 * sharePostChannel
	 *
	 * @param   string  $message   Param
	 * @param   object  &$channel  Param
	 * @param   object  &$post     Param
	 *
	 * @return	array
	 */
	protected function sharePostChannel($message, &$channel, &$post)
	{
		// Send message
		$attempt = 0;
		$response = array('state' => AutotweetPostHelper::POST_ERROR, 'result_msg' => 'COM_AUTOTWEET_ERR_CHANNEL_UNPROCESSED');

		// Post message for channel; multiple attempts if needed
		do
		{
			$resend = false;
			$attempt++;

			// Post to channel, et the result code
			$result_status = $channel->sendMessage($message, $post);

			if (($attempt < $this->resend_attempts) && !$result_status[0])
			{
				$resend = true;

				$this->logger->log(
						JLog::WARNING, 'sendRequest: ' . $channel->getChannelType() . ':' . $channel->getChannelName()
						. ' - service unavailable or timeout, return code = ' . $result_status[1]
						. ' - sending message again in ' . self::RESEND_DELAY . ' seconds'
				);

				sleep(self::RESEND_DELAY);
			}
		}
		while ($resend);

		$result_msg = $result_status[1];
		$response['result_msg'] = $result_msg;
		if ($result_status[0])
		{
			$response['state'] = AutotweetPostHelper::POST_SUCCESS;
			$this->logger->log(
					JLog::INFO, 'sendRequest: ' . $channel->getChannelType() . ':' . $channel->getChannelName()
					. ' - status has been updated, ref_id = ' . $post->ref_id . '  (attempts: ' . $attempt . ')'
			);
		}
		else
		{
			$response['state'] = AutotweetPostHelper::POST_ERROR;
			$this->logger->log(
					JLog::ERROR, 'sendRequest: error when sending message to ' . $channel->getChannelType() . ':'
					. $channel->getChannelName() . ', - ref_id = ' . $post->ref_id . ', return code = ' . $result_status[1]
			);

			TextHelper::adminNotification($channel->getChannelName(), $result_msg);
		}

		return $response;
	}

}
