<?php

/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - AutoTweetNG posts content to social channels (Twitter, Facebook, LinkedIn, etc).
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2012 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

/**
 * Helper for posts form AutoTweet to channels (twitter, Facebook, ...)
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
 */
abstract class TextHelper
{

	/**
	 * truncString.
	 *
	 * @param   string   $text       Param
	 * @param   integer  $length     Param
	 * @param   integer  $max_chars  Param
	 * @param   bool     $withDots   Param
	 *
	 * @return	string
	 *
	 * @since	1.5
	 */
	public static function truncString($text, $length, $max_chars, $withDots = false)
	{
		if ($length > $max_chars)
		{
			if ($withDots)
			{
				// -Dots
				// -2 Utf8 case
				$max_chars = $max_chars - 3 - 2;
			}
			$text = JString::substr($text, 0, $max_chars);

			// Yes, it can return more characters, but utf8_strlen($text) is Ok, so ...
			$l = $max_chars;
			while (strlen($text) > $max_chars)
			{
				$l--;
				$text = JString::substr($text, 0, $l);
			}

			if ($withDots)
			{
				$text = $text . '...';
			}
		}

		return $text;
	}

	/**
	 * cleanText.
	 *
	 * @param   string  $text  Param
	 *
	 * @return	string
	 *
	 * @since	1.5
	 */
	public static function cleanText($text)
	{
		return trim(strip_tags(JString::str_ireplace('&nbsp;', ' ', htmlspecialchars_decode($text))));
	}

	/**
	 * getMessageWithURL.
	 *
	 * @param   string   $message          Param
	 * @param   string   $long_url         Param
	 * @param   string   $short_url        Param
	 * @param   integer  $max_chars        Param
	 * @param   integer  $show_url         Param
	 * @param   boolean  $shorturl_always  Param
	 *
	 * @return	string
	 *
	 * @since	1.5
	 */
	public static function getMessageWithURL($message, $long_url, $short_url, $max_chars, $show_url, $shorturl_always)
	{
		// Shared functions with cronjob
		$url = '';

		// Is short url needed?
		if ((AutotweetPostHelper::SHOWURL_OFF != $show_url) && !empty($long_url))
		{
			$longmsg_len = JString::strlen($message) + JString::strlen($long_url) + 1;
			if (($shorturl_always) || ($longmsg_len > $max_chars))
			{
				$url = $short_url;
			}
			else
			{
				$url = $long_url;
			}
		}

		// Add space
		$url_len = JString::strlen($url) + 1;
		$msg_len = JString::strlen($message) + $url_len;

		// Trunc text if needed
		if ($msg_len > $max_chars)
		{
			// Needs min 3 chars for replacement with 3 dots
			$substr_len = $max_chars - $url_len - 3;
			$message = JString::substr($message, 0, $substr_len) . '...';
		}

		return array($url, $message);
	}

	/**
	 * adminNotification
	 *
	 * @param   string  $channel  Param
	 * @param   string  $msg      Param
	 *
	 * @return	boolean
	 *
	 * @since	1.5
	 */
	public static function adminNotification($channel, $msg)
	{
		if (!CparamsHelper::getParam('admin_notification', 0))
		{
			return;
		}

		JLoader::import('extly.mail.notification');

		$emailSubject = 'AutoTweetNG Notification - Error on Channel "' . $channel . '"';
		$emailBody = "<h2>Hi,</h2>
		<p>This is an <b>AutoTweetNG</b> Notification, about an error on channel \"{$channel}\".</p>
		<h3>Error Message</h3>
		<p>{$msg}</p>
		<p>If you are working in the configuration, it must be related with your work. However, if the site is stable, you should check if there's any problem (E.g. an expired token).</p>
		<p><br></p>
		<p>If you have any question, the support team is ready to answer!</p>
		<p>Best regards,</p>
		<p>Support Team<br> <b>Extly.com</b> - Extensions<br> Support: <a target=\"_blank\" href=\"http://support.extly.com\">http://support.extly.com</a><br> Twitter @extly | Facebook <a target=\"_blank\" href=\"http://www.facebook.com/extly\">www.facebook.com/extly</a></p>";
		Notification::mailToAdmin($emailSubject, $emailBody);
	}

}
