<?php

/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - AutoTweetNG posts content to social channels (Twitter, Facebook, LinkedIn, etc).
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2012 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

/**
 * UpdateHelper - Helper to get some infos about installed AutoTweet extensions.
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
*/
class UpdateHelper
{

	const EXT_NOTINSTALLED = 'COM_AUTOTWEET_STATE_PLUGIN_NOTINSTALLED';
	const EXT_DISABLED = 'COM_AUTOTWEET_STATE_PLUGIN_DISABLED';
	const EXT_ENABLED = 'COM_AUTOTWEET_STATE_PLUGIN_ENABLED';
	const EXT_UNKNOWN = 'COM_AUTOTWEET_VIEW_ABOUT_VERSIONINFO_UNKNOWN';

	/*
	 const SERVER_INI_FILE = 'autotweetng16-test.ini';
	*/

	const SERVER_INI_FILE = 'autotweetng16.ini';

	const SERVER_INI_PATH = 'http://www.extly.com/download/';

	const COMP_INSTALL_FILE = 'a_autotweet.xml';
	const KEY_COMP = 'component';
	const FM_EXT_SOURCE = 'Extly.com';

	private static $_compinfo = null;

	private static $_pluginfo = null;

	private static $_thirdparty = null;

	/**
	 * Helper to search an array recursive for a given value
	 *
	 * @param   string  $needle    Param
	 * @param   string  $haystack  Param
	 *
	 * @return	boolean
	 *
	 * @since	1.5
	 */
	private static function in_array_recursive($needle, $haystack)
	{
		$it = new RecursiveIteratorIterator(new RecursiveArrayIterator($haystack));

		foreach ($it AS $element)
		{
			if ($element == $needle)
			{
				return true;
			}
		}

		return false;
	}

	/**
	 * no public access (static class)
	 *
	 * @since	1.6
	 */
	private function UpdateHelper()
	{
		// Static class
	}

	/**
	 * getComponentInfo
	 *
	 * @return	array
	 *
	 * @since	1.6
	 */
	public static function getComponentInfo()
	{
		self::loadINI();
		return self::$_compinfo;
	}

	/**
	 * getPluginInfo
	 *
	 * @return	array
	 *
	 * @since	1.6
	 */
	public static function getPluginInfo()
	{
		self::loadINI();
		return self::$_pluginfo;
	}

	/**
	 * getThirdpartyInfo
	 *
	 * @return	array
	 *
	 * @since	1.6
	 */
	public static function getThirdpartyInfo()
	{
		self::loadINI();
		return self::$_thirdparty;
	}

	/**
	 * loadINI
	 *
	 * @return	bool
	 *
	 * @since	1.6
	 */
	protected static function loadINI()
	{
		if ((self::$_compinfo) && (self::$_pluginfo))
		{
			return true;
		}
		// Get component parameter
		$version_check = CparamsHelper::getParam('version_check', 1);

		$remoteFile = self::SERVER_INI_PATH . self::SERVER_INI_FILE;
		$localFile = JPATH_AUTOTWEET_HELPERS . '/' . self::SERVER_INI_FILE;
		if ($version_check)
		{
			$file = JInstallerHelper::downloadPackage($remoteFile, self::SERVER_INI_FILE);
			if ($file)
			{
				$config = JFactory::getConfig();
				JFile::move($config->get('tmp_path') . '/' . $file, $localFile);
				$file = $localFile;
			}
		}
		else
		{
			$file = $localFile;
		}

		jimport('joomla.error.error');
		jimport('joomla.registry.registry');
		$registry = new JRegistry;

		if (!$registry->loadFile($file, 'INI', array('processSections' => 'true')))
		{
			$logger->log(JLog::ERROR, 'AutoTweetNG - error reading INI file ' . $file);
			JError::raiseWarning('3', JText::sprintf('COM_AUTOTWEET_MSG_ERROR_READINGINIFILE', $file));
			return false;
		}

		// Init logging
		$logger = AutotweetLog::getInstance();

		$db = JFactory::getDBO();

		self::$_pluginfo = array();
		self::$_thirdparty = array();

		// Get component info and remove from array
		$data = JApplicationHelper::parseXMLInstallFile(JPATH_COMPONENT_ADMINISTRATOR . DIRECTORY_SEPARATOR . self::COMP_INSTALL_FILE);
		self::$_compinfo = array(
				'id' => $registry->get('component.id'),
				'name' => $registry->get('component.name'),
				'server_version' => $registry->get('component.version'),
				'client_version' => $data['version'],
				'home' => $registry->get('component.home'),
				'faq' => $registry->get('component.faq'),
				'download' => $registry->get('component.download'),
				'support' => $registry->get('component.support'),
				'products' => $registry->get('component.products'),
				'twitter' => $registry->get('component.twitter'),
				'jed' => $registry->get('component.jed'),
				'message' => $registry->get('component.message'),
				'news' => $registry->get('component.news')
		);
		$extensions = explode(',', $registry->get('component.extensions'));

		foreach ($extensions as $extension)
		{
			$state = self::EXT_NOTINSTALLED;
			$config = '';
			$client_version = '';
			$type = $registry->get($extension . '.type');
			$id = $registry->get($extension . '.id');
			$source = $registry->get($extension . '.source');

			if ('module' == $type)
			{
				$mod_filename = 'mod_' . $id;

				// Get the module id and set url for config
				$query = 'SELECT * FROM ' . $db->quoteName('#__modules')
				. ' WHERE ' . $db->quoteName('module') . ' = ' . $db->Quote($mod_filename);
				$db->setQuery($query);
				$row = $db->loadObject();

				if (!empty($row))
				{
					$path = JPATH_ADMINISTRATOR . '/modules/' . $mod_filename . DIRECTORY_SEPARATOR . $mod_filename . '.xml';
					$data = JApplicationHelper::parseXMLInstallFile($path);
					$client_version = $data['version'];
					if (JModuleHelper::isEnabled($id))
					{
						$state = self::EXT_ENABLED;
					}
					else
					{
						$state = self::EXT_DISABLED;
					}

					$config = 'index.php?option=com_modules&task=module.edit&id=' . $row->id;
				}
			}
			elseif ('cb_user' == $type)
			{
				$pluginsModel = FOFModel::getTmpInstance('Plugins', 'AutoTweetModel');
				$pluginsModel->savestate(false)->set('element_id', $id);
				$row = $pluginsModel->getItemList();

				if (!empty($row))
				{
					$row = $row[0];
					$path = JPATH_ROOT . '/components/com_comprofiler/plugin/' . $row->folder . DIRECTORY_SEPARATOR . $row->element . DIRECTORY_SEPARATOR . $row->element . '.xml';
					$xml = JFactory::getXML($path);

					if ($xml)
					{
						$client_version = $xml->getAttribute('version');

						if (0 < (int) $row->published)
						{
							$state = self::EXT_ENABLED;
						}
						else
						{
							$state = self::EXT_DISABLED;
						}

						$config = 'index.php?option=com_comprofiler&task=editPlugin&cid=' . $row->id;
					}
				}
			}
			else
			{
				// Get the plugin id and set url for config
				$pluginsModel = FOFModel::getTmpInstance('Plugins', 'AutoTweetModel');
				$pluginsModel->savestate(false)->set('element_id', $id);
				$row = $pluginsModel->getItemList();

				if (!empty($row))
				{
					$row = $row[0];
					$path = JPATH_PLUGINS . DIRECTORY_SEPARATOR . $row->folder . DIRECTORY_SEPARATOR . $row->element . DIRECTORY_SEPARATOR . $row->element . '.xml';
					$data = JApplicationHelper::parseXMLInstallFile($path);

					$client_version = $data['version'];

					if (JPluginHelper::isEnabled($row->folder, $row->element))
					{
						$state = self::EXT_ENABLED;
					}
					else
					{
						$state = self::EXT_DISABLED;
					}

					$config = 'index.php?option=com_plugins&task=plugin.edit&extension_id=' . $row->id;
				}
			}

			// Append plugin state to result arrays
			if (self::FM_EXT_SOURCE == $source)
			{
				self::$_pluginfo[] = array(
						'id' => $id,
						'name' => $registry->get($extension . '.name'),
						'state' => $state,
						'client_version' => $client_version,
						'server_version' => $registry->get($extension . '.version'),
						'message' => $registry->get($extension . '.message'),
						'config' => $config
				);
			}
			else
			{
				self::$_thirdparty[] = array(
						'id' => $id,
						'name' => $registry->get($extension . '.name'),
						'state' => $state,
						'client_version' => $client_version,
						'message' => $registry->get($extension . '.message'),
						'config' => $config,
						'source' => $source,
						'download' => $registry->get($extension . '.download')
				);
			}
		}

		// Add installed plugins without entry in ini file to 3rd party list
		$pluginsModel = FOFModel::getTmpInstance('Plugins', 'AutoTweetModel');
		$pluginsModel->savestate(false);
		$plugins = $pluginsModel->getItemList();
		foreach ($plugins as $plugin)
		{
			$id = $plugin->element;
			$type = $plugin->folder;

			if (!self::in_array_recursive($id, self::$_pluginfo)
				&& !self::in_array_recursive($id, self::$_thirdparty))
			{
				$path = JPATH_PLUGINS . DIRECTORY_SEPARATOR . $type . DIRECTORY_SEPARATOR . $id . DIRECTORY_SEPARATOR . $id . '.xml';
				$data = JApplicationHelper::parseXMLInstallFile($path);

				$client_version = $data['version'];

				if (JPluginHelper::isEnabled($type, $id))
				{
					$state = self::EXT_ENABLED;
				}
				else
				{
					$state = self::EXT_DISABLED;
				}
				$config = 'index.php?option=com_plugins&task=plugin.edit&extension_id=' . $plugin->id;

				if (!empty($data['authorUrl']))
				{
					$source = $data['authorUrl'];
					$download = $data['authorUrl'];
				}
				else
				{
					$source = self::EXT_UNKNOWN;
					$download = '';
				}

				self::$_thirdparty[] = array(
						'id' => $id,
						'name' => $plugin->name,
						'state' => $state,
						'client_version' => $client_version,
						'message' => 'unknown extension plugin',
						'config' => $config,
						'source' => $source,
						'download' => $download
				);
			}

		}
		return true;
	}

}
