<?php

/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - AutoTweetNG posts content to social channels (Twitter, Facebook, LinkedIn, etc).
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2012 Prieco, S.A. All rights reserved.
 * @license     http://http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

/**
 * AutotweetTableRule
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
*/
class AutotweetTableRule extends FOFTable
{

	/**
	 * Instantiate the table object
	 *
	 * @param   string     $table  Param
	 * @param   string     $key    Param
	 * @param   JDatabase  &$db    The Joomla! database object
	 */
	public function __construct($table, $key, &$db)
	{
		parent::__construct('#__autotweet_rules', 'id', $db);

		$this->_columnAlias = array(
				'enabled'		=> 'published',
				'created_on'	=> 'created',
				'modified_on'	=> 'modified',
				'locked_on'		=> 'checked_out_time',
				'locked_by'		=> 'checked_out',
		);

	}

	/**
	 * onAfterLoad
	 *
	 * @param   bool  &$result  Param
	 *
	 * @return	void
	 */
	protected function onAfterLoad(&$result)
	{
		if ($this->id == 0)
		{
			$this->published = true;
			$this->autopublish = 'on';
			$this->show_url = 'end_of_message';
			$this->show_static_text = 'off';
		}
	}

	/**
	 * onAfterLoad
	 *
	 * @param   array  $cid  Param
	 *
	 * @return	bool
	 */
	public function copy($cid = null)
	{
		JArrayHelper::toInteger($cid);
		$user_id	= (int) $user_id;
		$k			= $this->_tbl_key;

		if (count($cid) < 1)
		{
			if ($this->$k)
			{
				$cid = array($this->$k);
			}
			else
			{
				$this->setError("No items selected.");
				return false;
			}
		}

		$created_by		= $this->getColumnAlias('created_by');
		$created_on		= $this->getColumnAlias('created_on');
		$modified_by	= $this->getColumnAlias('modified_by');
		$modified_on	= $this->getColumnAlias('modified_on');

		$locked_byName	= $this->getColumnAlias('locked_by');
		$checkin 		= in_array($locked_byName, array_keys($this->getProperties()));

		foreach ($cid as $item)
		{
			// Prevent load with id = 0
			if (!$item)
			{
				continue;
			}

			$this->load($item);

			if ($checkin)
			{
				// We're using the checkin and the record is used by someone else
				if ($this->isCheckedOut($item))
				{
					continue;
				}
			}

			if (!$this->onBeforeCopy($item))
			{
				continue;
			}

			$this->$k 			= null;
			$this->$created_by 	= null;
			$this->$created_on 	= null;
			$this->$modified_on	= null;
			$this->$modified_by = null;

			// Let's fire the event only if everything is ok
			if ($this->store())
			{
				$this->onAfterCopy($item);
			}

			$this->reset();
		}

		return true;
	}

}
