<?php

/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - AutoTweetNG posts content to social channels (Twitter, Facebook, LinkedIn, etc).
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2012 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

/**
 * AutoTweetToolbar
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
*/
class AutotweetToolbar extends FOFToolbar
{

	protected $isModule = false;

	/**
	 * Class constructor
	 *
	 * @param   array  $config  Configuration parameters
	 */
	public function __construct($config = array())
	{
		parent::__construct($config);

		$layout = FOFInput::getCmd('layout', null, $this->input);
		$this->isModule = ($layout == 'module');
	}

	/**
	 * getMyViews.
	 *
	 * @return	array
	 *
	 * @since	1.5
	 */
	protected function getMyViews()
	{
		$views = array(
				'cpanels',
				'requests',
				'rules',
				'posts',
				'channels',
				'infos'
		);
		return $views;
	}

	/**
	 * onCpanelBrowse.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function onCpanelsBrowse()
	{
		$this->_onAllPages();
		JToolBarHelper::title(VersionHelper::getTitle(JText::_('COM_AUTOTWEET_TITLE_CPANELS')), 'autotweet');
		JToolBarHelper::preferences('com_autotweet', '600', '900');
	}

	/**
	 * onPostsBrowse.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function onPostsBrowse()
	{
		if (!$this->isModule)
		{
			$this->_onAllPages();
			JToolBarHelper::title(VersionHelper::getTitle(JText::_('COM_AUTOTWEET_TITLE_POSTS')), 'autotweet-logo.png');

			// Add toolbar buttons
			if ($this->perms->create)
			{
				JToolBarHelper::addNew();
			}
			if ($this->perms->edit)
			{
				JToolBarHelper::editList();
			}
			if ($this->perms->create || $this->perms->edit)
			{
				JToolBarHelper::divider();
			}

			if ($this->perms->editstate)
			{
				JToolBarHelper::publishList();
				JToolBarHelper::unpublishList();
				JToolBarHelper::divider();
			}
			if ($this->perms->create)
			{
				JToolBarHelper::custom('copy', 'copy.png', 'copy_f2.png', 'COM_AUTOTWEET_COMMON_COPY_LABEL', false);
				JToolBarHelper::divider();
			}
			if ($this->perms->delete)
			{
				$msg = JText::_(FOFInput::getCmd('option', 'com_autotweet', $this->input) . '_CONFIRM_DELETE');
				JToolBarHelper::deleteList($msg);
			}
		}
	}

	/**
	 * onRequestsBrowse.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function onRequestsBrowse()
	{
		$this->_onAllPages();
		JToolBarHelper::title(VersionHelper::getTitle(JText::_('COM_AUTOTWEET_TITLE_REQUESTS')), 'autotweet-logo.png');

		if ($this->perms->create)
		{
			JToolBarHelper::addNew();
		}
		if ($this->perms->edit)
		{
			JToolBarHelper::editList();
		}
		if ($this->perms->create || $this->perms->edit)
		{
			JToolBarHelper::divider();
		}

		if ($this->perms->editstate)
		{
			JToolBarHelper::custom('process', 'process.png', 'process.png', 'COM_AUTOTWEET_COMMON_PROCESS_LABEL', false);
			JToolBarHelper::divider();
		}

		if ($this->perms->create)
		{
			JToolBarHelper::custom('copy', 'copy.png', 'copy_f2.png', 'COM_AUTOTWEET_COMMON_COPY_LABEL', false);
			JToolBarHelper::divider();
		}
		if ($this->perms->delete)
		{
			$msg = JText::_(FOFInput::getCmd('option', 'com_autotweet', $this->input) . '_CONFIRM_DELETE');
			JToolBarHelper::deleteList($msg);
		}
	}

	/**
	 * onRequestsAdd.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function onRequestsAdd()
	{
		parent::onAdd();
		JToolBarHelper::title(VersionHelper::getTitle(JText::_('COM_AUTOTWEET_TITLE_REQUEST_EDIT')), 'autotweet-logo.png');
	}

	/**
	 * onChannelsBrowse.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function onChannelsBrowse()
	{
		$this->_onAllPages();
		JToolBarHelper::title(VersionHelper::getTitle(JText::_('COM_AUTOTWEET_TITLE_CHANNELS')), 'autotweet-logo.png');
		$this->_onBrowseWithCopy();
	}

	/**
	 * onChannelAdd.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function onChannelsAdd()
	{
		parent::onAdd();
		JToolBarHelper::title(VersionHelper::getTitle(JText::_('COM_AUTOTWEET_TITLE_CHANNELS_EDIT')), 'autotweet-logo.png');
	}

	/**
	 * onRulesBrowse.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function onRulesBrowse()
	{
		$this->_onAllPages();
		JToolBarHelper::title(VersionHelper::getTitle(JText::_('COM_AUTOTWEET_TITLE_RULES')), 'autotweet-logo.png');
		$this->_onBrowseWithCopy();
	}

	/**
	 * onRulesAdd.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function onRulesAdd()
	{
		parent::onAdd();
		JToolBarHelper::title(VersionHelper::getTitle(JText::_('COM_AUTOTWEET_TITLE_RULE_EDIT')), 'autotweet-logo.png');
	}

	/**
	 * onPostBrowse.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function onPostBrowse()
	{
		$this->_onAllPages();
		JToolBarHelper::title(VersionHelper::getTitle(JText::_('COM_AUTOTWEET_TITLE_POST')), 'autotweet-logo.png');
	}

	/**
	 * onPostsAdd.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function onPostsAdd()
	{
		parent::onAdd();
		JToolBarHelper::title(VersionHelper::getTitle(JText::_('COM_AUTOTWEET_TITLE_POST_EDIT')), 'autotweet-logo.png');
	}

	/**
	 * onInfoBrowse.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function onInfoBrowse()
	{
		$this->_onAllPages();
		JToolBarHelper::title(VersionHelper::getTitle(JText::_('COM_AUTOTWEET_TITLE_INFOS')), 'autotweet-logo.png');
	}

	/**
	 * _onAllPages.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function _onAllPages()
	{
		// On frontend, buttons must be added specifically
		list($isCli, $isAdmin) = FOFDispatcher::isCliAdmin();

		if ($isAdmin || $this->renderFrontendSubmenu)
		{
			$this->renderSubmenu();
		}

		if (!$isAdmin && !$this->renderFrontendButtons)
		{
			return;
		}
	}

	/**
	 * _onBrowseWithCopy.
	 *
	 * @param   bool  $allowCopy  Param.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function _onBrowseWithCopy($allowCopy = true)
	{
		// Add toolbar buttons
		if ($this->perms->create)
		{
			JToolBarHelper::addNew();
		}
		if ($this->perms->edit)
		{
			JToolBarHelper::editList();
		}
		if ($this->perms->create || $this->perms->edit)
		{
			JToolBarHelper::divider();
		}

		if ($this->perms->editstate)
		{
			JToolBarHelper::publishList();
			JToolBarHelper::unpublishList();
			JToolBarHelper::divider();
		}
		if (($allowCopy) && ($this->perms->create))
		{
			JToolBarHelper::custom('copy', 'copy.png', 'copy_f2.png', 'COM_AUTOTWEET_COMMON_COPY_LABEL', false);
			JToolBarHelper::divider();
		}
		if ($this->perms->delete)
		{
			$msg = JText::_(FOFInput::getCmd('option', 'com_autotweet', $this->input) . '_CONFIRM_DELETE');
			JToolBarHelper::deleteList($msg);
		}

		// $this->renderSubmenu();
	}

	/*

	$status = FOFModel::getTmpInstance('Users','TestModel')
	->setId($id)
	->getItem()
	->save($data);

	$table->reset();
	$table->load($row->$k);
	$table->save($row);

	*/

}
