<?php

/**
 * @package     Extly.Components
 * @subpackage  plgSystemKmAutoTweet - Post comments to AutoTweet
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2012 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

// Check for component
if (!JComponentHelper::getComponent('com_autotweet', true)->enabled)
{
	JError::raiseWarning('5', 'AutoTweetNG for Komento-Plugin - AutoTweetNG Component is not installed or not enabled.');
	return;
}

require_once JPATH_ROOT . '/administrator/components/com_autotweet/helpers/autotweetbase.php';

/**
 * plgSystemKmAutoTweet class.
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
 */
class PlgSystemKmAutoTweet extends plgAutotweetBase
{

	const TYPE_FIRSTCOMMENT = 1;
	const TYPE_REPLY = 2;

	protected $onlyfirst = 0;

	protected $templatecomment = null;

	protected $templatereply = null;

	protected $use_text_count = null;

	/**
	 * Constructor.
	 *
	 * @param   string  &$subject  Param
	 * @param   array   $config    Configuration
	 *
	 * @since	1.5
	 */
	public function plgSystemKmAutoTweet(&$subject, $config)
	{
		parent::__construct($subject, $config);

		$pluginParams = $this->params;
		if ($pluginParams)
		{
			$this->onlyfirst = $pluginParams->get('onlyfirst', 0);
			$this->templatecomment = $pluginParams->get('templatecomment', 'New comment: [title]!');
			$this->templatereply = $pluginParams->get('templatereply', 'New comment: [title]!');
			$this->use_text_count = $pluginParams->get('use_text_count', 100);
		}
	}

	/**
	 * onAfterKomentoProcessNewComment
	 *
	 * @param   array  $data  New Comment
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function onAfterKomentoProcessNewComment($data)
	{
		// Get object's owner id
		$owner = $data['comment']->created_by;

		// Check that this is first comment to the object
		$isfirst = ($data['comment']->lft == 1);

		$url = JRequest::getString('HTTP_REFERER', null, 'SERVER');
		if ($isfirst)
		{
			$typeinfo = TYPE_FIRSTCOMMENT;
		}
		else
		{
			$typeinfo = TYPE_REPLY;
		}

		$message = $this->_cleanText($data['comment']->comment);
		$data['message'] = $message;
		$this->postData($data['comment']->id, JFactory::getDate()->toSql(), $typeinfo, $message, $url, $data);
	}

	/**
	 * postData
	 *
	 * @param   integer  $commentid  Param
	 * @param   date     $date       Param
	 * @param   int      $typeinfo   Param
	 * @param   string   $message    Param
	 * @param   string   $url        Param
	 * @param   string   &$data      Param
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function postData($commentid, $date, $typeinfo, $message, $url, &$data)
	{
		$image_url = $this->getImageFromText($data['html']);

		$native_object = json_encode($data);
		$this->postStatusMessage($commentid, $date, $message, $typeinfo, $url, $image_url, $native_object);
	}

	/**
	 * getExtendedData
	 *
	 * @param   string  $id              Param.
	 * @param   string  $typeinfo        Param.
	 * @param   string  &$native_object  Param.
	 *
	 * @return	array
	 *
	 * @since	1.5
	 */
	public function getExtendedData($id, $typeinfo, &$native_object)
	{
		if ($typeinfo == self::TYPE_FIRSTCOMMENT)
		{
			$template = $this->templatecomment;
		}
		else
		{
			$template = $this->templatereply;
		}

		$data = json_decode($native_object, true);
		$comment = $data['comment'];

		$message = $data['message'];
		$title = $message;

		$message = $template;
		$message = str_replace("[title]", $title, $message);
		$message = $this->getMessagetext(0, $this->use_text_count, $message, $message);

		$config = new JConfig;

		// Return values
		$data = array(
			'title' => $title,
			'text' => $message,

			// 'hashtags'		=> $hashtags,

			// Already done when msg is inserted in queue
			'url' => null,

			// Already done when msg is inserted in queue
			'image_url' => null,

			'fulltext' => $config->MetaTitle,

			// 'catids'		=> $catids,
			'author' => $this->getAuthorUsername($comment['created_by']),
			'is_valid' => true
		);

		return $data;
	}

	/**
	 * cleanText.
	 *
	 * @param   string  $text  Param
	 *
	 * @return	string
	 *
	 * @since	1.5
	 */
	private function _cleanText($text)
	{
		return trim(strip_tags(JString::str_ireplace('&nbsp;', ' ', htmlspecialchars_decode($text))));
	}

}
