/**
 * @package Extly.Components
 * @subpackage com_autotweet - AutoTweet posts content to social channels
 *             (Twitter, Facebook, LinkedIn, etc).
 * 
 * @author Prieco S.A.
 * @copyright Copyright (C) 2007 - 2012 Prieco, S.A. All rights reserved.
 * @license http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link http://www.extly.com http://support.extly.com
 */

/* jslint plusplus: true, browser: true, sloppy: true */
/* global jQuery, Request, Joomla, alert, Backbone */

var FbValidationView = Backbone.View.extend({
	events : {
		'change #use_own_api' : 'onChangeOwnApi',
		'click #authbutton' : 'onAuthorizationReq',
		'click #fbvalidationbutton' : 'onValidationReq'
	},

	initialize : function() {
		var view = this;

		this.collection.on('add', this.loadvalidation, this);

		this.$el.ajaxStart(function() {
			view.$(".loaderspinner").addClass('loading');
		}).ajaxStop(function() {
			view.$(".loaderspinner").removeClass('loading');
		});
	},

	onChangeOwnApi : function onChangeChannelType(e) {
		var ownApp = this.$('#use_own_api option:selected').val();

		if (ownApp === '1') {
			this.$('#own-app-details').fadeIn();
		} else {
			this.$('#own-app-details').fadeOut();
		}
	},

	onAuthorizationReq : function onAuthorizationReq(e) {
		var params = appParamsHelper.get(this);

		e.preventDefault();

		var canvas_page = params.p_canvas_page + '?' + params.p_url_params;
		window.open(canvas_page);
	},

	onValidationReq : function onValidationReq() {
		var view = this, params = appParamsHelper.get(view);

		this.collection.create(this.collection.model, {
			attrs : {
				own_app : params.p_own_app,
				app_id : params.p_app_id,
				secret : params.p_secret,
				access_token : params.p_access_token
			},
			urlBase : (view.options.urlBase || ''),
			urlTest : (view.options.urlTest || null),

			wait : true,
			dataType:     'text',
			error : function(model, fail, xhr) {
				validationHelper.showError(view, fail.responseText);
			}
		});
	},

	loadvalidation : function loadvalidation(resp) {
		var status = resp.get('status'), error_message = resp
				.get('error_message'), user = resp.get('user');

		if (status) {
			validationHelper.showSuccess(this, user.id);
			this.options.dispatcher.trigger("fbapp:channelschanged");
		} else {
			validationHelper.showError(this, error_message);
		}
	}

});
