<?php

/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - AutoTweetNG posts content to social channels (Twitter, Facebook, LinkedIn, etc).
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2012 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */

/**
 * Based on script.akeeba.php / AkeebaBackup
 * Copyright (c)2009-2013 Nicholas K. Dionysopoulos
 * GNU General Public License version 3, or later
 */

defined('_JEXEC') or die();

jimport('joomla.filesystem.folder');
jimport('joomla.filesystem.file');

class Com_AutoTweetInstallerScript
{
	/** @var string The component's name */
	protected $_extly_extension = 'com_autotweet';

	/** @var array The list of extra modules and plugins to install */
	private $installation_queue = array(

			// modules => { (folder) => { (module) => { (position), (published) } }* }*
			'modules' => array(
					'admin' => array(
							'autotweet_latest' => array('cpanel', 0),
					),
					'site' => array(
					)
			),

			// plugins => { (folder) => { (element) => (published) }* }*
			'plugins' => array(
					'system' => array(
							'autotweetautomator' => 1,
							'autotweetcontent'	=> 1
					),
					'autotweet' => array(
							'autotweetpost' => 1
					),
					'content' => array(
							'autotweetweblinks' => 0
					),
			),

	);

	/** @var array Obsolete files and folders to remove from the Core release only */
	private $extlyRemoveFilesCore = array(
			'files'	=> array(
					// FOF 1.x files
					'libraries/fof/controller.php',
					'libraries/fof/dispatcher.php',
					'libraries/fof/inflector.php',
					'libraries/fof/input.php',
					'libraries/fof/model.php',
					'libraries/fof/query.abstract.php',
					'libraries/fof/query.element.php',
					'libraries/fof/query.mysql.php',
					'libraries/fof/query.mysqli.php',
					'libraries/fof/query.sqlazure.php',
					'libraries/fof/query.sqlsrv.php',
					'libraries/fof/render.abstract.php',
					'libraries/fof/render.joomla.php',
					'libraries/fof/render.joomla3.php',
					'libraries/fof/render.strapper.php',
					'libraries/fof/string.utils.php',
					'libraries/fof/table.php',
					'libraries/fof/template.utils.php',
					'libraries/fof/toolbar.php',
					'libraries/fof/view.csv.php',
					'libraries/fof/view.html.php',
					'libraries/fof/view.json.php',
					'libraries/fof/view.php',

					// AutoTweetNG 6.3, and before
					'administrator/components/com_autotweet/autotweet.xml',
					'administrator/components/com_autotweet/controller.php',
					'administrator/components/com_autotweet/controllers/account.json.php',
					'administrator/components/com_autotweet/controllers/account.php',
					'administrator/components/com_autotweet/controllers/account.raw.php',
					'administrator/components/com_autotweet/controllers/accounts.php',
					'administrator/components/com_autotweet/controllers/accounts.raw.php',
					'administrator/components/com_autotweet/controllers/autotweetentries.php',
					'administrator/components/com_autotweet/controllers/autotweetentry.php',
					'administrator/components/com_autotweet/controllers/autotweet.php',
					'administrator/components/com_autotweet/controllers/autotweet.raw.php',
					'administrator/components/com_autotweet/controllers/dashboard.php',
					'administrator/components/com_autotweet/controllers/fbwizzardbaseaccount.php',
					'administrator/components/com_autotweet/controllers/fbwizzardbaseaccounts.php',
					'administrator/components/com_autotweet/controllers/fbwizzardbaseaccounts.raw.php',
					'administrator/components/com_autotweet/controllers/manualmessage.php',
					'administrator/components/com_autotweet/controllers/queue.php',
					'administrator/components/com_autotweet/controllers/rule.php',
					'administrator/components/com_autotweet/controllers/rules.php',
					'administrator/components/com_autotweet/helpers/autotweetcronjob.php',
					'administrator/components/com_autotweet/helpers/autotweetextaccesshelper.php',
					'administrator/components/com_autotweet/helpers/autotweetguihelper.php',
					'administrator/components/com_autotweet/helpers/autotweetinfohelper.php',
					'administrator/components/com_autotweet/helpers/autotweetng16-test.ini',
					'administrator/components/com_autotweet/helpers/autotweetng.ini',
					'administrator/components/com_autotweet/helpers/autotweetpluginfactory.php',
					'administrator/components/com_autotweet/helpers/autotweetroutehelper.php',
					'administrator/components/com_autotweet/helpers/autotweetruleengine.php',
					'administrator/components/com_autotweet/helpers/autotweetutil.php',
					'administrator/components/com_autotweet/helpers/autotweetview.php',
					'administrator/components/com_autotweet/helpers/channels/autotweetaccountfactory.php',
					'administrator/components/com_autotweet/helpers/channels/autotweetchannelfactoryhelper.php',
					'administrator/components/com_autotweet/helpers/channels/autotweetchannel.php',
					'administrator/components/com_autotweet/helpers/channels/autotweetchannelfactory',
					'administrator/components/com_autotweet/helpers/channels/autotweetfacebookaccount.php',
					'administrator/components/com_autotweet/helpers/channels/autotweetfacebookbase.php',
					'administrator/components/com_autotweet/helpers/channels/autotweetfacebookevent.php',
					'administrator/components/com_autotweet/helpers/channels/autotweetfacebooklink.php',
					'administrator/components/com_autotweet/helpers/channels/autotweetfacebook.php',
					'administrator/components/com_autotweet/helpers/channels/autotweetlinkedinbase.php',
					'administrator/components/com_autotweet/helpers/channels/autotweetlinkedingroup.php',
					'administrator/components/com_autotweet/helpers/channels/autotweetlinkedin.php',
					'administrator/components/com_autotweet/helpers/channels/autotweetmail.php',
					'administrator/components/com_autotweet/helpers/channels/autotweetpingfm.php',
					'administrator/components/com_autotweet/helpers/channels/autotweetseesmicping.php',
					'administrator/components/com_autotweet/helpers/channels/autotweettwitter.php',
					'administrator/components/com_autotweet/helpers/channels/fmhttpcodehelper.php',
					'administrator/components/com_autotweet/helpers/fmregistryhelper.php',
					'administrator/components/com_autotweet/helpers/urlshortservices/autotweetshortservicefactory.php',
					'administrator/components/com_autotweet/install.autotweet.php',
					'administrator/components/com_autotweet/installer.php',
					'administrator/components/com_autotweet/models/account.php',
					'administrator/components/com_autotweet/models/accounts.php',
					'administrator/components/com_autotweet/models/autotweetentries.php',
					'administrator/components/com_autotweet/models/autotweetentry.php',
					'administrator/components/com_autotweet/models/autotweet.php',
					'administrator/components/com_autotweet/models/fbwizzardbaseaccount.php',
					'administrator/components/com_autotweet/models/fbwizzardbaseaccounts.php',
					'administrator/components/com_autotweet/models/info.php',
					'administrator/components/com_autotweet/models/manualmessage.php',
					'administrator/components/com_autotweet/models/queueentry.php',
					'administrator/components/com_autotweet/models/queue.php',
					'administrator/components/com_autotweet/models/rule.php',
					'administrator/components/com_autotweet/sql/install.sql',
					'administrator/components/com_autotweet/sql/uninstall.sql',
					'administrator/components/com_autotweet/tables/autotweetaccount.php',
					'administrator/components/com_autotweet/tables/autotweetautomator.php',
					'administrator/components/com_autotweet/tables/autotweetchannel.php',
					'administrator/components/com_autotweet/tables/autotweetchanneltype.php',
					'administrator/components/com_autotweet/tables/autotweetmsglog.php',
					'administrator/components/com_autotweet/tables/autotweetqueue.php',
					'administrator/components/com_autotweet/tables/autotweetrule.php',
					'administrator/components/com_autotweet/tables/autotweetruletype.php',
					'administrator/components/com_autotweet/uninstall.autotweet.php',
					'administrator/components/com_autotweet/views/rule/tmpl/default.php',
					'administrator/components/com_autotweet/views/rule/tmpl/edit.php'

			),
			'folders' => array(
					'administrator/components/com_autotweet/assets',
					'administrator/components/com_autotweet/cronjob',
					'administrator/components/com_autotweet/help',
					'administrator/components/com_autotweet/helpers/channels/atfacebook',
					'administrator/components/com_autotweet/helpers/channels/atlinkedin',
					'administrator/components/com_autotweet/helpers/channels/atmailer',
					'administrator/components/com_autotweet/helpers/channels/atpingfm',
					'administrator/components/com_autotweet/helpers/channels/attwitter',
					'administrator/components/com_autotweet/helpers/jsonwrapper',
					'administrator/components/com_autotweet/language',
					'administrator/components/com_autotweet/models/fields',
					'administrator/components/com_autotweet/models/forms',
					'administrator/components/com_autotweet/views/account',
					'administrator/components/com_autotweet/views/accounts',
					'administrator/components/com_autotweet/views/autotweet',
					'administrator/components/com_autotweet/views/autotweetentries',
					'administrator/components/com_autotweet/views/autotweetentry',
					'administrator/components/com_autotweet/views/dashboard',
					'administrator/components/com_autotweet/views/fbwizzardbaseaccount',
					'administrator/components/com_autotweet/views/fbwizzardbaseaccounts',
					'administrator/components/com_autotweet/views/info',
					'administrator/components/com_autotweet/views/manualmessage',
					'administrator/components/com_autotweet/views/queue'
				)
	);

	/** @var array Obsolete files and folders to remove from the Core and Pro releases */
	private $extlyRemoveFilesPro = array(
			'files'	=> array(
			),
			'folders' => array(
			)
	);

	private $extlyCliScripts = array(
			'autotweetstartcronjob.php'
	);

	protected $_broken510 = false;


	/**
	 * Joomla! pre-flight event
	 *
	 * @param string $type Installation type (install, update, discover_install)
	 * @param JInstaller $parent Parent object
	*/
	public function preflight($type, $parent)
	{
		// Bugfix for "Can not build admin menus"
		if(in_array($type, array('install','discover_install'))) {
			$this->_bugfixDBFunctionReturnedNoError();
		} else {
			$this->_bugfixCantBuildAdminMenus();
		}

		// Only allow to install on Joomla! 2.5.0 or later
		$check = version_compare(JVERSION, '2.5.0', 'ge');

		if ($check)
		{
			if ($type == 'update')
			{
				$this->_uninstallAutotweetPlugins($parent);

				$this->_broken510 = $this->_isBroken510();
			}
		}
		return $check;
	}

	/**
	 * _uninstallAutotweetPlugins.
	 *
	 * @param   object  $parent  Params.
	 *
	 * @return	boolean.
	 *
	 * @since	1.5
	 */
	public function _uninstallAutotweetPlugins($parent)
	{
		$db = JFactory::getDBO();

		// Uninstall additional elements (plugins and modules)

		$inst = new JInstaller;
		$query = 'SELECT * FROM ' . $db->quoteName('#__extensions')
		. ' WHERE (' .
		// $db->quoteName('type') . ' = ' . $db->Quote('package') . ' OR ' .
		$db->quoteName('type') . ' = ' . $db->Quote('plugin') . ' OR ' .
		$db->quoteName('type') . ' = ' . $db->Quote('module') . ') AND (' .
		$db->quoteName('element') . ' like ' . $db->Quote('%autotweet%') . ' OR ' .
		$db->quoteName('name') . ' like ' . $db->Quote('%AutoTweet%') . ')' .
		' ORDER BY ' . $db->quoteName('extension_id');

		$db->setQuery($query);
		$extensions = $db->loadAssocList();

		foreach ($extensions as $ext)
		{
			$inst_result = $inst->uninstall($ext['type'], $ext['extension_id']);
			if (!$inst_result)
			{
				echo '<p>' . JText::sprintf('COM_AUTOTWEET_UNINSTALL_EXT_FAILED', $ext['name']) . '</p>';
			}
			else
			{
				echo '<p>' . JText::sprintf('COM_AUTOTWEET_UNINSTALL_EXT_SUCCESS', $ext['name']) . '</p>';
			}
		}

		if (file_exists(JPATH_ROOT . '/administrator/components/com_comprofiler/plugin.foundation.php'))
		{
			// Special code to uninstall cb plugin
			$query = 'SELECT * FROM ' . $db->quoteName('#__comprofiler_plugin')
			. ' WHERE ' . $db->quoteName('element') . ' = ' . $db->Quote('autotweetcb');
			$db->setQuery($query);
			$row = $db->loadObject();

			if (!empty($row))
			{
				include_once JPATH_ROOT . '/administrator/components/com_comprofiler/plugin.foundation.php';
				include_once JPATH_ROOT . '/administrator/components/com_comprofiler/plugin.class.php';
				include_once JPATH_ROOT . '/administrator/components/com_comprofiler/comprofiler.class.php';
				cbimport('cb.installer');

				$id = $row->id;
				$inst = new cbInstallerPlugin;
				$inst_result = $inst->uninstall($id, 'com_comprofiler');

				if (!$inst_result)
				{
					echo '<p>' . JText::sprintf('COM_AUTOTWEET_UNINSTALL_EXT_FAILED', 'Community Builder Plugin') . '</p>';
				}
				else
				{
					echo '<p>' . JText::sprintf('COM_AUTOTWEET_UNINSTALL_EXT_SUCCESS', 'Community Builder Plugin') . '</p>';
				}
			}
		}

		echo '<p><strong>' . JText::_('COM_AUTOTWEET_UNINSTALL_SUCCESS') . '</strong></p>';
	}

	/**
	 * Runs after install, update or discover_update
	 * @param string $type install, update or discover_update
	 * @param JInstaller $parent
	 */
	function postflight( $type, $parent )
	{
		// Install subextensions
		$status = $this->_installSubextensions($parent);

		// Remove obsolete files and folders
		/*
		$isAutotweetPro = is_dir($parent->getParent()->getPath('source').'/plugins/system/srp');
		if($isAutotweetPro) {
		$extlyRemoveFiles = $this->extlyRemoveFilesPro;
		} else {
		$extlyRemoveFiles = array(
				'files'		=> array_merge($this->extlyRemoveFilesPro['files'], $this->extlyRemoveFilesCore['files']),
				'folders'	=> array_merge($this->extlyRemoveFilesPro['folders'], $this->extlyRemoveFilesCore['folders']),
		);
		}
		$this->_removeObsoleteFilesAndFolders($extlyRemoveFiles);
		*/

		// Install FOF
		$fofStatus = $this->_installFOF($parent);

		// Remove obsolete files and folders
		$extlyRemoveFiles = array(
				'files'		=> array_merge($this->extlyRemoveFilesPro['files'], $this->extlyRemoveFilesCore['files']),
				'folders'	=> array_merge($this->extlyRemoveFilesPro['folders'], $this->extlyRemoveFilesCore['folders']),
		);
		$this->_removeObsoleteFilesAndFolders($extlyRemoveFiles);

		$this->_copyCliFiles($parent);

		// Remove Professional version plugins from AutoTweetNG Core
		/*
		if(!$isAutotweetPro) {
		$this->_removeProPlugins($parent);
		}
		*/

		// Make sure the two plugins folders exist in Core release and are empty
		/*
		if(!$isAutotweetPro) {
		if(!JFolder::exists(JPATH_ADMINISTRATOR.'/components/com_autotweet/plugins')) {
		JFolder::create(JPATH_ADMINISTRATOR.'/components/com_autotweet/plugins');
		}
		if(!JFolder::exists(JPATH_ADMINISTRATOR.'/components/com_autotweet/autotweet/plugins')) {
		JFolder::create(JPATH_ADMINISTRATOR.'/components/com_autotweet/autotweet/plugins');
		}
		}
		*/

		// Install Extly Straper
		$straperStatus = $this->_installStraper($parent);

		// Show the post-installation page
		$this->_renderPostInstallation($type, $status, $fofStatus, $straperStatus, $parent);

		// Kill update site
		$this->_killUpdateSite();

		// Several DB Updates
		$this->_updateDB();

		// Migrate previous info
		if ($type == 'update')
		{
			$this->_migrate();
		}
	}

	/**
	 * Runs on uninstallation
	 *
	 * @param JInstaller $parent
	 */
	function uninstall($parent)
	{
		$this->_uninstallAutotweetPlugins($parent);

		// Uninstall subextensions
		$status = $this->_uninstallSubextensions($parent);

		// Show the post-uninstallation page
		$this->_renderPostUninstallation($status, $parent);
	}

	/**
	 * Removes the Professional edition's plugins from the Core version
	 *
	 * @param JInstaller $parent
	 */
	private function _removeProPlugins($parent)
	{
		$src = $parent->getParent()->getPath('source');
		$db = JFactory::getDbo();

		# ----- System - System Restore Points
		$sql = $db->getQuery(true)
		->select($db->qn('extension_id'))
		->from($db->qn('#__extensions'))
		->where($db->qn('type').' = '.$db->q('plugin'))
		->where($db->qn('element').' = '.$db->q('srp'))
		->where($db->qn('folder').' = '.$db->q('system'));
		$db->setQuery($sql);
		$id = $db->loadResult();
		if($id)
		{
			$installer = new JInstaller;
			$result = $installer->uninstall('plugin',$id,1);
		}

		# ----- System - Ak--ba Update Check
		/*
		$sql = $db->getQuery(true)
		->select($db->qn('extension_id'))
		->from($db->qn('#__extensions'))
		->where($db->qn('type').' = '.$db->q('plugin'))
		->where($db->qn('element').' = '.$db->q('ak--baupdatecheck'))
		->where($db->qn('folder').' = '.$db->q('system'));
		$db->setQuery($sql);
		$id = $db->loadResult();
		if($id)
		{
		$installer = new JInstaller;
		$result = $installer->uninstall('plugin',$id,1);
		}
		*/
	}

	/**
	 * Copies the CLI scripts into Joomla!'s cli directory
	 *
	 * @param JInstaller $parent
	 */
	private function _copyCliFiles($parent)
	{
		$src = $parent->getParent()->getPath('source');

		jimport("joomla.filesystem.file");
		jimport("joomla.filesystem.folder");

		foreach($this->extlyCliScripts as $script) {
			if(JFile::exists(JPATH_ROOT.'/cli/'.$script)) {
				JFile::delete(JPATH_ROOT.'/cli/'.$script);
			}
			if(JFile::exists($src.'/cli/'.$script)) {
				JFile::move($src.'/cli/'.$script, JPATH_ROOT.'/cli/'.$script);
			}
		}
	}

	/**
	 * Renders the post-installation message
	 */
	private function _renderPostInstallation($type, $status, $fofStatus, $straperStatus, $parent)
	{
		?>

<?php $rows = 1;?>

<div class="extly">

	<img src="../media/com_autotweet/images/autotweet-logo.png" width="57" height="57" alt="Extly - Joomla Extensions" align="right" />

	<h1>Welcome to AutoTweetNG!</h1>

	<p>
		<strong>Enhance your social media management!</strong>
	</p>

<?php
	if ($type == 'update')
	{
?>
	<div class="alert">
			<p>This is a full update. Please, remember to <b>install your additional plugins</b> and <b>re-validate the channels</b>.</p>
	</div>
<?php
	}
?>
	<h2>Installation Status</h2>

	<table class="table table-striped table-bordered">
		<thead>
			<tr>
				<th class="title" colspan="2">Extension</th>
				<th width="30%">Status</th>
			</tr>
		</thead>
		<tfoot>
			<tr>
				<td colspan="3"></td>
			</tr>
		</tfoot>
		<tbody>
			<tr class="row0 success">
				<td class="key" colspan="2">AutoTweetNG component</td>
				<td>Installed</td>
			</tr>

			<tr class="row0 <?php echo $straperStatus['required'] ? ($straperStatus['installed'] ? 'success' : 'error') : 'info'; ?>">
				<td class="key" colspan="2">Extly Framework <?php echo $straperStatus['version']?> [<?php echo $straperStatus['date'] ?>]
				</td>
				<td><span style=""> <?php echo $straperStatus['required'] ? ($straperStatus['installed'] ?'Installed':'Not Installed') : 'Already up-to-date'; ?>
				</span>
				</td>
			</tr>

			<?php
			if (count($status->modules))
			{
				?>
			<tr>
				<th>Module</th>
				<th>Client</th>
				<th></th>
			</tr>
			<?php
			foreach ($status->modules as $module)
			{
				?>
			<tr class="row<?php
			echo ($rows++ % 2);
			echo ($module['result']) ? " success" :  "error";
			?>">
				<td class="key"><?php echo $module['name']; ?></td>
				<td class="key"><?php echo ucfirst($module['client']); ?></td>
				<td><?php echo ($module['result'])?'Installed':'Not installed'; ?>
				</td>
			</tr>
			<?php
			}
			}
			?>
			<?php if (count($status->plugins)) : ?>
			<tr>
				<th>Plugin</th>
				<th>Group</th>
				<th></th>
			</tr>
			<?php foreach ($status->plugins as $plugin) : ?>
			<tr class="row<?php echo ($rows++ % 2); ?> <?php echo ($plugin['result'])? "success" : "error"; ?>">
				<td class="key"><?php echo ucfirst($plugin['name']); ?></td>
				<td class="key"><?php echo ucfirst($plugin['group']); ?></td>
				<td><?php echo ($plugin['result'])?'Installed':'Not installed'; ?>
				</td>
			</tr>
			<?php endforeach; ?>
			<?php endif; ?>

			<tr class="row1 <?php echo $fofStatus['required'] ? ($fofStatus['installed']?'success':'error') : 'info'; ?> ">
				<td class="key" colspan="2">Framework on Framework (FOF) <?php echo $fofStatus['version']?> [<?php echo $fofStatus['date'] ?>]
				</td>
				<td><span style=""> <?php echo $fofStatus['required'] ? ($fofStatus['installed'] ?'Installed':'Not Installed') : 'Already up-to-date'; ?>
				</span>
				</td>
			</tr>
		</tbody>
	</table>

	<h2>Tutorials</h2>

	<ul style="list-style-type: none;">
		<li><img src="../media/lib_extly/images/icons/star.png"> <a href="http://www.extly.com/how-to-autotweet-in-5-minutes-from-joomla.html" target="_blank">How to AutoTweet from Joomla in 5 minutes</a></li>
		<li><img src="../media/lib_extly/images/icons/star.png"> <a href="http://www.extly.com/how-to-autotweet-from-your-own-facebook-app.html" target="_blank">How to AutoTweet from Your Own Facebook App</a></li>
		<li><img src="../media/lib_extly/images/icons/star.png"> <a href="http://www.extly.com/how-to-autotweet-from-your-own-facebook-heroku-app.html" target="_blank"> How to AutoTweet from your own Facebook-Heroku App (No SSL certificate)</a></li>
	</ul>

	<h2>How to AutoTweet from Joomla in 5 minutes</h2>

	<iframe width="420" height="315" src="http://www.youtube.com/embed/6laZTbx_osM" frameborder="0" allowfullscreen></iframe>

	<hr />

	<p>
		If you have any question, please, don't hesitate to contact us.<br /> Technical Support: <img src="../media/lib_extly/images/icons/thumbs-up.png"> <a href="http://support.extly.com" target="_blank">http://support.extly.com</a>
	</p>

	<p>
		We are passionately committed to your success.<br /> Support Team<br /> <strong>Extly.com - Extensions</strong><br /> <img src="../media/lib_extly/images/icons/thumbs-up.png"> <a href="http://support.extly.com" target="_blank">http://support.extly.com</a>
		| <img src="../media/lib_extly/images/icons/twitter.png"> <a href="http://twitter.com/extly" target="_blank">@extly</a> | <img src="../media/lib_extly/images/icons/facebook.png"> <a href="http://www.facebook.com/extly" target="_blank">facebook.com/extly</a>
	</p>


</div>

<?php
	}

	private function _renderPostUninstallation($status, $parent)
	{
		?>
<?php $rows = 0;?>
<h2>AutoTweetNG uninstallation status</h2>
<table class="adminlist">
	<thead>
		<tr>
			<th class="title" colspan="2"><?php echo JText::_('Extension'); ?></th>
			<th width="30%"><?php echo JText::_('Status'); ?></th>
		</tr>
	</thead>
	<tfoot>
		<tr>
			<td colspan="3"></td>
		</tr>
	</tfoot>
	<tbody>
		<tr class="row0 success">
			<td class="key" colspan="2"><?php echo 'AutoTweetNG '.JText::_('Component'); ?></td>
			<td><strong><?php echo JText::_('Removed'); ?> </strong></td>
		</tr>
		<?php if (count($status->modules)) : ?>
		<tr>
			<th><?php echo JText::_('Module'); ?></th>
			<th><?php echo JText::_('Client'); ?></th>
			<th></th>
		</tr>
		<?php foreach ($status->modules as $module) : ?>
		<tr class="row<?php echo (++ $rows % 2); ?> <?php echo ($module['result'])? "success" : "error"; ?>">
			<td class="key"><?php echo $module['name']; ?></td>
			<td class="key"><?php echo ucfirst($module['client']); ?></td>
			<td><strong><?php echo ($module['result'])?JText::_('Removed'):JText::_('Not removed'); ?> </strong></td>
		</tr>
		<?php endforeach;?>
		<?php endif;?>
		<?php if (count($status->plugins)) : ?>
		<tr>
			<th><?php echo JText::_('Plugin'); ?></th>
			<th><?php echo JText::_('Group'); ?></th>
			<th></th>
		</tr>
		<?php foreach ($status->plugins as $plugin) : ?>
		<tr class="row<?php echo (++ $rows % 2); ?> <?php echo ($plugin['result'])? "success" : "error"; ?>">
			<td class="key"><?php echo ucfirst($plugin['name']); ?></td>
			<td class="key"><?php echo ucfirst($plugin['group']); ?></td>
			<td><strong><?php echo ($plugin['result'])?JText::_('Removed'):JText::_('Not removed'); ?> </strong></td>
		</tr>
		<?php endforeach; ?>
		<?php endif; ?>
	</tbody>
</table>
<p></p>
<p></p>
<?php
	}



	/**
	 * Joomla! 1.6+ bugfix for "DB function returned no error"
	 */
	private function _bugfixDBFunctionReturnedNoError()
	{
		$db = JFactory::getDbo();

		// Fix broken #__assets records
		$query = $db->getQuery(true);
		$query->select('id')
		->from('#__assets')
		->where($db->qn('name').' = '.$db->q($this->_extly_extension));
		$db->setQuery($query);
		$ids = $db->loadColumn();
		if(!empty($ids)) foreach($ids as $id) {
			$query = $db->getQuery(true);
			$query->delete('#__assets')
			->where($db->qn('id').' = '.$db->q($id));
			$db->setQuery($query);
			$db->execute();
		}

		// Fix broken #__extensions records
		$query = $db->getQuery(true);
		$query->select('extension_id')
		->from('#__extensions')
		->where($db->qn('element').' = '.$db->q($this->_extly_extension));
		$db->setQuery($query);
		$ids = $db->loadColumn();
		if(!empty($ids)) foreach($ids as $id) {
			$query = $db->getQuery(true);
			$query->delete('#__extensions')
			->where($db->qn('extension_id').' = '.$db->q($id));
			$db->setQuery($query);
			$db->execute();
		}

		// Fix broken #__menu records
		$query = $db->getQuery(true);
		$query->select('id')
		->from('#__menu')
		->where($db->qn('type').' = '.$db->q('component'))
		->where($db->qn('menutype').' = '.$db->q('main'))
		->where($db->qn('link').' LIKE '.$db->q('index.php?option='.$this->_extly_extension));
		$db->setQuery($query);
		$ids = $db->loadColumn();
		if(!empty($ids)) foreach($ids as $id) {
			$query = $db->getQuery(true);
			$query->delete('#__menu')
			->where($db->qn('id').' = '.$db->q($id));
			$db->setQuery($query);
			$db->execute();
		}
	}

	/**
	 * Joomla! 1.6+ bugfix for "Can not build admin menus"
	 */
	private function _bugfixCantBuildAdminMenus()
	{
		$db = JFactory::getDbo();

		// If there are multiple #__extensions record, keep one of them
		$query = $db->getQuery(true);
		$query->select('extension_id')
		->from('#__extensions')
		->where($db->qn('element').' = '.$db->q($this->_extly_extension));
		$db->setQuery($query);
		$ids = $db->loadColumn();
		if(count($ids) > 1) {
			asort($ids);
			$extension_id = array_shift($ids); // Keep the oldest id

			foreach($ids as $id) {
				$query = $db->getQuery(true);
				$query->delete('#__extensions')
				->where($db->qn('extension_id').' = '.$db->q($id));
				$db->setQuery($query);
				$db->execute();
			}
		}

		// @todo

		// If there are multiple assets records, delete all except the oldest one
		$query = $db->getQuery(true);
		$query->select('id')
		->from('#__assets')
		->where($db->qn('name').' = '.$db->q($this->_extly_extension));
		$db->setQuery($query);
		$ids = $db->loadObjectList();
		if(count($ids) > 1) {
			asort($ids);
			$asset_id = array_shift($ids); // Keep the oldest id

			foreach($ids as $id) {
				$query = $db->getQuery(true);
				$query->delete('#__assets')
				->where($db->qn('id').' = '.$db->q($id));
				$db->setQuery($query);
				$db->execute();
			}
		}

		// Remove #__menu records for good measure!
		$query = $db->getQuery(true);
		$query->select('id')
		->from('#__menu')
		->where($db->qn('type').' = '.$db->q('component'))
		->where($db->qn('menutype').' = '.$db->q('main'))
		->where($db->qn('link').' LIKE '.$db->q('index.php?option='.$this->_extly_extension));
		$db->setQuery($query);
		$ids1 = $db->loadColumn();
		if(empty($ids1)) $ids1 = array();
		$query = $db->getQuery(true);
		$query->select('id')
		->from('#__menu')
		->where($db->qn('type').' = '.$db->q('component'))
		->where($db->qn('menutype').' = '.$db->q('main'))
		->where($db->qn('link').' LIKE '.$db->q('index.php?option='.$this->_extly_extension.'&%'));
		$db->setQuery($query);
		$ids2 = $db->loadColumn();
		if(empty($ids2)) $ids2 = array();
		$ids = array_merge($ids1, $ids2);
		if(!empty($ids)) foreach($ids as $id) {
			$query = $db->getQuery(true);
			$query->delete('#__menu')
			->where($db->qn('id').' = '.$db->q($id));
			$db->setQuery($query);
			$db->execute();
		}
	}

	/**
	 * Installs subextensions (modules, plugins) bundled with the main extension
	 *
	 * @param JInstaller $parent
	 * @return JObject The subextension installation status
	 */
	private function _installSubextensions($parent)
	{
		$src = $parent->getParent()->getPath('source');

		$db = JFactory::getDbo();

		$status = new JObject();
		$status->modules = array();
		$status->plugins = array();

		// Modules installation
		if(count($this->installation_queue['modules'])) {
			foreach($this->installation_queue['modules'] as $folder => $modules) {
				if(count($modules)) foreach($modules as $module => $modulePreferences) {
					// Install the module
					if(empty($folder)) $folder = 'site';
					$path = "$src/modules/$folder/$module";
					if(!is_dir($path)) {
						$path = "$src/modules/$folder/mod_$module";
					}
					if(!is_dir($path)) {
						$path = "$src/modules/$module";
					}
					if(!is_dir($path)) {
						$path = "$src/modules/mod_$module";
					}
					if(!is_dir($path)) continue;
					// Was the module already installed?
					$sql = $db->getQuery(true)
					->select('COUNT(*)')
					->from('#__modules')
					->where($db->qn('module').' = '.$db->q('mod_'.$module));
					$db->setQuery($sql);
					$count = $db->loadResult();
					$installer = new JInstaller;
					$result = $installer->install($path);
					$status->modules[] = array(
							'name'=>'mod_'.$module,
							'client'=>$folder,
							'result'=>$result
					);
					// Modify where it's published and its published state
					if(!$count) {
						// A. Position and state
						list($modulePosition, $modulePublished) = $modulePreferences;
						if($modulePosition == 'cpanel') {
							$modulePosition = 'icon';
						}
						$sql = $db->getQuery(true)
						->update($db->qn('#__modules'))
						->set($db->qn('position').' = '.$db->q($modulePosition))
						->where($db->qn('module').' = '.$db->q('mod_'.$module));
						if($modulePublished) {
							$sql->set($db->qn('published').' = '.$db->q('1'));
						}
						$db->setQuery($sql);
						$db->execute();

						// B. Change the ordering of back-end modules to 1 + max ordering
						if($folder == 'admin') {
							$query = $db->getQuery(true);
							$query->select('MAX('.$db->qn('ordering').')')
							->from($db->qn('#__modules'))
							->where($db->qn('position').'='.$db->q($modulePosition));
							$db->setQuery($query);
							$position = $db->loadResult();
							$position++;

							$query = $db->getQuery(true);
							$query->update($db->qn('#__modules'))
							->set($db->qn('ordering').' = '.$db->q($position))
							->where($db->qn('module').' = '.$db->q('mod_'.$module));
							$db->setQuery($query);
							$db->execute();
						}

						// C. Link to all pages
						$query = $db->getQuery(true);
						$query->select('id')->from($db->qn('#__modules'))
						->where($db->qn('module').' = '.$db->q('mod_'.$module));
						$db->setQuery($query);
						$moduleid = $db->loadResult();

						$query = $db->getQuery(true);
						$query->select('*')->from($db->qn('#__modules_menu'))
						->where($db->qn('moduleid').' = '.$db->q($moduleid));
						$db->setQuery($query);
						$assignments = $db->loadObjectList();
						$isAssigned = !empty($assignments);
						if(!$isAssigned) {
							$o = (object)array(
								'moduleid'	=> $moduleid,
								'menuid'	=> 0
							);
							$db->insertObject('#__modules_menu', $o);
						}
					}
				}
			}
		}

		// Plugins installation
		if(count($this->installation_queue['plugins'])) {
			foreach($this->installation_queue['plugins'] as $folder => $plugins) {
				if(count($plugins)) foreach($plugins as $plugin => $published) {
					$path = "$src/plugins/$folder/$plugin";
					if(!is_dir($path)) {
						$path = "$src/plugins/$folder/plg_$plugin";
					}
					if(!is_dir($path)) {
						$path = "$src/plugins/$plugin";
					}
					if(!is_dir($path)) {
						$path = "$src/plugins/plg_$plugin";
					}
					if(!is_dir($path)) continue;

					// Was the plugin already installed?
					$query = $db->getQuery(true)
					->select('COUNT(*)')
					->from($db->qn('#__extensions'))
					->where($db->qn('element').' = '.$db->q($plugin))
					->where($db->qn('folder').' = '.$db->q($folder));
					$db->setQuery($query);
					$count = $db->loadResult();

					$installer = new JInstaller;
					$result = $installer->install($path);

					$status->plugins[] = array('name'=>'plg_'.$plugin,'group'=>$folder, 'result'=>$result);

					if($published && !$count) {
						$query = $db->getQuery(true)
						->update($db->qn('#__extensions'))
						->set($db->qn('enabled').' = '.$db->q('1'))
						->where($db->qn('element').' = '.$db->q($plugin))
						->where($db->qn('folder').' = '.$db->q($folder));
						$db->setQuery($query);
						$db->execute();
					}
				}
			}
		}

		return $status;
	}

	/**
	 * Uninstalls subextensions (modules, plugins) bundled with the main extension
	 *
	 * @param JInstaller $parent
	 * @return JObject The subextension uninstallation status
	 */
	private function _uninstallSubextensions($parent)
	{
		jimport('joomla.installer.installer');

		$db = & JFactory::getDBO();

		$status = new JObject();
		$status->modules = array();
		$status->plugins = array();

		$src = $parent->getParent()->getPath('source');

		// Modules uninstallation
		if(count($this->installation_queue['modules'])) {
			foreach($this->installation_queue['modules'] as $folder => $modules) {
				if(count($modules)) foreach($modules as $module => $modulePreferences) {
					// Find the module ID
					$sql = $db->getQuery(true)
					->select($db->qn('extension_id'))
					->from($db->qn('#__extensions'))
					->where($db->qn('element').' = '.$db->q('mod_'.$module))
					->where($db->qn('type').' = '.$db->q('module'));
					$db->setQuery($sql);
					$id = $db->loadResult();
					// Uninstall the module
					if($id) {
						$installer = new JInstaller;
						$result = $installer->uninstall('module',$id,1);
						$status->modules[] = array(
							'name'=>'mod_'.$module,
							'client'=>$folder,
							'result'=>$result
						);
					}
				}
			}
		}

		// Plugins uninstallation
		if(count($this->installation_queue['plugins'])) {
			foreach($this->installation_queue['plugins'] as $folder => $plugins) {
				if(count($plugins)) foreach($plugins as $plugin => $published) {
					$sql = $db->getQuery(true)
					->select($db->qn('extension_id'))
					->from($db->qn('#__extensions'))
					->where($db->qn('type').' = '.$db->q('plugin'))
					->where($db->qn('element').' = '.$db->q($plugin))
					->where($db->qn('folder').' = '.$db->q($folder));
					$db->setQuery($sql);

					$id = $db->loadResult();
					if($id)
					{
						$installer = new JInstaller;
						$result = $installer->uninstall('plugin',$id,1);
						$status->plugins[] = array(
							'name'=>'plg_'.$plugin,
							'group'=>$folder,
							'result'=>$result
						);
					}
				}
			}
		}

		return $status;
	}

	/**
	 * Removes obsolete files and folders
	 *
	 * @param array $extlyRemoveFiles
	 */
	private function _removeObsoleteFilesAndFolders($extlyRemoveFiles)
	{
		// Remove files
		jimport('joomla.filesystem.file');
		if(!empty($extlyRemoveFiles['files'])) foreach($extlyRemoveFiles['files'] as $file) {
			$f = JPATH_ROOT.'/'.$file;
			if (!file_exists($f))
			{
				continue;
			}
			if (!@unlink($f))
			{
				JFile::delete($f);
			}
		}

		// Remove folders
		jimport('joomla.filesystem.file');
		if(!empty($extlyRemoveFiles['folders'])) foreach($extlyRemoveFiles['folders'] as $folder) {
			$f = JPATH_ROOT.'/'.$folder;
			if (!JFolder::exists($f))
			{
				continue;
			}
			JFolder::delete($f);
		}
	}

	/**
	 * Installs the FOF framework only if the currently installed version on the
	 * user's site is out of date or if there is no version already installed.
	 *
	 * @param JInstaller $parent
	 * @return array
	 */
	private function _installFOF($parent)
	{
		$src = $parent->getParent()->getPath('source');

		// Install the FOF framework
		jimport('joomla.filesystem.folder');
		jimport('joomla.filesystem.file');
		jimport('joomla.utilities.date');
		$source = $src.'/fof';
		if(!defined('JPATH_LIBRARIES')) {
			$target = JPATH_ROOT.'/libraries/fof';
		} else {
			$target = JPATH_LIBRARIES.'/fof';
		}
		$haveToInstallFOF = false;
		if(!JFolder::exists($target)) {
			$haveToInstallFOF = true;
		} else {
			$fofVersion = array();
			if(JFile::exists($target.'/version.txt')) {
				$rawData = JFile::read($target.'/version.txt');
				$info = explode("\n", $rawData);
				$fofVersion['installed'] = array(
					'version'	=> trim($info[0]),
					'date'		=> new JDate(trim($info[1]))
				);
			} else {
				$fofVersion['installed'] = array(
					'version'	=> '0.0',
					'date'		=> new JDate('2011-01-01')
				);
			}
			$rawData = JFile::read($source.'/version.txt');
			$info = explode("\n", $rawData);
			$fofVersion['package'] = array(
				'version'	=> trim($info[0]),
				'date'		=> new JDate(trim($info[1]))
			);

			$haveToInstallFOF = $fofVersion['package']['date']->toUNIX() > $fofVersion['installed']['date']->toUNIX();
		}

		$installedFOF = false;
		if($haveToInstallFOF) {
			$versionSource = 'package';
			$installer = new JInstaller;
			$installedFOF = $installer->install($source);
		} else {
			$versionSource = 'installed';
		}

		if(!isset($fofVersion)) {
			$fofVersion = array();
			if(JFile::exists($target.'/version.txt')) {
				$rawData = JFile::read($target.'/version.txt');
				$info = explode("\n", $rawData);
				$fofVersion['installed'] = array(
					'version'	=> trim($info[0]),
					'date'		=> new JDate(trim($info[1]))
				);
			} else {
				$fofVersion['installed'] = array(
					'version'	=> '0.0',
					'date'		=> new JDate('2011-01-01')
				);
			}
			$rawData = JFile::read($source.'/version.txt');
			$info = explode("\n", $rawData);
			$fofVersion['package'] = array(
				'version'	=> trim($info[0]),
				'date'		=> new JDate(trim($info[1]))
			);
			$versionSource = 'installed';
		}

		if(!($fofVersion[$versionSource]['date'] instanceof JDate)) {
			$fofVersion[$versionSource]['date'] = new JDate();
		}

		return array(
			'required'	=> $haveToInstallFOF,
			'installed'	=> $installedFOF,
			'version'	=> $fofVersion[$versionSource]['version'],
			'date'		=> $fofVersion[$versionSource]['date']->format('%Y-%m-%d'),
		);
	}

	private function _installStraper($parent)
	{
		$src = $parent->getParent()->getPath('source');

		// Install the FOF framework
		jimport('joomla.filesystem.folder');
		jimport('joomla.filesystem.file');
		jimport('joomla.utilities.date');

		/*
		 $source = $src.'/strapper';
		$target = JPATH_ROOT.'/media/extly_strapper';
		*/
		$source = $src.'/strapper/lib_extly';
		$target = JPATH_ROOT.'/libraries/extly';

		$haveToInstallStraper = false;
		if(!JFolder::exists($target)) {
			$haveToInstallStraper = true;
		} else {
			$straperVersion = array();
			if(JFile::exists($target.'/version.txt')) {
				$rawData = JFile::read($target.'/version.txt');
				$info = explode("\n", $rawData);
				$straperVersion['installed'] = array(
					'version'	=> trim($info[0]),
					'date'		=> new JDate(trim($info[1]))
				);
			} else {
				$straperVersion['installed'] = array(
					'version'	=> '0.0',
					'date'		=> new JDate('2011-01-01')
				);
			}
			$rawData = JFile::read($source.'/version.txt');
			$info = explode("\n", $rawData);
			$straperVersion['package'] = array(
				'version'	=> trim($info[0]),
				'date'		=> new JDate(trim($info[1]))
			);

			$haveToInstallStraper = $straperVersion['package']['date']->toUNIX() > $straperVersion['installed']['date']->toUNIX();
		}

		$installedStraper = false;
		if($haveToInstallStraper) {
			$versionSource = 'package';
			$installer = new JInstaller;
			$installedStraper = $installer->install($source);
		} else {
			$versionSource = 'installed';
		}

		if(!isset($straperVersion)) {
			$straperVersion = array();
			if(JFile::exists($target.'/version.txt')) {
				$rawData = JFile::read($target.'/version.txt');
				$info = explode("\n", $rawData);
				$straperVersion['installed'] = array(
					'version'	=> trim($info[0]),
					'date'		=> new JDate(trim($info[1]))
				);
			} else {
				$straperVersion['installed'] = array(
					'version'	=> '0.0',
					'date'		=> new JDate('2011-01-01')
				);
			}
			$rawData = JFile::read($source.'/version.txt');
			$info = explode("\n", $rawData);
			$straperVersion['package'] = array(
				'version'	=> trim($info[0]),
				'date'		=> new JDate(trim($info[1]))
			);
			$versionSource = 'installed';
		}

		if(!($straperVersion[$versionSource]['date'] instanceof JDate)) {
			$straperVersion[$versionSource]['date'] = new JDate();
		}

		return array(
			'required'	=> $haveToInstallStraper,
			'installed'	=> $installedStraper,
			'version'	=> $straperVersion[$versionSource]['version'],
			'date'		=> $straperVersion[$versionSource]['date']->format('Y-m-d'),
		);
	}

	/**
	 * Remove the update site specification from Joomla! – we no longer support
	 * that misbehaving crap, thank you very much...
	 */
	private function _killUpdateSite()
	{
		// Get some info on all the stuff we've gotta delete
		$db = JFactory::getDbo();
		$query = $db->getQuery(true)
		->select(array(
				$db->qn('s').'.'.$db->qn('update_site_id'),
				$db->qn('e').'.'.$db->qn('extension_id'),
				$db->qn('e').'.'.$db->qn('element'),
				$db->qn('s').'.'.$db->qn('location'),
			))
			->from($db->qn('#__update_sites').' AS '.$db->qn('s'))
			->join('INNER',$db->qn('#__update_sites_extensions').' AS '.$db->qn('se').' ON('.
				$db->qn('se').'.'.$db->qn('update_site_id').' = '.
				$db->qn('s').'.'.$db->qn('update_site_id')
				.')')
			->join('INNER',$db->qn('#__extensions').' AS '.$db->qn('e').' ON('.
				$db->qn('e').'.'.$db->qn('extension_id').' = '.
				$db->qn('se').'.'.$db->qn('extension_id')
				.')')
			->where($db->qn('s').'.'.$db->qn('type').' = '.$db->q('extension'))
			->where($db->qn('e').'.'.$db->qn('type').' = '.$db->q('component'))
			->where($db->qn('e').'.'.$db->qn('element').' = '.$db->q($this->_extly_extension))
		;
		$db->setQuery($query);
		$oResult = $db->loadObject();

		// If no record is found, do nothing. We've already killed the monster!
		if(is_null($oResult)) return;

		// Delete the #__update_sites record
		$query = $db->getQuery(true)
		->delete($db->qn('#__update_sites'))
		->where($db->qn('update_site_id').' = '.$db->q($oResult->update_site_id));
		$db->setQuery($query);
		try {
			$db->execute();
		} catch (Exception $exc) {
			// If the query fails, don't sweat about it
		}

		// Delete the #__update_sites_extensions record
		$query = $db->getQuery(true)
		->delete($db->qn('#__update_sites_extensions'))
		->where($db->qn('update_site_id').' = '.$db->q($oResult->update_site_id));
		$db->setQuery($query);
		try {
			$db->execute();
		} catch (Exception $exc) {
			// If the query fails, don't sweat about it
		}

		// Delete the #__updates records
		$query = $db->getQuery(true)
		->delete($db->qn('#__updates'))
		->where($db->qn('update_site_id').' = '.$db->q($oResult->update_site_id));
		$db->setQuery($query);
		try {
			$db->execute();
		} catch (Exception $exc) {
			// If the query fails, don't sweat about it
		}
	}

	private function _updateDB()
	{
		// Update Automator
		/*
		$db = JFactory::getDbo();
		$query = $db->getQuery(true)
		->update($db->qn('#__autotweet_automator'))
		->set($db->qn('lastexec') . ' = NOW()');
		$db->setQuery($query);
		try {
			$db->execute();
		} catch (Exception $exc) {
			// If the query fails, don't sweat about it
		}
		*/

		if ($this->_broken510)
		{
			$sqlfile = JPATH_ROOT . '/administrator/components/com_autotweet/sql/install/mysql/install.sql';
			$buffer = file_get_contents($sqlfile);
			$queries = JInstallerHelper::splitSql($buffer);
			if (count($queries) > 0)
			{
				foreach ($queries as $query)
				{
					$query = trim($query);
					if ($query != '' && $query{0} != '#')
					{
						$db->setQuery($query);
						try {
							$db->execute();
						} catch (Exception $exc) {
							// If the query fails, don't sweat about it
						}
					}
				}
			}
		}
	}

	private function _isBroken510()
	{
		$db = JFactory::getDbo();
		$sql = $db->getQuery(true)
		->select($db->qn('version_id'))
		->from($db->qn('#__extensions', 'e'))
		->from($db->qn('#__schemas', 's'))
		->where($db->qn('s') . ' . ' . $db->qn('extension_id') . ' = ' . $db->qn('e') . ' . ' . $db->qn('extension_id'))
		->where($db->qn('name') . ' = ' . $db->q('com_autotweet'))
		->where($db->qn('type') . ' = ' . $db->q('component'));
		$db->setQuery($sql);
		$version_id = $db->loadResult();

		return (empty($version_id));
	}

	private function _migrate()
	{
		$db = JFactory::getDbo();

		// Channels Facebook
		try {
			$sql = $db->getQuery(true)
			->select('COUNT(*)')
			->from($db->qn('#__autotweet_account'));
			$db->setQuery($sql);
			$result = $db->loadResult();
			if ($result)
			{
				$db->setQuery("INSERT INTO `#__autotweet_channels` (`published`,`channeltype_id`,`autopublish`,`name`,`description`,`media_mode`,`params`)
SELECT `p`.`published`,`p`.`channeltype_id`,`p`.`autopublish`,`p`.`name`,`p`.`description`,`p`.`media_mode`,
CONCAT_WS('',
  '{\"use_own_api\":\"',`a`.`use_own_api`,
'\",\"app_id\":\"',`a`.`api_key`,
'\",\"secret\":\"',`a`.`api_secret`,
'\",\"canvas_page\":\"',`a`.`api_authurl`,
'\",\"access_token\":\"',`a`.`token`,
'\",\"user_id\":\"',`a`.`account_id`,
'\",\"fbchannel_id\":\"',`p`.`id_1`,
'\",\"fbchannel_access_token\":\"',`p`.`id_2`,
'\"}')
FROM
`#__autotweet_channel` AS `p`
JOIN `#__autotweet_account` AS `a` ON `p`.`account_id` = `a`.`id`
LEFT OUTER JOIN `#__autotweet_channels` AS `c` ON `p`.`name` = `c`.`name`
WHERE `c`.`id` IS NULL;");
				$db->execute();
			}
		} catch (Exception $exc) {
			// If the query fails, don't sweat about it
		}

		// Channels
		try {
			$sql = $db->getQuery(true)
				->select('COUNT(*)')
				->from($db->qn('#__autotweet_channel'));
			$db->setQuery($sql);
			$result = $db->loadResult();
			if ($result)
			{
				$db->setQuery("INSERT INTO `#__autotweet_channels` (`published`,`channeltype_id`,`autopublish`,`name`,`description`,`media_mode`,`params`)
SELECT `p`.`published`,`p`.`channeltype_id`,`p`.`autopublish`,`p`.`name`,`p`.`description`,`p`.`media_mode`,
CONCAT_WS('',
'{\"consumer_key\":\"',api_key,
'\",\"consumer_secret\":\"',api_secret,
'\",\"access_token\":\"',id_1,
'\",\"','access_token_secret\":\"',id_1_secret,
'\",\"','api_key\":\"',api_key,
'\",\"secret_key\":\"',api_secret,
'\",\"oauth_user_token\":\"',id_1,
'\",\"','oauth_user_secret\":\"',id_1_secret,
'\",\"','group_id\":\"',id_2,
'\"}')
FROM `#__autotweet_channel` AS `p` LEFT OUTER JOIN `#__autotweet_channels` AS `c` ON `p`.`name` = `c`.`name`
WHERE `c`.`id` IS NULL;");
				$db->execute();
			}
		} catch (Exception $exc) {
			// If the query fails, don't sweat about it
		}

		// Posts
		try {
			$sql = $db->getQuery(true)
			->select('COUNT(*)')
			->from($db->qn('#__autotweet_msglog'));
			$db->setQuery($sql);
			$result = $db->loadResult();
			if ($result)
			{
				$db->setQuery("INSERT INTO `#__autotweet_posts` (`ref_id`,`plugin`,`channel_id`,`postdate`,`pubstate`,`resultmsg`,`message`,`url`,`image_url`,`title`,`fulltext`,`show_url`,`event_data`)
SELECT `p`.`ref_id`,`p`.`plugin`,`p`.`channel_id`,`p`.`postdate`,`p`.`pubstate`,`p`.`resultmsg`,`p`.`message`,`p`.`url`,`p`.`image_url`,`p`.`title`,`p`.`fulltext`,`p`.`show_url`,`p`.`event_data`
FROM `#__autotweet_msglog` AS `p` LEFT OUTER JOIN `#__autotweet_posts` AS `c` ON `p`.`ref_id` = `c`.`ref_id` AND `p`.`plugin` = `c`.`plugin` AND `p`.`postdate` = `c`.`postdate`
WHERE `c`.`id` IS NULL;");
				$db->execute();
			}
		} catch (Exception $exc) {
			// If the query fails, don't sweat about it
		}

		// Rules
		try {
			$sql = $db->getQuery(true)
			->select('COUNT(*)')
			->from($db->qn('#__autotweet_rule'));
			$db->setQuery($sql);
			$result = $db->loadResult();
			if ($result)
			{
				$db->setQuery("INSERT INTO `#__autotweet_rules` (`name`,`published`,`ruletype_id`,`plugin`,`channel_id`,`cond`,`autopublish`,`show_url`,`show_static_text`,`statix_text`,`reg_ex`,`reg_replace`, `rmc_textpattern`)
SELECT CONCAT_WS('', 'Rule: ', `p`.`id`),`p`.`published`,`p`.`ruletype_id`,`p`.`plugin`,`p`.`channel_id`,`p`.`cond`,`p`.`autopublish`,`p`.`show_url`,`p`.`show_static_text`,`p`.`statix_text`,`p`.`reg_ex`,`p`.`reg_replace`,`p`.`rmc_textpattern`
FROM `#__autotweet_rule` AS `p` LEFT OUTER JOIN `#__autotweet_rules` AS `c` ON `p`.`ruletype_id` = `c`.`ruletype_id` AND `p`.`channel_id` = `c`.`channel_id` AND `p`.`plugin` = `c`.`plugin`
WHERE `c`.`id` IS NULL;");
				$db->execute();
			}
		} catch (Exception $exc) {
			// If the query fails, don't sweat about it
		}

	}

}
