<?php

/**
 * @package     Extly.Library
 * @subpackage  lib_extly - Extly Framework
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2012 Prieco, S.A. All rights reserved.
 * @license     http://http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('EXTLY_VERSION') or die('Restricted access');

/**
 * This is the base class for the Extly framework.
 *
 * @package     Extly.Library
 * @subpackage  lib_extly
 * @since       1.0
*/
class Extly
{

	const APP_INITIALIZATION_J25 = '
			</script><script data-main="{DATA_MAIN}" src="{JURI_ROOT}media/lib_extly/js/require/require.min.js"></script>
			<script>
			// Require.js allows us to configure shortcut alias
			require.config({
			// The shim config allows us to configure dependencies for
			// scripts that do not call define() to register a module
			shim: {
			\'bootstrap\': {
			deps: [
			\'jquery\',
			],
},
			\'underscore\': {
			deps: [
			\'jquery\',
			],
			exports: \'_\'
},
			\'chosen\': {
			deps: [
			\'jquery\',
			],
},
			\'backbone\': {
			deps: [
			\'underscore\',
			],
			exports: \'Backbone\'
},
			\'j3compat\': {
			deps: [
			\'bootstrap\',
			\'chosen\',
			],
},
			\'extlycore\': {
			deps: [
			\'backbone\',
			\'j3compat\',
			],
},
{CUSTOM_DEPENDENCY}
},
			paths: {
			jquery: \'{JURI_ROOT}media/lib_extly/js/jquery/jquery.min\',
			bootstrap: \'{JURI_ROOT}media/lib_extly/js/bootstrap/bootstrap.min\',
			underscore: \'{JURI_ROOT}media/lib_extly/js/lodash.min\',
			chosen: \'{JURI_ROOT}media/lib_extly/js/j3/chosen.jquery.min\',
			backbone: \'{JURI_ROOT}media/lib_extly/js/backbone/backbone.min\',
			j3compat: \'{JURI_ROOT}media/lib_extly/js/j3compat.min\',
			extlycore: \'{JURI_ROOT}media/lib_extly/js/extlycore.min\',
			{CUSTOM_PATH}
}
});
			';

	const APP_INITIALIZATION_J30 = '
			</script><script data-main="{DATA_MAIN}" src="{JURI_ROOT}media/lib_extly/js/require/require.min.js"></script>
			<script>

			jQuery(document).ready(function() {
			// Require.js allows us to configure shortcut alias
			require.config({
			// The shim config allows us to configure dependencies for
			// scripts that do not call define() to register a module
			shim: {
			\'underscore\': {
			exports: \'_\'
},
			\'backbone\': {
			deps: [
			\'underscore\',
			],
			exports: \'Backbone\'
},
			\'extlycore\': {
			deps: [
			\'backbone\',
			],
},
{CUSTOM_DEPENDENCY}
},
			paths: {
			underscore: \'{JURI_ROOT}media/lib_extly/js/lodash.min\',
			backbone: \'{JURI_ROOT}media/lib_extly/js/backbone/backbone.min\',
			extlycore: \'{JURI_ROOT}media/lib_extly/js/extlycore.min\',
			{CUSTOM_PATH}
}
});
});
			';

	/**
	 * loadMeta.
	 *
	 * @return	void
	 *
	 * @since	1.0
	 */
	public static function loadMeta()
	{
		$document = JFactory::getDocument();
		$document->setMetaData('X-UA-Compatible', 'IE=edge,chrome=1');
	}

	/**
	 * loadStyle.
	 *
	 * @return	void
	 *
	 * @since	1.0
	 */
	public static function loadStyle()
	{
		// Joomla 2.5
		if (version_compare(JVERSION, '3.0.0', 'lt'))
		{
			JHtml::stylesheet('lib_extly/extly-bootstrap.min.css', false, true);
			JHtml::stylesheet('lib_extly/bootstrap-extended.css', false, true);
			JHtml::stylesheet('lib_extly/chosen.css', false, true);
		}
		else
		{
			// Joomla 3.0 or superior
			// Assumed bootstrap.framework
		}
		JHtml::stylesheet('lib_extly/extly-base.css', false, true);
	}

	/**
	 * loadStyle.
	 *
	 * @return	void
	 *
	 * @since	1.0
	 */
	public static function loadAwesome()
	{
		JHtml::stylesheet('lib_extly/extly-font-awesome.min.css', false, true);
	}

	/**
	 * loadHtml
	 *
	 * @return	void
	 *
	 * @since	1.0
	 */
	public static function loadHtml()
	{
		JHtml::addIncludePath(JPATH_ROOT . '/libraries/extly/helpers/html');
	}

	/**
	 * initApp.
	 *
	 * @param   string  $extensionmainjs  Param
	 * @param   array   $dependencies     {key2 => {key1, keyi}}
	 * @param   array   $paths            {key1 => pathjs1, key2 => pathjs2}
	 *
	 * @return	void
	 *
	 * @since	1.0
	 */
	public static function initApp($extensionmainjs = null, $dependencies = null, $paths = null)
	{
		$document = JFactory::getDocument();

		// Joomla 2.5
		if (version_compare(JVERSION, '3.0.0', 'lt'))
		{
			$jsapp_init = self::APP_INITIALIZATION_J25;
			JHTML::_('behavior.mootools');
		}
		else
		{
			// Joomla 3.0 or superior
			$jsapp_init = self::APP_INITIALIZATION_J30;
			JHtml::_('bootstrap.framework');
			JHtml::_('formbehavior.chosen', 'select');
			JHtml::_('behavior.tooltip');
		}

		if ($extensionmainjs)
		{
			$jsapp_init = str_replace('{DATA_MAIN}', $extensionmainjs, $jsapp_init);
		}
		else
		{
			$jsapp_init = str_replace('{DATA_MAIN}', JURI::root() . 'media/lib_extly/js/defaultmain.min.js', $jsapp_init);
		}

		$listofdepsjs = '';
		if ($dependencies)
		{
			$listofdeps = array();
			foreach ($dependencies as $dependency_key => $dependency_values)
			{
				$deps = array();
				foreach ($dependency_values as $dependency)
				{
					$deps = '\'' . $dependency . '\'';
				}
				$listofdeps[] = '\'{' . $dependency_key . '}\': { deps: [' . join(',', $deps) . '], }';
			}
			$listofdepsjs = join(',', $listofdeps);
		}
		$jsapp_init = str_replace('{CUSTOM_DEPENDENCY}', $listofdepsjs, $jsapp_init);

		$listofpathsjs = '';
		if ($paths)
		{
			$listofpaths = array();
			foreach ($paths as $path_key => $path_value)
			{
				$listofpaths[] = $paths_key . ': \'' . $paths_value . '\'';
			}
			$listofpathsjs = join(',', $listofpaths);
		}
		$jsapp_init = str_replace('{CUSTOM_PATH}', $listofpathsjs, $jsapp_init);

		$jsapp_init = str_replace('{JURI_ROOT}', JURI::root(), $jsapp_init);

		$document->addScriptDeclaration($jsapp_init);
	}

	/**
	 * getFormId.
	 *
	 * @return	void
	 *
	 * @since	1.0
	 */
	public static function getFormId()
	{
		/*
		 return ((version_compare(JVERSION, '3.0.0', 'lt')) ?
					'adminlist' :
					'adminForm');
		*/
		return 'adminForm';
	}

	/**
	 * showInvalidFormAlert.
	 *
	 * @return	void
	 *
	 * @since	1.0
	 */
	public static function showInvalidFormAlert()
	{
		?>
<div id="invalid-form" class="alert alert-block alert-error" style="display: none;">
	<button data-dismiss="alert" class="close" type="button">×</button>
	<h4 class="alert-heading">
		<?php echo JText::_('JGLOBAL_VALIDATION_FORM_FAILED'); ?>
	</h4>
</div>
<?php
	}

}
