<?php

/**
 * @package     Extly.Library
 * @subpackage  lib_extly - Extly Framework
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2012 Prieco, S.A. All rights reserved.
 * @license     http://http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('EXTLY_VERSION') or die('Restricted access');

/**
 * Utility class for creating HTML Grids
 *
 * @package     Extly.Library
 * @subpackage  lib_extly - Extly Framework
 * @since       3.0
 */
abstract class EHtmlGrid
{

	/**
	 * Method to create a clickable icon to change the state of an item
	 *
	 * @param   mixed    $value   Either the scalar value or an object (for backward compatibility, deprecated)
	 * @param   integer  $i       The index
	 * @param   string   $img1    Image for a positive or on value
	 * @param   string   $img0    Image for the empty or off value
	 * @param   string   $prefix  An optional prefix for the task
	 *
	 * @return  string
	 *
	 * @since   11.1
	 */
	public static function published($value, $i, $img1 = 'lib_extly/icons/ok.png', $img0 = 'lib_extly/icons/minus.png', $prefix = '')
	{
		if (is_object($value))
		{
			$value = $value->published;
		}

		$img = $value ? $img1 : $img0;
		$task = $value ? 'unpublish' : 'publish';
		$alt = $value ? JText::_('JPUBLISHED') : JText::_('JUNPUBLISHED');
		$action = $value ? JText::_('JLIB_HTML_UNPUBLISH_ITEM') : JText::_('JLIB_HTML_PUBLISH_ITEM');

		$href = '
		<a href="#" onclick="return listItemTask(\'cb' . $i . '\',\'' . $prefix . $task . '\')" title="' . $action . '">'
			. JHtml::_('image', $img, $alt, null, true) . '</a>';

		return $href;
	}

	/**
	 * Method to create a clickable icon to change the state of an item
	 *
	 * @param   mixed    $value   Either the scalar value or an object (for backward compatibility, deprecated)
	 * @param   integer  $i       The index
	 * @param   string   $img1    Image for a positive or on value
	 * @param   string   $img0    Image for the empty or off value
	 * @param   string   $prefix  An optional prefix for the task
	 *
	 * @return  string
	 *
	 * @since   11.1
	 */
	public static function publishedWithIcons($value, $i, $img1 = '<i style="color: green;" class="xticon-check"></i>', $img0 = '<i style="color: red;" class="xticon-remove"></i>', $prefix = '')
	{
		if (is_object($value))
		{
			$value = $value->published;
		}

		$img = $value ? $img1 : $img0;
		$task = $value ? 'unpublish' : 'publish';
		$alt = $value ? JText::_('JPUBLISHED') : JText::_('JUNPUBLISHED');
		$action = $value ? JText::_('JLIB_HTML_UNPUBLISH_ITEM') : JText::_('JLIB_HTML_PUBLISH_ITEM');

		$href = '
		<a href="#" onclick="return listItemTask(\'cb' . $i . '\',\'' . $prefix . $task . '\')" title="' . $action . '">'
					. $img . '</a>';

		return $href;
	}

	/**
	 * Method to create a icon
	 *
	 * @param   mixed   $locked     Param
	 * @param   string  $img1       Param
	 * @param   string  $img0       Param
	 * @param   bool    $optimized  Param
	 *
	 * @return  string
	 */
	public static function locked($locked, $img1 = 'lib_extly/icons/lock.png', $img0 = 'lib_extly/icons/unlock.png', $optimized = false)
	{
		if (($optimized) && (!$locked))
		{
			return null;
		}

		$img = $locked ? $img1 : $img0;
		$alt = $locked ? JText::_('JLOCKED') : JText::_('JUNLOCKED');
		return JHtml::_('image', $img, $alt, null, true);
	}

}
