<?php

/**
 * @package     Extly.Library
 * @subpackage  lib_extly - Extly Framework
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2012 Prieco, S.A. All rights reserved.
 * @license     http://http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

/**
 * SchedulerTableTask
 *
 * @package     Extly.Library
 * @subpackage  lib_extly
 * @since       1.0
*/
class SchedulerTableTask extends FOFTable
{

	/**
	 * Instantiate the table object
	 *
	 * @param   string     $table  Param
	 * @param   string     $key    Param
	 * @param   JDatabase  &$db    The Joomla! database object
	 */
	public function __construct($table, $key, &$db)
	{
		parent::__construct('#__xtcronjob_tasks', 'id', $db);

		$this->_columnAlias = array(
				'enabled'		=> 'published',
				'created_on'	=> 'created',
				'modified_on'	=> 'modified',
				'locked_on'		=> 'checked_out_time',
				'locked_by'		=> 'checked_out',
		);

	}

	/**
	 * Checks the record for validity
	 *
	 * @return  int  True if the record is valid
	 */
	public function check()
	{
		// If the title is missing, throw an error
		if (!$this->title)
		{
			$this->setError(JText::_('SCHEDULER_TASK_ERR_NEEDS_TITLE'));
			return false;
		}

		// If the catid is missing, throw an error
		if (!$this->catid)
		{
			$this->setError(JText::_('SCHEDULER_TASK_ERR_NEEDS_CATEGORY'));
			return false;
		}

		// If the catid is missing, throw an error
		if (!$this->type)
		{
			$this->setError(JText::_('SCHEDULER_TASK_ERR_NEEDS_TYPE'));
			return false;
		}

		// If the file is missing, throw an error
		if (strlen($this->file) == 0)
		{
			$this->setError(JText::_('SCHEDULER_TASK_ERR_NEEDS_FILE'));
			return false;
		}

		JLoader::import('extly.helpers.CronParser');
		$cron = new CronParser;
		if (!$cron->calcLastRan($this->unix_mhdmd))
		{
			$this->setError(JText::_('SCHEDULER_TASK_ERR_CRONEXPRESSION'));
			return false;
		}

		JLoader::import('extly.scheduler.backend.helpers.taskmanager');
		if ($this->type == TaskManagerHelper::TASKS_TYPE_PLUGIN_FOLDER_EVENT)
		{
			$found = TaskManagerHelper::loadPluginFolder($this->file);
			if (!$found)
			{
				$this->setError(JText::_('SCHEDULER_TASK_ERR_PLUGINNOTFOUND'));
				return false;

			}
		}

		if (TaskManagerHelper::isConfigurablePlugin($this->type))
		{
			$plugin = TaskManagerHelper::loadPlugin($this->file);
			if (!$plugin)
			{
				$this->setError(JText::_('SCHEDULER_TASK_ERR_PLUGINNOTFOUND'));
				return false;

			}
		}

		return true;
	}

	/**
	 * onAfterLoad
	 *
	 * @param   bool  &$result  Param
	 *
	 * @return	void
	 */
	protected function onAfterLoad(&$result)
	{
		if ($this->id == 0)
		{
			$this->published = true;
		}
	}

}
