<?php

/**
 * @package     Extly.Library
 * @subpackage  lib_extly - Extly Framework
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2012 Prieco, S.A. All rights reserved.
 * @license     http://http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('EXTLY_VERSION') or die('Restricted access');

jimport('joomla.application.component.controller');

JLoader::register('PluginsHelper', JPATH_ADMINISTRATOR . '/components/com_plugins/helpers/plugins.php');

/**
 * XluginsController.
 *
 * @package     Extly.Library
 * @subpackage  lib_extly - Extly Framework
 * @since       1.0
*/
class XluginsController extends JController
{

	/**
	 * Constructor.
	 *
	 * @param   array  $config  An optional associative array of configuration settings.
	 *
	 * @see     JController
	 * @since   11.1
	 */
	public function __construct($config = array())
	{
		parent::__construct();

		$lang = JFactory::getLanguage();
		$option = 'com_plugins';

		// Load common and local language files.
		$lang->load($option, JPATH_BASE, null, false, false) || $lang->load($option, JPATH_COMPONENT, null, false, false)
		|| $lang->load($option, JPATH_BASE, $lang->getDefault(), false, false)
		|| $lang->load($option, JPATH_COMPONENT, $lang->getDefault(), false, false);

		$this->context = 'plugin';

		$this->addModelPath(EPATH_XLUGINS_MODELS);
		$this->addViewPath(EPATH_XLUGINS_VIEWS);
	}

	/**
	 * Method to display a view.
	 *
	 * @param   boolean  $cachable   If true, the view output will be cached
	 * @param   boolean  $urlparams  An array of safe url parameters and their variable types, for valid values see {@link JFilterInput::clean()}.
	 *
	 * @return  JController		This object to support chaining.
	 *
	 * @since	1.5
	 */
	public function display($cachable = false, $urlparams = false)
	{
		JLoader::load('PluginsHelper');

		// Load the submenu.
		PluginsHelper::addSubmenu(JRequest::getCmd('view', 'plugins'));

		$view		= JRequest::getCmd('view', 'plugins');
		$layout 	= JRequest::getCmd('layout', 'default');
		$id			= JRequest::getInt('extension_id');

		// Check for edit form.
		if ($view == 'plugin' && $layout == 'edit' && !$this->checkEditId('com_plugins.edit.plugin', $id))
		{
			$option = $this->getName();

			// Somehow the person just went to the form - we don't allow that.
			$this->setError(JText::sprintf('JLIB_APPLICATION_ERROR_UNHELD_ID', $id));
			$this->setMessage($this->getError(), 'error');
			$this->setRedirect(JRoute::_('index.php?option=com_' . $option . '&view=tasks', false));

			return false;
		}

		parent::display();
	}

	/**
	 * Method to get a reference to the current view and load it if necessary.
	 *
	 * @param   string  $name    The view name. Optional, defaults to the controller name.
	 * @param   string  $type    The view type. Optional.
	 * @param   string  $prefix  The class prefix. Optional.
	 * @param   array   $config  Configuration array for view. Optional.
	 *
	 * @return  object  Reference to the view or an error.
	 *
	 * @since   11.1
	 */
	public function getView($name = '', $type = '', $prefix  = 'XluginsView', $config = array())
	{
		if (empty($prefix))
		{
			$prefix  = 'XluginsView';
		}

		$view = parent::getView($name, $type, $prefix, $config);
		$view->assignRef('option', $this->getName());
		return $view;
	}
}
