<?php

/**
 * @package     Extly.Library
 * @subpackage  lib_extly - Extly Framework
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2012 Prieco, S.A. All rights reserved.
 * @license     http://http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('EXTLY_VERSION') or die('Restricted access');

jimport('joomla.application.component.modeladmin');
JLoader::import('components.com_plugins.models.plugin', JPATH_ADMINISTRATOR);

/**
 * XluginsModelXlugin model.
 *
 * @package     Joomla.Administrator
 * @subpackage  com_plugins
 * @since       1.6
*/
class XluginsModelXlugin extends PluginsModelPlugin
{

	/**
	 * Constructor
	 *
	 * @param   array  $config  An array of configuration options (name, state, dbo, table_path, ignore_request).
	 *
	 * @since   11.1
	 */
	public function __construct($config = array())
	{
		parent::__construct($config);
	}

	/**
	 * Method to get the record form.
	 *
	 * @param   array    $data      Data for the form.
	 * @param   boolean  $loadData  True if the form is to load its own data (default case), false if not.
	 *
	 * @return	JForm	A JForm object on success, false on failure
	 *
	 * @since	1.6
	 */
	public function getForm($data = array(), $loadData = true)
	{
		JForm::addFormPath(JPATH_ADMINISTRATOR . '/components/com_plugins/models/forms');
		JForm::addFieldPath(JPATH_ADMINISTRATOR . '/components/com_plugins/models/fields');

		return parent::getForm($data, $loadData);
	}

	/**
	 * preprocessForm
	 *
	 * @param   object  $form   A form object.
	 * @param   mixed   $data   The data expected for the form.
	 * @param   string  $group  The data expected for the form.
	 *
	 * @return	mixed	True if successful.
	 *
	 * @throws	Exception if there is an error in the form event.
	 *
	 * @since	1.6
	 */
	protected function preprocessForm(JForm $form, $data, $group = 'content')
	{
		// Trigger the default form events.
		parent::preprocessForm($form, $data, $group);

		// Initialise variables.
		$folder		= $this->getState('item.folder');
		$element	= $this->getState('item.element');
		$lang		= JFactory::getLanguage();
		$client		= JApplicationHelper::getClientInfo(0);

		// Try 1.6 format: /plugins/folder/element/element.xml
		$formFile = JPath::clean(JPATH_PLUGINS . '/' . $folder . '/' . $element . '/' . $element . '.xml');

		if (file_exists($formFile))
		{
			// Get the plugin form.
			if (!$form->loadFile($formFile, false, '//xtextconfig'))
			{
				throw new Exception(JText::_('JERROR_LOADFILE_FAILED'));
			}
		}

		// Attempt to load the xml file.
		if (!$xml = simplexml_load_file($formFile))
		{
			throw new Exception(JText::_('JERROR_LOADFILE_FAILED'));
		}

		if (file_exists($formFile))
		{
			// Get the plugin form.
			if (!$form->loadFile($formFile, false, '//xtruleconfig'))
			{
				throw new Exception(JText::_('JERROR_LOADFILE_FAILED'));
			}
		}

		// Attempt to load the xml file.
		if (!$xml = simplexml_load_file($formFile))
		{
			throw new Exception(JText::_('JERROR_LOADFILE_FAILED'));
		}
	}

	/**
	 * Override method to save the form data.
	 *
	 * @param   array  $data  The form data.
	 *
	 * @return	boolean	True on success.
	 *
	 * @since	1.6
	 */
	public function save($data)
	{
		$status = parent::save($data);
		if ($status)
		{
			$params = $data['ruleparams'];
			$rule_id  = $params['rule_id'];
			unset($params['rule_id']);

			$registry = new JRegistry;
			$registry->loadArray($params);
			$params = (string) $registry;

			$rule_data = array(
					'id' => $rule_id,
					'params' => $params
			);

			$config = array(
					'tablepath' => EPATH_SCHEDULER_TABLES,
					'option' => $this->option
			);

			$task = FOFTable::getAnInstance('Task', 'SchedulerTable', $config);
			$status = $task->load($rule_id);
			if (!$status)
			{
				foreach ($table->getErrors() as $error)
				{
					if (!empty($error))
					{
						$this->setError($error);
					}
				}
			}

			$status = $task->save($rule_data);

			if (!$status)
			{
				foreach ($table->getErrors() as $error)
				{
					if (!empty($error))
					{
						$this->setError($error);
					}
				}
			}
		}
		return $status;
	}

	/**
	 * Method to get a single record.
	 *
	 * @param   integer  $pk       The id of the primary key.
	 * @param   integer  $rule_id  The id of the primary key.
	 *
	 * @return	mixed	Object on success, false on failure.
	 */
	public function getItem($pk = null, $rule_id = null)
	{
		$item = parent::getItem($pk);

		if ($rule_id)
		{
			$config = array(
					'tablepath' => EPATH_SCHEDULER_TABLES,
					'option' => $this->option
			);

			$task = FOFTable::getAnInstance('Task', 'SchedulerTable', $config);
			$status = $task->load($rule_id);
			if ($status)
			{
				$params = $task->get('params');

				// Convert the params field to an array.
				$registry = new JRegistry;
				$registry->loadString($params);
				$params = $registry->toArray();
				$item->set('ruleparams', $params);
			}
			else
			{
				foreach ($table->getErrors() as $error)
				{
					if (!empty($error))
					{
						$this->setError($error);
					}
				}
			}
		}
		return $item;
	}

}
